      subroutine vvpkk(ig,u1,n1,p1,u2,n2,p2,u,n,p,g,mass,wdth)
c----------------------------------------------------------------------c
c Returns a scalar array being a contraction of a triple 
c pi0-vector meson-vector meson coupling with vectors u1 and u2
c
c Input: ig=0(fixed coupling g)/else(running coupling g),
c        u1,u2 - vector arrays,
c        n1,n2,n - numbers of polarization states,
c        p1,p2 - momenta,
c        g - a triple pi0-vector meson-vector meson coupling,
c        mass, wdth - mass and width of a scalar particle.
c Output: u - scalar array,
c         p - its momentum.
c All four momenta are cosidered as incoming to the vertex.
c
c Karol Kolodziej, University of Silesia, karol.kolodziej@us.edu.pl
c 23 June, 2014; modified on  17 July, 2014;
c 19 March, 2015 (call to subroutine runcoupl implemented)
c----------------------------------------------------------------------c

      implicit none

      integer, parameter :: dp=kind(1.d0)
      integer :: ig,n1,n2,n,i1,i2,i

      real(dp) :: mass,wdth
      real(dp), dimension(0:3) :: p1,p2,p
      complex(dp) :: u1(0:3,n1),u2(0:3,n2),u(1,n),g,gden,rg

      p=-p1-p2

      if (ig == 0) then
       rg=g
      else
       call runcoupl(ig,p,rg)
      end if

      gden=rg/cmplx(p(0)**2-p(1)**2-p(2)**2-p(3)**2-mass**2,
     &                                             mass*wdth,dp)

      i=0
      do i2=1,n2
       do i1=1,n1
        i=i+1
        u(1,i)=gden*(
     &      u1(0,i1)*(-u2(1,i2)*cmplx(p1(2)*p2(3)-p1(3)*p2(2),0.d0,dp)
     &                +u2(2,i2)*cmplx(p1(1)*p2(3)-p1(3)*p2(1),0.d0,dp)
     &                -u2(3,i2)*cmplx(p1(1)*p2(2)-p1(2)*p2(1),0.d0,dp))
     &     -u1(1,i1)*(-u2(0,i2)*cmplx(p1(2)*p2(3)-p1(3)*p2(2),0.d0,dp)
     &                +u2(2,i2)*cmplx(p1(0)*p2(3)-p1(3)*p2(0),0.d0,dp)
     &                -u2(3,i2)*cmplx(p1(0)*p2(2)-p1(2)*p2(0),0.d0,dp))
     &     -u1(2,i1)*( u2(0,i2)*cmplx(p1(1)*p2(3)-p1(3)*p2(1),0.d0,dp)
     &                -u2(1,i2)*cmplx(p1(0)*p2(3)-p1(3)*p2(0),0.d0,dp)
     &                +u2(3,i2)*cmplx(p1(0)*p2(1)-p1(1)*p2(0),0.d0,dp))
     &     -u1(3,i1)*(-u2(0,i2)*cmplx(p1(1)*p2(2)-p1(2)*p2(1),0.d0,dp)
     &                +u2(1,i2)*cmplx(p1(0)*p2(2)-p1(2)*p2(0),0.d0,dp)
     &                -u2(2,i2)*cmplx(p1(0)*p2(1)-p1(1)*p2(0),0.d0,dp)))
       end do
      end do

      end subroutine vvpkk
