      subroutine calcdis(p,cross)
c----------------------------------------------------------------------c
c Calculates distributions
c
c Input:  p - particle momenta,
c         cross - corresponding cross section.
c Select the number 'ndist' of distributions to be calculated and
c adjust the maximum number of bins 'nbmax' in 'distribs.f'
c
c Karol Kolodziej, University of Silesia, kolodzie@us.edu.pl
c 31 October, 2008, modiffied on 3 March, 2013
c----------------------------------------------------------------------c
      use inprms
      use drivekk
      use distribs
      use qrkinds
      use prtinds
      use kincuts

      implicit none

      integer :: i,j,k,imxup=0
      real(dp) :: cross,p(0:3,nepl),ptlep,ppl,pb(0:3),pl(0:3),ph(0:3),
     &            pn(0:3),pt(0:3),pw(0:3),sqs,g,v(3),q(0:3)
      real :: rand
      character(80) ind
      save imxup

      if (idisn == 1) then
       idisn=0
       nbs=0
       xmin=0.d0
       xmax=0.d0
c Distrbution 1:
       desc(1)='Q^2 of the pion system'
       nbs(1)=60
c       xmin(1)=4.d0*mpi**2
       xmin(1)=0.d0
       do i=nisp+1,nepl     ! Photon must be the last particle on the list
        xmin(1)=xmin(1)+m(i)
       end do
       xmin(1)=xmin(1)**2
       xmax(1)=ecm*(ecm-2.d0*ecuta)
c Distrbution 1:
c       desc(1)='Rapidity of the lepton'
c       nbs(1)=20
c       xmin(1)=-2.1d0
c       xmax(1)=2.1d0
c Distrbution 2:
c       desc(2)='pT of the lepton'
c       nbs(2)=40
c       xmin(2)=30.d0
c       xmax(2)=500.d0
c Distrbution 3:
c       desc(3)='Cosine of the lepton with respect to beam'
c       nbs(3)=20
c       xmin(3)=-1.d0
c       xmax(3)=1.d0
c Distrbution 4:
c       desc(4)='Energy of the lepton'
c       nbs(4)=40
c       xmin(4)=30.d0
c       xmax(4)=500.d0
c Distrbution 5:
c       desc(5)='Cosine of the angle between p_l and p_t in top rest'//
c     &         ' frame' 
c       nbs(5)=20
c       xmin(5)=-1.d0
c       xmax(5)=1.d0
c Distrbution 6:
c       desc(6)='Cosine of the angle between p_l and -p_b in top rest'//
c     &         ' frame'
c       nbs(6)=20
c       xmin(6)=-1.d0
c       xmax(6)=1.d0
c Distrbution 7:
c       desc(7)='Cosine of the angle between p_l and -p_b in W rest'//
c     &         ' frame'
c       nbs(7)=20
c       xmin(7)=-1.d0
c       xmax(7)=1.d0

       do j=1,ndist
        if (nbs(j) > 0) then
         xdi(0,j)=xmin(j)
         dx(:,j)=(xmax(j)-xmin(j))/nbs(j)
         do i=1,nbs(j)
          xdi(i,j)=xdi(i-1,j)+dx(i,j)
         end do
        end if
       end do
      end if

c      q=p(:,3)+p(:,4)
c Sum four momenta particles other than the photon:
      q=0.d0
      do i=nisp+1,nepl
       if (i /= iya(i)) then
        q=q+p(:,i)
       end if
      end do

      xd(1)=q(0)**2-q(1)**2-q(2)**2-q(3)**2

c      if (ilept > 0) then
c       xd(1)=0.5d0*log((p(0,ilept)+p(3,ilept))/(p(0,ilept)-p(3,ilept)))
c       ptlep=sqrt(p(1,ilept)**2+p(2,ilept)**2)
c       xd(2)=ptlep
c       ppl=sqrt(ptlep**2+p(3,ilept)**2)
c       xd(3)=p(3,ilept)/ppl
c       xd(4)=p(0,ilept)
c      else
c       xd(1)=xmax(1)+1.d0
c       xd(2)=xmax(2)+1.d0
c       xd(3)=xmax(3)+1.d0
c       xd(4)=xmax(4)+1.d0
c      end if

c      if (ibqrk > 0 .and. ilept > 0 .and. ineut > 0) then
c Boost to the top rest frame:
c       pb=p(:,ibqrk)
c       pl=p(:,ilept)
c       pn=p(:,ineut)
c       pt=pb+pl+pn
c       sqs=sqrt(pt(0)**2-pt(1)**2-pt(2)**2-pt(3)**2)
c       v=pt(1:)/pt(0)
c       g=pt(0)/sqs
c       call boost(g,-v,pl)
c       call boost(g,-v,pn)
c       call boost(g,-v,pb)
c       xd(5)=(pl(1)*pt(1)+pl(2)*pt(2)+pl(3)*pt(3))
c     &            /sqrt(pl(1)**2+pl(2)**2+pl(3)**2)
c     &            /sqrt(pt(1)**2+pt(2)**2+pt(3)**2)
c Reverse b momentum:
c       pb(1:)=-pb(1:)
c       xd(6)=(pl(1)*pb(1)+pl(2)*pb(2)+pl(3)*pb(3))
c     &            /sqrt(pl(1)**2+pl(2)**2+pl(3)**2)
c     &            /sqrt(pb(1)**2+pb(2)**2+pb(3)**2)
c Boost to the W rest frame:
c       pw=pl+pn
c       sqs=sqrt(pw(0)**2-pw(1)**2-pw(2)**2-pw(3)**2)
c       v=pw(1:)/pw(0)
c       g=pw(0)/sqs
c       call boost(g,-v,pl)
c       call boost(g,-v,pb)
c       xd(7)=(pl(1)*pb(1)+pl(2)*pb(2)+pl(3)*pb(3))
c     &            /sqrt(pl(1)**2+pl(2)**2+pl(3)**2)
c     &            /sqrt(pb(1)**2+pb(2)**2+pb(3)**2)
c      else
c       xd(5)=xmax(5)+1.d0
c       xd(6)=xmax(6)+1.d0
c       xd(7)=xmax(7)+1.d0
c      end if

      do j=1,ndist
       if (nbs(j) > 0) then
        if (xd(j) >= xmin(j) .and. xd(j) <= xmax(j)) then
         do i=1,nbs(j)
          if (xd(j) <= xdi(i,j)) then
           ibin(j)=i
           ncs(i,j)=ncs(i,j)+1
           if (imc > 0) then
            if (imc == j) then
c             if (cross > cdmax(i)) then
             if (cross-cdmax(i) > 1.d-14*cdmax(i)) then
              cdmax(i)=cross
              itmax(i)=iit
              imxdif=1
              imxup=1
             end if
c             call random_number(rand)
             call ranlux(rand,1)
             if (cross > rand*cdmax(i)) then
              ieff=ieff+1
              ncsa(i)=ncsa(i)+1
              if (imxup == 1) then
               close(11)
               write(ind,'(f10.1)') ecm
               open(unit=11,file='mc_events.'//trim(adjustl(ind)),
     &                            status='replace',action='write')
               imxup=0
               ieff=0
              else
               write(11,'(a,i16)') '# Event No. ',ieff
               do k=nisp+1,nepl
                write(11,'(a,i2,a,4(e16.8))') 'p(',k,')=',p(:,k)
               end do
              end if
             end if
            end if
           end if
           exit
          end if
         end do
        else
         ibin(j)=0
         nofr(j)=nofr(j)+1
        end if
       else
        exit
       end if
      end do

      end
