      program carlocom
c=============================================================================c
c Calculates the total cross of a process whose matrix element has been
c generated with a program 'carlomat'
c
c                            Karol Kolodziej
c                         University of Silesia
c                            Katowice, Poland
c                        e-mail: karol.kolodziej@us.edu.pl
c
c Major modifications: 
c 7 Aug. 2009: 
c ISR in the structure function approach implemented
c 8 June 2010:
c Printing information on cuts reorganized, a new subroutine cutinfo.f created;
c errors in the case of no call accepted (zero cross section) eliminated;
c a cut on the transverse momentum of the final state lepton added
c=============================================================================c
 
      use mathprms
      use inprms 
      use partcls
      use drivekk
      use distribs
      use kincuts
      use topan
      use vmesoncrit

      implicit none
 
      integer :: iborn,iscan0,nscan0,ie,j,ndim,ncall,neff0,
     &           i,iopkch,ncalli,niter,n,iwfrst=1,iwpsn

      real(dp) :: crosskk,cs0,cssc(nkin),sd2,po,
     &            cs0i,var0i,sd0i,sumin,dxmc,awmax,qw,
     &            cs0i2,cs02,csnum,csden,csacc,sdacc,chi2,chi2s,
     &            arg,val,sval,diter,th_min,pi3

      character(80) ind,ind1,ind2,ind3,indj,nmb,nml

      external crosskk

c Please supply the necessary information below by giving values to
c the parameters and variables

c      integer, parameter :: ne=66  ! No of the cms energies
      integer, parameter :: ne=3  ! No of the cms energies

      real(dp), dimension(ne) :: aecm=(/ ! The cms energies in GeV:
c     &        0.35d0,0.4d0,0.45d0,0.5d0,0.525d0,0.55d0,0.575d0,
c     &        0.6d0,0.625d0,0.65d0,
c     &        0.66d0,0.67d0,0.68d0,0.69d0,0.7d0,0.71d0,0.72d0,0.73d0,
c     &        0.74d0,0.745d0,0.75d0,0.752d0,0.754d0,0.756d0,0.758d0,
c     &        0.76d0,0.762d0,0.764d0,0.766d0,0.768d0,0.77d0,
c     &        0.775d0,0.776d0,0.777d0,0.778d0,0.779d0,0.7795d0,
c     &        0.78d0,0.7805d0,0.781d0,0.782d0,0.783d0,0.784d0,0.785d0,
c     &        0.786d0,0.788d0,0.79d0,
c     &        0.792d0,0.794d0,0.796d0,0.798d0,0.8d0,0.81d0,
c     &        0.82d0,0.83d0,0.84d0,0.85d0,0.86d0,0.87d0,0.88d0,0.89d0,
c     &        0.9d0,0.92d0,0.94d0,0.96d0,0.98d0/)
c     &         0.8d0,1.d0,1.5d0,2.d0/)
c     &         0.760d0,0.765d0,0.770d0,0.775d0,0.780d0,0.785d0,
c     &         0.790d0,0.795d0,0.800d0,0.805d0,0.810d0/)
     &         0.8d0,1.d0,1.5d0/)

      real(dp), dimension(ne) :: acsacc,asdacc
      integer, dimension(ne) :: acall

c Calculate the Born cross section, iborn = 1(yes)/else(no), 
c in 'niter' iterations with 'ncalli' calls per iteration, i.e. 
c with 'niter x ncalli' calls;
c po is the fraction of old integration weights which should be
c used in the next iterration, recomended value from 0.d0 to 0.5d0

      iborn=1
      niter=10
      ncalli=200000
      po=0.d0

c Generate the unweighted events: imc < 0(no)/else(yes)?
c If imc = 0 => use the maximum of the total cross section
c If imc = 1,2,... => use the maxima of the differential cross section No. imc.
c          See that idis=1 below, distribution imc is defined in calcdis.f
c          and the corresponding test results in the total cross section

      imc=-1

c Scan the Born cross section with 'nscan0' calls: iscan0 = 1(yes)/else(no)?

      iscan0=1
      nscan0=200

c Calculate the pio form factor: ipiff=1(yes)/else(calculate the cross section)?
c Makes sens in principle only for e+ e- -> pi+ pi-.

      ipiff=0

c The following options are relevant only if the corresponding mixing is included 
c at the code generation stage:
c gamma-rho mixing: iarho=1(yes)/else(no)?
      iarho=1
c-with imrho=1(the weight wmrho > 0.d0, the phase dmrho in deg,
c s-depnedent)/else(plain cupling aome, as defined in couplsm.f):
      imrho=0
      wmrho=1.0d0
      dmrho=0.d0  ! in deg
c gamma-omega mixing: iaome=1(yes)/else(no)?
      iaome=1
c-with imome=1(the weight wmome > 0, the phase dmome in deg,
c s-depnedent)/else(plain cupling aome, as defined in couplsm.f):
      imome=0
      wmome=0.375d0
      dmome=105.0d0  ! in deg
c gamma-phi mixing: iaphi=1(yes)/else(no)?
      iaphi=1
c-with imphi=1(the weight wmphi > 0, the phase dmphi in deg,
c s-depnedent)/else(plain cupling aphi, as defined in couplsm.f):
      imphi=0
      wmphi=0.375d0
      dmphi=105.0d0  ! in deg
c gamma-rho1 mixing: iarho1=1(yes)/else(no)?
      iarho1=1
c-with imrho=1(the weight wmrho > 0.d0, the phase dmrho in deg,
c s-depnedent)/else(plain coupling aome, as defined in couplsm.f):
      imrh1=0
      wmrh1=1.d0
      dmrh1=0.d0  ! in deg
c gamma-rho2 mixing: iarho2=1(yes)/else(no)?
      iarho2=1
c-with imrho=1(the weight wmrho > 0.d0, the phase dmrho in deg,
c s-depnedent)/else(plain cupling aome, as defined in couplsm.f):
      imrh2=0
      wmrh2=1.d0
      dmrh2=0.d0  ! in deg

c Use the running width prescription for a vector meson: 
c 0(fixed width)/1,2 or 3(running width), see runwidth.f in carlolib for details

      igmrh=0    ! for rho0
      igmom=0    ! for omega
      igmr1=0    ! for rho1
      igmr2=0    ! for rho2

c Include the initial state radiation 
c  iisr = 1 (yes, inclusive photon in a full angular range)/
c  iisr = 2 (yes, inclusive photon in the angular range 
c                                               th_min < th < 180 - th_min)/
c  iisr = else (no inclusive photon radiation included)

      iisr=0
c If iisr=1 then choose an option chce for including non-leading term, 
c see gllee.f for details and epsisr, see crosskk.f for details. 
c The splitting scale is set to qsc2=ecm**2 below.
c Both initial state particles must have integer electric charge, must be
c light, but must have nonzero masses.
      epsisr=1.d-10
      chce=3
c If iisr=2 then choose the photon angular cut th_min in degs and minimal photon
c energy ecuta in GeV.
      th_min=5.d0
      ecuta=0.01d0

c Check U(1) gauge invariance for an external photon,
c igauge=the photon No., counted from left to right,(yes)/else(no)

      igauge=0

c Should distributions be calculated, idis=1(yes)/0(no)? If yes, then specify
c the number of distributions to be calculated in distribs.f and define their
c parameters in calcdis.f

      idis=1

c Impose cuts, icuts=1(yes)/0(no)

      icuts=1

c If icuts=1 then select the cuts to be imposed

      if (icuts == 1) then
c on
c  transverse momentum of the final state lepton (GeV):
      iptl=0
      ptl=30.d0
c  transverse momentum of the final state jet: light quark or gluon (GeV):
      iptj=0
      ptj=30.d0
c  transverse momentum of the final state b-jet (GeV):
      iptb=0
      ptb=30.d0
c  pseudorapidity of the final state lepton:
      ietal=0
      etal=2.1d0
c  pseudorapidity of the final state jet: light quark or gluon:
      ietaj=0
      etaj=2.4d0
c  pseudorapidity of the final state b-jet:
      ietab=0
      etab=2.4d0
c separation in the azimuthal angle-pseudorapidity plane of two leptons,
c two jets and a lepton and a jet:
      idelr=0
      delr=0.4d0
c  missing transverse energy (GeV):
      iemt=0
      ecutt=20.d0
c  rapidity of the final state lepton:
      irapl=0
      rapl=2.d0
c  rapidity of the final state jet: light quark or gluon:
      irapj=0
      rapj=2.5d0
c  rapidity of the final state b-jet:
      irapb=0
      rapb=2.5d0
c  cos theta(lepton,beam) - ictlb=1/0:
      ictlb=0
c      ctlb=0.985d0   ! 
      ctlb=cos(17.d0/180.d0*pi)
c  cos theta(quark,beam), including b - ictqb=1/0:
      ictqb=0
      ctqb=0.985d0   ! 
c      ctqb=cos(5.d0/180.d0*pi)
c  cos theta(gluon,beam) - ictgb=1/0:
      ictgb=0
c      ctgb=0.985d0   ! 
      ctgb=ctqb
c  cos theta(photon,beam) - ictab=1/0:
      ictab=1
c      ctab=0.985d0
      ctab=cos(th_min/180.d0*pi)
c  cos theta(proton,beam) - ictpb=1/0:
      ictpb=0
c      ctpb=0.985d0
      ctpb=cos(30.d0/180.d0*pi)
c  cos theta(scalar,beam) - ictsb=1/0:
      ictsb=0
c      ctsb=0.985d0
      ctsb=cos(5.d0/180.d0*pi)
c  cos theta(photon,lepton) - ictal=1/0:
      ictal=0
      ctal=cos(5.d0/180.d0*pi)
c  cos theta(photon,quark), including b - ictaq=1/0:
      ictaq=0
      ctaq=cos(5.d0/180.d0*pi)
c  cos theta(photon,scalar) - ictas=1/0:
      ictas=0
      ctas=cos(5.d0/180.d0*pi)
c  cos theta(quark,quark'), including b - ictqq=1/0:
      ictqq=0
c      ctqq=0.985d0   ! 
      ctqq=cos(10.d0/180.d0*pi)
c  cos theta(lepton,light quark) - ictlq=1/0:
      ictlq=0
c      ctlq=0.985d0   ! 
      ctlq=cos(5.d0/180.d0*pi)
c  cos theta(lepton,light quark) - ictlq=1/0:
      ictlbq=0
c      ctlbq=0.985d0   ! 
      ctlbq=cos(5.d0/180.d0*pi)
c  cos theta(lepton,gluon) - ictlg=1/0:
      ictlg=0
c      ctlg=0.985d0   ! 
      ctlg=ctlq
c  cos theta(lepton,lepton') - ictll=1/0:
      ictll=0
c      ctll=0.985d0   ! 
      ctll=cos(5.d0/180.d0*pi)
c  lepton energy (GeV):
      ienl=0
      ecutl=5.d0
c  light quark energy (GeV):
      ienq=0
      ecutq=10.d0
c  b-quark energy (GeV):
      ienb=0
      ecutb=15.d0
c  gluon energy (GeV):
      ieng=0
      ecutg=1.d0 !ecutq
c  photon energy (GeV):
      iena=1
      ecuta=0.01d0
c  quark-quark' invariant mass, including b (GeV):
      imqq=0
      mqq=10.d0
c  jet-jet' invariant mass, including b (GeV):
      imjj=0
      mjj=10.d0
c  lepton-lepton' invariant mass (GeV):
      imll=0
      mll=10.d0
c  b-b' invarint mass (GeV):
      imbb=0
      mbb=20.d0
c  p-p~ invarint mass (GeV):
      impp=0
      mpp1=4.d0*mp**2
      mpp2=10.d0 ! GeV^2
c  angular cut for t-channel processes, cos(p_i,p_f)
      ctc=ctlb
c invariant mass cut, used in mappings of the photon and gluon propagators 
c that couple to massless fermions relevant if light fermion masses are zero
      sqscut=1.d-8

c cuts on invariant masses for the associated ttH production,
c imtth=1(yes)/0(no)
      imtth=0
      if (imtth == 1) then
c departure of the two b-jets invariant mass from the higgs mass (GeV):
       imbbh=1
       mbbh=10.d0
c limits on invariant mass of two hadronic non-b-jets (GeV):
       imnb=1
       mqq1=60.d0
       mqq2=90.d0
c departure of the invariant mass of one b-jet and the two non-b-jets 
c from the top mass (GeV):
       imbqq=1      ! imnb must be 1
       mbqq=30.d0
c limits on transverse mass of the lepton-neutrino system (GeV):
       imln=1
       mln1=0.d0
       mln2=90.d0
c limits on the transverse mass a b-quar, lepton and missing transverse energy
c system, see K. Kolodziej, S, Szczypinski, NPB 801 (2008) 153
       imbln=1      ! imln must be 1
       mbln1=30.d0
       mbln2=10.d0
      end if

      end if ! icuts

c Choose the scheme:
c     ischeme = 1(complex mass scheme)/else(fixed width scheme).

      ischeme=0

c Should couplings be parametrized in terms of alpha_W or alpha_0?
c     ialpha = 0 (alpha_0)/1 (alpha_W)/ 2 (alphao - whose inverse must be
c                                          specified in inprms.f)

      ialpha=0

c Choose print option: iprint = 1 (print info relevant for low energy processes)/
c                               2 (print info relevant for high energy processes)

      iprint=1

c Optimize the No of kinematical channels, iopkch=1(yes)/else(no)

      iopkch=0

c Discard kinematical channels which contribute less than qw of the dominant 
c channel in the initial scan

      qw=0.d0

c Discard kinematical channels where the inacurate phase space normalization
c is detected during the scan, iwpsn=1(yes)/else(no)

      iwpsn=1

c----------------------------------------------------------------------c
c Options relevant only for high energy or hadron scattering processes:
c----------------------------------------------------------------------c

c Treat the initial state particles as partons of the hadron-hadron scattering:
c ihad=0(no)/1(yes p p~)/2(yes p p)?

      ihad=0  

c If ihad=1,2 then choose parton distribution functions to be used.
c ipdf=1(MSTW)/2(CTEQ6L)

      ipdf=1      ! MSTW
      iset=0      ! the central PDF set
c If ipdf=1, then speocify prefix for the grid files:
      prefix='../../mstw/Grids/mstw2008lo'     ! LO
c      prefix='../../mstw/Grids/mstw2008nlo'    ! NLO
c      prefix='../../mstw/Grids/mstw2008nnlo'   ! NNLO


c      ipdf=2    ! CTEQ6L
c      iset=3    ! Leading Order, Alpha_s(Mz)=0.118

c Choose factorization scale (= renormalization scale) and scale factor, 
c iscl=1(sclf*sqrt(s'))/2(fixed value = sclf)/3(sclf*sqrt(mt^2+sum of ptj^2)). 
c Go to crosskk.f to define other scales.

      iscl=3 
      sclf=1.0
c      sclf=2*mt ! e.g. if iscl=2

c Include the anomalous Wtb coupling, see K. Kolodziej, Phys. Lett. B584 
c (2004) 89, hep-ph/0312168,
c     iwtb=1 (Yes, CP even)/2 (Yes, CP odd)/c /else (No)? 
c Take into account that the matrix element must be generated by caromat.f 
c with iwtb=1.

      iwtb=0
c t -> b w+, Eq.(4):
      f1p=0.d0
      f1m=1.0d0
      f2p=0.0d0
      f2m=0.0d0
      if (iwtb == 2) then
c t~ -> b~ w-, Eq.(5), conditions of Eq.(6) need not be satisfied:
       f1bp=0.d0
       f1bm=1.d0
       f2bp=0.0d0
       f2bm=0.0d0
      end if

c Include the anomalous Htt coupling, see J.A. Aguilar-Saavedra, Nucl. Phys.
c B821 (2009) 215,  arXiv:0904.2387,   ihtta=1 (Yes)/else (No)? 

      ihtt=0       ! dimesionless departure from htt coupling, see couplsm.f,
      httv=0.d0    ! scalar part
      htta=0.d0   ! pseudo scalar part
                   ! SM corresponds to httv=1, htta=0
c----------------------------------------------------------------------c
c End of input
c----------------------------------------------------------------------c

c Determine parameters of the proces:

      call parfixkk

c Modify the short name of the proces:

      if (imtth == 1) then
       write(ind,'(f4.0)') mbbh
       prcsnm=trim(adjustl(prcsnm))//'_mbbh'//trim(adjustl(ind))
      end if
      if (ihtt == 1) then
       write(ind,'(f4.1)') httv
       write(ind1,'(f4.1)') htta
       prcsnm=trim(adjustl(prcsnm))//'_htt'//trim(adjustl(ind))
     &        //'_'//trim(adjustl(ind1))
      end if
      if (inwtb == 1) then
       write(ind,'(f4.1)') f2p
       write(ind1,'(f4.1)') f2m
       if (iwtb == 2) then
        write(ind2,'(f4.1)') f2bp
        write(ind3,'(f4.1)') f2bm
       else
        write(ind2,'(f4.1)') f2m
        write(ind3,'(f4.1)') f2p
       end if
       prcsnm=trim(adjustl(prcsnm))//'_'//trim(adjustl(ind))//'_'
     &          //trim(adjustl(ind1))//'_'//trim(adjustl(ind2))//
     &          '_'//trim(adjustl(ind3))
      end if

      if (nisr == 1) then
       prcsnm=trim(prcsnm)//'_ISR'
      else if (nisr == 2) then
       prcsnm=trim(prcsnm)//'_FSR'
      else if (iisr == 1) then
       prcsnm=trim(prcsnm)//'_isr1'
      else if (iisr == 2) then
       prcsnm=trim(prcsnm)//'_isr2'
      end if

      if (igauge > 0) then                            ! added on 13 June, 2016
       write(ind,'(i16)') igauge
       prcsnm=trim(prcsnm)//'_ginv'//trim(adjustl(ind))
      end if

      open(unit=6,file='tot_'//trim(adjustl(prcsnm)),
     &                        status='replace',action='write')

      call logo(6)

      mthrs=zero
      sumin=zero

      do j=nisp+1,nepl
       mthrs=mthrs+m(j)
      end do
      do j=1,nisp
       sumin=sumin+m(j)
      end do

      if (icuts == 1) then
       write(6,4000)
       call cutinfo(6)
       if (iisr == 2) then
        write(6,'(a)') '# No cuts other than the minimum photon '//
     &   'angle and energy are imposed'
       end if
      end if ! icuts

      if (iisr == 1) then
       write(6,4000)
       write(6,'(a,i2)') '# The ISR included in a full photon angular '
     &                 //'range with chce =',chce
      else if (iisr == 2) then
       write(6,4000)
       write(6,'(a,f8.2,a,f8.2)') '# The ISR included with a photon '
     &  //'in the range:',th_min,' < Theta_ab <',180.d0-th_min
       write(6,'(a,f10.4,a)') 
     &                '# and energy:    E_a >',ecuta,' GeV'
      end if

c The energy loop:

      do ie=1,ne

      ecm=aecm(ie)     !  The c.m.s. energy
      ecm2=ecm**2

      if (iisr == 1) then
       qsc2=ecm2
      else if (iisr == 2) then
       cth_min=cos(th_min/180.d0*pi)
       q2_min=0.d0
       do i=nisp+1,nepl
        q2_min=q2_min+m(i)
       end do
       q2_min=q2_min**2
       q2_max=ecm*(ecm-2.d0*ecuta)
       fct_pt=4.d0*pi/ialph0**2/3.d0
       sig_fct=4.d0/ialph0**3/(3.d0*ecm2)
       lctmin=log((1.d0+cth_min)/(1.d0-cth_min))
      end if

      idisn=1 ! Characteristics of the distributions should be calculated anew

      if (ecm < sumin) then 
       write(6,*)'carlocom.f: The CMS energy below the mass of initial',
     &           ' state particles'
       write(6,*) sumin,' GeV'
       exit
      end if
      if (ecm < mthrs) then 
       write(6,*)'carlocom.f: The CMS energy below the threshold',mthrs,
     &           ' GeV'
       exit
      end if

      write(6,4000)
 4000 format('#-------------------------------------------------------',
     &       '----------------------#')
      write(6,1006) ecm
      write(6,4000)
 1006 format('#                       E_cm = ',f11.3,' GeV')

      if (iborn == 1) then
       if (iisr == 1 .or. iisr == 2) then
        write(6,'(a,i2,a)') '# The Born + ISR cross section with full ',
     &    nepl-nisp,' particle kinematics:'
       else
        write(6,'(a,i2,a)') '# The Born cross section with full ',
     &    nepl-nisp,' particle kinematics:'
       end if

c Initialize integation weights

ckk>
c      aw0=0.d0
c      aw0(19:22)=1.d0/4.d0
c      aw0(41:48)=1.d0/8.d0
ckk<
       aw=aw0
       aw_w=0.d0
       iwkin=0

       if (iisr == 1 .or. ihad > 0) then
        ndim=3*(nepl-nisp)-2
       else if (iisr == 2) then
        ndim=3*(nepl-nisp)-3
       else
        ndim=3*(nepl-nisp)-4
       end if

c Optimize the No of kinematical channels

       if (iopkch == 1) then
        call chanopt(ndim)
       end if

       if (imc >= 0) then
c Initialize maximum value of the cross section
        write(ind,'(f10.1)') ecm
        open(unit=11,file='mc_events.'//trim(adjustl(ind)),
     &                           status='replace',action='write')
        ieff=0
        imxtot=0
        imxdif=0
        itmax=0

        if (imc == 0) then
         crmax=0.d0
         open(unit=12,file='csmax_tot.'//trim(adjustl(ind)),
     &                           status='old',action='read',err=201)
c         read(12,'(e22.15)') crmax
         read(12,*) crmax
 201     close(12)
        else if (imc > 0) then
         cdmax=0.d0
         write(ind1,'(i16)') imc
         open(unit=12,file='csmax_'//trim(adjustl(ind1))//'dif.'//
     &       trim(adjustl(ind)),status='old',action='read',err=202)
         do i=1,nbmax
c          read(12,'(e22.15)') cdmax(i)
          read(12,*) cdmax(i)
         end do
 202     close(12)
        end if
       end if

       iscan=iscan0

       if (iscan == 1) then

c Scan the Born cross section in order to find the dominant channels:

        ncall=nscan0
        cs0=zero
        icall=0
        cssc=zero

        write(6,5110)
 5110   format('# Results of the initial scan:')
        write(6,5120)
 5120   format('#     Kinematics       Weight         Integral      ',
     &       'St.dev.       No of calls')

        sd2=zero

        do ikin=1,nkin
         if (aw(ikin) > 0.d0) then
          neff0=icall
          call carlos(crosskk,ndim,nkin,xmc,po,ncall,cs0i,var0i,sd0i)
          write(6,3002) ikin,aw(ikin),aw(ikin)*cs0i,aw(ikin)*sd0i,
     &                                                icall-neff0
 3002     format('#  ',i10,'     ',e15.6,e15.6,e12.3,i10)
          cs0=cs0+aw(ikin)*cs0i
          cssc(ikin)=aw(ikin)*cs0i
          sd2=sd2+(aw(ikin)*sd0i)**2
         end if
        end do

        if (iwpsn == 1) then        ! 15 July, 2016
         do i=1,nkin
          if (iwkin(i) > 0) then
           cs0=cs0-cssc(i)
           cssc(i)=0.d0
           aw(i)=0.d0
c           iwkin(i)=0
           write(6,'(a,i7,a)')'# Contribution of kinematics',i,
     &        ' cancelled'
          end if
         end do
        end if

        write(6,4000)
        if (iisr == 1 .or. iisr == 2) then
         write(6,3798) icall,cs0,sqrt(sd2)
        else
         write(6,3800) icall,cs0,sqrt(sd2)
        end if
 3798   format('# Born + ISR scanned with',i8,' calls = ',e15.8,'+/-',
     &                                                           e10.3)
 3800   format('# Born scanned with',i8,' calls = ',e15.8,'+/-',e10.3)

        if (cs0 == 0.d0) exit

        if (imc == 0) then
         write(6,3810) crmax
        end if
 3810   format('# The maximum value of the cross section= ',e15.8)
c         crmax=1.5d0*crmax
c         write(6,3820) crmax
c 3820 format('# shifted to ',e15.8)
        write(6,4000)

c Initialize integration weights:

        neff0=0
        xmc(0)=zero
        dxmc=zero
        awmax=zero
        do ikin=1,nkin
         if (aw(ikin) > zero) then
          aw(ikin)=cssc(ikin)/cs0
          if (aw(ikin) > awmax) awmax=aw(ikin)
         end if
        end do
        do ikin=1,nkin
         if (aw(ikin) > zero) then
          if (aw(ikin) < qw*awmax) then
           dxmc=dxmc+aw(ikin)
           aw(ikin)=zero
          end if
         end if
         xmc(ikin)=xmc(ikin-1)+aw(ikin)
        end do

        xmc=xmc/(1.d0-dxmc)

        iscan=0

        if (qw > 0.d0) then
         write(6,'(a,e9.2,a)') '# Kinematical channels contributing '
     &    //'less than',qw,' of the dominant channel'
         write(6,'(a)') '# discarded.'
        end if
       else ! iscan
c Initialize integration weights:
        neff0=0
        xmc(0)=zero
        do ikin=1,nkin
         xmc(ikin)=xmc(ikin-1)+aw(ikin)
        end do
       end if ! iscan

c The Born cross section with standard deviation:

c         ncall=ncall0
       ncall=niter*ncalli

       cs0=0.d0
       sd2=0
       icall=0

       if (idis == 1) then
        ncs=0
        nofr=0
        di=zero
        vdi=0.d0
        ncsa=0
       end if

       if (iisr == 1 .or. iisr == 2) then
        write(6,3202)
       else
        write(6,3200)
       end if
 3200 format('# Calculating the Born cross section with the weights:')
 3202 format('# Calculating the Born + ISR cross section with the',
     &       ' weights:')

       csnum=0.d0
       csden=0.d0
       cs02=0.d0
       chi2s=0.d0

       do iit=1,niter
        write(6,4000)
        write(6,'(a,i3,a)') '# Iteration',iit,' with weights:'
        do ikin=1,nkin
         if (aw(ikin) > 0.d0) then
          write(6,'(a,i10, 2e16.8)') '#',ikin,aw(ikin),xmc(ikin)
         end if
        end do
        call carlos(crosskk,ndim,nkin,xmc,po,ncalli,cs0i,var0i,sd0i)
        cs0i2=cs0i*cs0i
        cs02=cs02+cs0i2
        if (cs0i > 0.d0) then
         csnum=csnum+cs0i*cs0i2/var0i
         csden=csden+cs0i2/var0i
         cs0=cs0+cs0i/niter
         chi2s=chi2s+cs0i2*(1.d0+1.d0/var0i)
        end if
        if (niter > 1) then
         sd2=sd2+(sd0i)**2/(niter*(niter-1))
        else
         sd2=(sd0i)**2
        end if

        if (csnum > 0.d0) then
         csacc=csnum/csden
         sdacc=csden*iit/cs02
         chi2=sdacc*(chi2s/csden-csacc*csacc)/(iit-0.999d0)
         sdacc=sqrt(1.d0/abs(sdacc))
        else
         csacc=0.d0
         sdacc=0.d0
         chi2=0.d0
         sdacc=0.d0
        end if

        write(6,4000)
        write(6,'(a,i3,a)') '# Iteration No. ',iit,':'
        write(6,2987) cs0i,sd0i,icall-neff0
        write(6,'(a,e13.6,a,e9.2,a,i11,a)') '# Accumulated result: ',
     &     csacc,' +/- ',sdacc,' with ',icall,' calls'
        if (iit == niter) then
         acsacc(ie)=csacc
         asdacc(ie)=sdacc
         acall(ie)=icall
c        write(10,'(f11.3,e15.6,e11.2,i13,e15.6,e11.2)') aecm(ie),
c     &        csacc,sdacc,icall,cs0,sqrt(sd2)
        end if
c       write(6,'(a,e9.2)') '# Chi^2 per iterration: ',chi2
        if (imc == 0) then
         write(6,3810) crmax
        end if
        neff0=icall
        do ikin=1,nkin
         aw(ikin)=xmc(ikin)-xmc(ikin-1)
        end do
       end do

       write(6,4000)
       write(6,'(a)') '# Arithmetic mean result:'

       write(6,2987) cs0,sqrt(sd2),neff0

 2987  format('# Integral =',e13.6,' +/- ',e9.2,' with ',i9,' calls')

c       write(6,4000)
c       write(6,'(a)') '# Analytic result for e+ e- -> pi+ pi-:'
c       write(6,'(a,e13.6)') '# cs_0 =',convc*pi/(3.d0*ialph0**2*ecm2)
c     &                 *(sqrt(1.d0-(2.d0*mpi/ecm)**2))**3


c       write(6,4000)
c       write(6,3030) dble(ieff)/dble(icall)
c 3030  format('# Efficiency of the event acceptance =',e10.3)

       if (idis == 1) then
        do j=1,ndist
         write(indj,'(i4)') j
         nmb='db'//trim(adjustl(indj))//'_'//trim(adjustl(prcsnm))
         nml='dl'//trim(adjustl(indj))//'_'//trim(adjustl(prcsnm))
ckk>
c         if (ne > 1) then
c          write(indj,'(i6)') ie
c          nmb=trim(nmb)//'_'//trim(adjustl(indj))
c          nml=trim(nml)//'_'//trim(adjustl(indj))
c         end if
         write(indj,'(f10.2)') aecm(ie)
         nmb=trim(nmb)//'_'//trim(adjustl(indj))
         nml=trim(nml)//'_'//trim(adjustl(indj))
ckk<
         open(unit=11,file=trim(nmb),status='replace',action='write')
         open(unit=9,file=trim(nml),status='replace',action='write')
         if (idis == 1) then
          call logo(11)
          call logo(9)
          if (icuts == 1) then
           write(11,4000)
           call cutinfo(11)
           write(11,4000)
           write(9,4000)
           call cutinfo(9)
           write(9,4000)
          end if
         else
         end if
         if (nbs(j) > 0) then
c          write(11,4000)
          write(11,3000) j,sum(ncs(:,j))
          write(9,3000) j,sum(ncs(:,j))
 3000     format('# Distribution No. ',i3,' calculated with',
     &              i10,' calls')
          write(11,'(a)')'# '//trim(desc(j))
          write(11,4000)
          write(9,'(a)')'# '//trim(desc(j))
          write(9,4000)

          do n=1,nbs(j)
           if (ncs(n,j) > 0) then
            if (dx(n,j) > zero) then
             val=di(n,j)/(dble(niter)*dx(n,j))
            else
             val=zero
            end if
            vdi(n,j)=ncs(n,j)*vdi(n,j)-di(n,j)*di(n,j)
            sdi(n,j)=sqrt(abs(vdi(n,j)/ncs(n,j)))
            if (dx(n,j) > zero) then
             sval=sdi(n,j)/dx(n,j)
             if (niter > 1) then
              diter=dble(niter*(niter-1))
              sval=sval/sqrt(diter)
             end if
            else
             sval=zero
            end if
           else
            val=zero
            vdi(n,j)=zero
            sdi(n,j)=zero
           end if
           arg=(xdi(n-1,j)+xdi(n,j))/2.d0
           write(11,'(3e11.4)') arg,val/csacc,val!,sval
           write(9,'(3e11.4)') xdi(n-1,j),val/csacc,val!,sval
           write(9,'(3e11.4)') xdi(n,j),val/csacc,val!,sval
          end do
          if (niter > 1) then
           diter=dble(niter*(niter-1))
          else
           diter=1
          end if
          write(11,'(a,e15.8,a,e15.8)') '# Integrated distribution = ',
     &    sum(di(:,j))/dble(niter),' +/-',
     &    sqrt(sum(sdi(:,j)**2)/diter)
          write(11,'(a,e15.8,a,e15.8,a,i10,a)') 
     &   '# Total cross section = ',cs0,' +/-',sqrt(sd2),' with',
     &    neff0,' calls'
          close(11)
          write(9,'(a,e15.8,a,e15.8)') '# Integrated distribution = ',
     &    sum(di(:,j))/dble(niter),' +/-',
     &    sqrt(sum(sdi(:,j)**2)/diter)
          write(9,'(a,e15.8,a,e15.8,a,i10,a)') 
     &   '# Total cross section = ',cs0,' +/-',sqrt(sd2),' with',
     &    neff0,' calls'
          close(9)
         else
          exit
         end if
        end do
        write(6,4000)
        write(6,'(a)')'# Test of the distributions:'

        write(6,'(a,e15.8,a,i10,a)') '# Total cross section of ',cs0,
     &    ' with',neff0,' calls'

        write(6,'(a)') '# Integrated distributions:'

        do j=1,ndist
         if (nbs(j) > 0) then
          write(6,'(a,i3,a,e15.8,a,e15.8,a,i10,a)') '#     No.',j,
     &                ' : ',
     &    sum(di(:,j))/dble(niter),'+/-',
     &                   sqrt(sum(sdi(:,j)**2)/diter),
c/(nbs(j)*(nbs(j)-1)))),
     &    ' with',sum(ncs(:,j)),' calls'
         else
          exit
         end if
        end do
       else
        open(unit=11,file='db0'//'_readme',
     &                        status='replace',action='write')
        open(unit=9,file='dl0'//'_readme',
     &                        status='replace',action='write')
        write(11,'(a)') '# Set idis=1 and modify calcdis.f and '//
     &     'distribs.f appropriately to calculate' 
        write(11,'(a)') '# distributions'
        write(9,'(a)') '# Set idis=1 and modify calcdis.f and '//
     &     'distribs.f appropriately to calculate' 
        write(9,'(a)') '# distributions'
        close(9)
        close(11)
       end if

      end if ! (iborn == 1)

      if (imc >= 0) then
       close(11)
       write(ind1,'(i16)') ieff
       write(ind,'(f10.1)') ecm
       write(6,4000)
       write(6,'(a)')'# '//trim(adjustl(ind1))//' MC events stored in '
     &     //'file '//'mc_events.'//trim(adjustl(ind))
       write(6,4000)

       if (imc == 0) then
        if (imxtot == 1) then
         open(unit=12,file='csmax_tot.'//trim(adjustl(ind)),
     &                           status='replace',action='write')
c         write(12,'(e22.15)') crmax
         write(12,*) crmax
         close(12)
        end if
       else if (imc > 0) then
        if (sum(ncs(:,imc)) /= neff0) then
         write(6,'(a,i3,a)') '# Distribution No. ',imc,' should not '//
     &    'have been used in the MC event generation.'
         write(6,'(a)') '# Generate MC events according to another '//
     &    'distribution, or change the limits'
         write(6,'(a)') '# within which the distribution is defined.'
         write(6,4000)
        end if
        if (imxdif == 1) then
         write(ind1,'(i16)') imc
         open(unit=12,file='csmax_'//trim(adjustl(ind1))//'dif.'//
     &        trim(adjustl(ind)),status='replace',action='write')
         do i=1,nbmax
c          write(12,'(e22.15)') cdmax(i)
          write(12,*) cdmax(i)
         end do
         close(12)
        end if
       end if
      end if

c      if (imc > 0) then
c       csmax=0.d0
c       do i=1,nbmax
c        if (ncs(i,imc) > 0) then
cc         csmax=csmax+cdmax(i)*dx(i,imc)*dble(ncsa(i))/dble(ncs(i,imc))
c         csmax=csmax+cdmax(i)*dble(ncsa(i))/dble(ncs(i,imc))
c        end if
c       end do
c       write(6,*)'Test:',csmax/nbs(imc)
c      else if (imc == 0) then
c       write(6,*)'Test:',crmax*ieff/neff0
c      end if

      end do                    ! ie

c      close(10)

      if (ipiff == 1) then
       write(6,'(a)') '#   E_cm     |F_pi|^2  +/-   sdev     with  '
     &              //'calls'
      else
       write(6,'(a)') '#   E_cm         cs    +/-   sdev     with  '
     &              //'calls'//'     pi3'
      end if

      do ie=1,ne
       write(6,'(e11.4,e13.6,e13.6,i12,e13.6)') 1.d3*aecm(ie),
     &                         acsacc(ie),asdacc(ie),acall(ie)
     &                                   ,pi3(aecm(ie))
      end do

      close(6)

      do i=1,nkin           !  14 July, 2016
       if (iwkin(i) > 0) then
        if (iwfrst == 1) then
         write(*,'(a)')'Inacurate phase space normalization detected'//
     &    ' in kinematical channels:'
         iwfrst=0
        end if
        write(*,'(i7,a,i7,a,e10.3,e10.3,e10.3)') i,':',iwkin(i),
     &  ' times, max discrepancy',aw_w(i,2),aw_w(i,3),aw_w(i,1)
       end if
      end do

      end
