      subroutine checktop(v1,d1,v2,d2,ipl,ncls,id,idr,nkm,accept)
c----------------------------------------------------------------------c
c Checks if a topology exists in the model
c
c Karol Kolodziej, University of Silesia, karol.kolodziej@us.edu.pl
c 14 June, 2006; 
c modified 30 June 2006, June 2007, 29 July, 2007, 13 Sept. 2007, 
c 24 and 27 Oct. 2007, 27 Jan. 2008; 20 March, 2008;
c 1 April, 2008: 
c mappings of massless propagators improved
c 15 April, 2008:
c wrong boundry of 'sqsa' corrected
c 10 July, 2008 (see below) 
c 20 July, 2008 (see below) 
c 24 May, 2009: 
c kinematics of a 2 -> 2 process implemented
c 15 Feb., 2010: 
c redundant local variables in wrkinrtn commented or removed
c 25 Feb., 2010:
c writing of kinematical routines modified: parts common for graphs of 
c the same topology are not repeated any more
c 18 March, 2010: 
c writing of ntnkin.f changed
c 19 March, 2010: 
c modifications of 25 February supplemented with the case of 2 -> 2
c reactions
c 13/14 May, 2010:
c writing to unit 271 in the case of 2 -> 2 reactions corrected,
c bug in writing initialization expressions for sqsl_i and q_i in
c some kinematical routines kinkk_i.f introduced in Version 2.2 
c corrected
c 14 May, 2010:
c writing of the redundant ending of subroutine psub_i.f eliminated
c 26 May, 2010:
c array references out of bounds for array 'opl' removed
c 10 June 2010:
c a bug corrected which caused wrong upper boundry of one invariant 
c in the phase space parametrization for some topologies
c 27 Sept. 2010:
c redundunt argument list of kinwrt cancelled
c 30 Sept. - 5 Oct. 2010:
c generation of repetitive kinematical routines eliminated
c 23 Nov. 2010:
c bugs in writing kincls.f and psub1.f corrected
c 17 Jan. 2011:
c a bug in writing kinematical routines for 2 -> 2 processes corrected
c 3 April 2014:
c particle mixing added
c 12 May 2014:
c mixing implementation corrected
c 15 May 2014:
c kinematic generation routine 'wrtss' modified
c 3 June 2014:
c writing of information on Feynman diagrams with mixing improved
c 9 June 2014:
c mixing implementation corrected
c 23 July, 2014:
c refernces to zero sized strings clfd and clx eliminated
c 1 Oct. 2014:
c internal subroutine 'clean' corrected
c 26 March, 2015:
c printing of the control info changed
c 14 July, 2016:
c Writing of subroutines psub*.f changed
c----------------------------------------------------------------------c

      use intpart
      use declends
      use colinfo

      implicit none

      integer, parameter :: nisp=2
      integer :: d1,d2,v1(d1,3),v2(d2,3),ipl,ncls,id,idr,nkm,id0,id1,
     &           i1,i2,i3,i,j,k,ipk(mxvpl),iord,iorx,lclfc,
     &           ipl0,abi1,abi2,abi3,lu,blad,dpl,e1,e2,iplin,
     &           ntop=0,ntop0,ii,sti,stj,plord(nexpl),
     &           ist,iso,sgn,istr,istr0,lstr0,icxa,icxb,icmm,
     &           ixx,iyy,iww,ivv,izz,icon0=0,iconx=0,!icony0=0,icony=0,
     &           iss,iss1,isn,isn0,isn1,isn2,isn3,
     &           opli,oplj,imap(nexpl-4),nkm0,nkmt=0,idobl=0,nkm1,
     &           nuga(nexpl-4),icnclx=0,icncl0=0,k1,k2,k3,oplm,
     &           iplx=0,iply=0


      integer, allocatable :: w1(:,:),w2(:,:),x1(:,:),x2(:,:)

      character(80) ind0,ind1,ind2,ind3,ind4,inds,indx0,ind5,indi,indj
      character(160) clfc,clfcx,clfcy,clkin,ind160
      character(3), dimension(nexpl-3) :: cst,cso
      character(80), dimension(nexpl-4) :: sqsl,sqsla,sqslb,
     &                                     sqsa,sqsb,sqsc,
     &                                     moml,momla,momlb,dlip,
     &                                     mapm,mapw
      character(80), dimension(nexpl-3) :: sqsd,sqse,momd,mome
      character(80) :: cntn,oplc,opld

      logical :: accept,modif,ftry,linwr,newfle=.true.,genkin

      type morepart
       integer :: id
       integer, pointer :: ip(:)
      end type morepart

      type order
       integer :: n,dx,s,s0,is,o2(2),o3(3),i3,ig,ii,im
       integer, pointer :: ix(:),iy(:),ip(:)
       character(80) :: m,w
      end type order

c ig - No of final state particles in opl(i,j)
c ii - No of initial state particles in opl(i,j)

      type boostp
       integer :: d
       integer, pointer :: p(:)
      end type boostp

      type(morepart) :: ipli(d1+d2),iplj(d1+d2)
      type(order) :: opl(2,d1+d2)
      type(boostp) :: momb(nexpl-4)

      save ntop,ntop0,icon0,iconx,iplx,iply !icony0,icony

c      write(*,*) 'Checking topology:'
c      do j=1,d1
c       write(6,'(4(i3))') j,v1(j,:)
c      end do
c      do j=1,d2
c       write(6,'(4(i3))') j,v2(j,:)
c      end do

      open(113,file='colfskk.txt',access='direct',form='formatted',
     &                                                     recl=160)

      nuga=0

      e1=d1
      e2=d2

      allocate (x1(e1,3),stat=blad)
      allocate (x2(e2,3),stat=blad)

      x1=v1
      x2=v2

      if (e1 >= e2) then

      do
       modif=.false.
       ftry=.true.
       if (e1-1 >= e2+1) then
        j=0
        do i=1,3
         if (x1(e1,i) < 0) then
          j=j+1
          k=i
         end if
        end do
        if (j == 1 .and. abs(x1(e1,k)) == e1-1) then
         allocate (w1(e1-1,3),stat=blad)
         allocate (w2(e2+1,3),stat=blad)
         w1=x1(:e1-1,:)
         w2(:e2,:)=x2
         w2(e2+1,:)=x1(e1,:)
         w2(e2+1,k)=-e2
         deallocate (x1,x2,stat=blad)
         modif=.true.
        else
         exit
        end if
       else
        exit
       end if
       if (modif) then
        e1=e1-1
        e2=e2+1
        allocate (x1(e1,3),stat=blad)
        allocate (x2(e2,3),stat=blad)
        x1=w1
        x2=w2
        deallocate (w1,w2,stat=blad)
        if (e1-1 <= e2+1) exit
       end if
      end do

      else ! (e1 >= e2) 

      do
       modif=.false.
       ftry=.true.
c       if (e2-1 >= e1+1) then
       if (e2 >= e1+1) then
        j=0
        do i=1,3
         if (x2(e2,i) < 0) then
          j=j+1
          k=i
         end if
        end do
        if (j == 1 .and. abs(x2(e2,k)) == e2-1) then
         allocate (w1(e1+1,3),stat=blad)
         allocate (w2(e2-1,3),stat=blad)
         w1(:e1,:)=x1
         w2=x2(:e2-1,:)
         w1(e1+1,:)=x2(e2,:)
         w1(e1+1,k)=-e1
         deallocate (x1,x2,stat=blad)
         modif=.true.
        else
         exit
        end if
       else
        exit
       end if
       if (modif) then
        e1=e1+1
        e2=e2-1
        allocate (x1(e1,3),stat=blad)
        allocate (x2(e2,3),stat=blad)
        x1=w1
        x2=w2
        deallocate (w1,w2,stat=blad)
c        if (e2-1 <= e1+1) exit
        if (e2 <= e1+1) exit
       end if
      end do

      end if ! (e1 >= e2)

c      write(*,*) 'Modified topology:',ic,e1,e2
c      do j=1,e1
c       write(6,8030) j,x1(j,:)
c      end do
c      do j=1,e2
c       write(6,8030) j,x2(j,:)
c      end do
c 8030 format(4(i3))

      iplin=ipl
      ipl0=ipl

      sti=0
      stj=0

      iply=iply+1

      do i=1,e1
       if (x1(i,1) > 0 .and. x1(i,2) > 0 .and. x1(i,3) > 0) then
        i1=x1(i,1)
        i2=x1(i,2)
        i3=x1(i,3)
        ipl0=ipl
        call genpart(i1,i2,i3,ipl)
        dpl=ipl-ipl0
        allocate (ipli(i)%ip(dpl*mxmpl),stat=blad)
        if (dpl > 0) then
         call checkpl(ipl0,ipl,ipk)
         ipli(i)%id=dpl
         ipli(i)%ip(:ipli(i)%id)=ipk(:dpl)
c Check if particles ipl0,ipl0+1,...,ipl0+dpl mix with other particles
         if (i < e1) then
          do j=1,dpl
           if (pl(ipk(j))%mx == 0) then
            call mixpl(ipk(j),ipl)
            if (ipk(j) <= ipl0) iplin=ipl
           end if
           ipli(i)%ip(ipli(i)%id+1:ipli(i)%id+pl(ipk(j))%dm)=
     &                                               pl(ipk(j))%im
           ipli(i)%id=ipli(i)%id+pl(ipk(j))%dm
          end do
         end if
        else
         sti=i
         call clean
         accept=.false.
         return
        end if
       else if (x1(i,1) > 0 .and. x1(i,2) > 0 .and. x1(i,3) < 0) then
        i1=x1(i,1)
        i2=x1(i,2)
        abi3=abs(x1(i,3))
        allocate (ipli(i)%ip(ipli(abi3)%id*mxvpl*mxmpl),stat=blad)
        ipli(i)%id=0
        do k3=1,ipli(abi3)%id
         i3=ipli(abi3)%ip(k3)
         ipl0=ipl
         call genpart(i1,i2,i3,ipl)
         dpl=ipl-ipl0
         if (dpl > 0) then
          call checkpl(ipl0,ipl,ipk)
          ipli(i)%ip(ipli(i)%id+1:ipli(i)%id+dpl)=ipk(:dpl)
          ipli(i)%id=ipli(i)%id+dpl
c Check if particles ipl0,ipl0+1,...,ipl0+dpl mix with other particles
          if (i < e1) then
           do j=1,dpl
            if (pl(ipk(j))%mx == 0) then
             call mixpl(ipk(j),ipl)
             if (ipk(j) <= ipl0) iplin=ipl
            end if
            ipli(i)%ip(ipli(i)%id+1:ipli(i)%id+pl(ipk(j))%dm)=
     &                                               pl(ipk(j))%im
            ipli(i)%id=ipli(i)%id+pl(ipk(j))%dm
           end do
          end if
         end if
        end do
        if (ipli(i)%id == 0) then
         sti=i
         call clean
         accept=.false.
         return
        end if
       else if (x1(i,1) > 0 .and. x1(i,2) > 0 .and. x1(i,3) == 0) then
        i1=x1(i,1)
        i2=x1(i,2)
        i3=x1(i,3)
        ipl0=ipl
        call genpart(i1,i2,i3,ipl)
        dpl=ipl-ipl0
        allocate (ipli(i)%ip(dpl*mxmpl),stat=blad)
        if (dpl > 0) then
         call checkpl(ipl0,ipl,ipk)
         ipli(i)%id=dpl
         ipli(i)%ip(:ipli(i)%id)=ipk(:dpl)
c Check if particles ipl0,ipl0+1,...,ipl0+dpl mix with other particles
         if (i < e1) then
          do j=1,dpl
           if (pl(ipk(j))%mx == 0) then
            call mixpl(ipk(j),ipl)
            if (ipk(j) <= ipl0) iplin=ipl
           end if
           ipli(i)%ip(ipli(i)%id+1:ipli(i)%id+pl(ipk(j))%dm)=
     &                                               pl(ipk(j))%im
           ipli(i)%id=ipli(i)%id+pl(ipk(j))%dm
          end do
         end if
        else
         sti=i
         call clean
         accept=.false.
         return
        end if
       else if (x1(i,1) > 0 .and. x1(i,2) < 0 .and. x1(i,3) > 0) then
        i1=x1(i,1)
        abi2=abs(x1(i,2))
        i3=x1(i,3)
        allocate (ipli(i)%ip(ipli(abi2)%id*mxvpl*mxmpl),stat=blad)
        ipli(i)%id=0
        do k2=1,ipli(abi2)%id
         i2=ipli(abi2)%ip(k2)
         ipl0=ipl
         call genpart(i1,i2,i3,ipl)
         dpl=ipl-ipl0
         if (dpl > 0) then
          call checkpl(ipl0,ipl,ipk)
          ipli(i)%ip(ipli(i)%id+1:ipli(i)%id+dpl)=ipk(:dpl)
          ipli(i)%id=ipli(i)%id+dpl
c Check if particles ipl0,ipl0+1,...,ipl0+dpl mix with other particles
          if (i < e1) then
           do j=1,dpl
            if (pl(ipk(j))%mx == 0) then
             call mixpl(ipk(j),ipl)
             if (ipk(j) <= ipl0) iplin=ipl
            end if
            ipli(i)%ip(ipli(i)%id+1:ipli(i)%id+pl(ipk(j))%dm)=
     &                                               pl(ipk(j))%im
            ipli(i)%id=ipli(i)%id+pl(ipk(j))%dm
           end do
          end if
         end if
        end do
        if (ipli(i)%id == 0) then
         sti=i
         call clean
         accept=.false.
         return
        end if
       else if (x1(i,1) > 0 .and. x1(i,2) < 0 .and. x1(i,3) < 0) then
        i1=x1(i,1)
        abi2=abs(x1(i,2))
        abi3=abs(x1(i,3))
        allocate (ipli(i)%ip(ipli(abi2)%id*ipli(abi3)%id*mxvpl*mxmpl),
     &                                                     stat=blad)
        ipli(i)%id=0
        do k2=1,ipli(abi2)%id
         i2=ipli(abi2)%ip(k2)
         do k3=1,ipli(abi3)%id
          i3=ipli(abi3)%ip(k3)
          ipl0=ipl
          call genpart(i1,i2,i3,ipl)
          dpl=ipl-ipl0
          if (dpl > 0) then
           call checkpl(ipl0,ipl,ipk)
           ipli(i)%ip(ipli(i)%id+1:ipli(i)%id+dpl)=ipk(:dpl)
           ipli(i)%id=ipli(i)%id+dpl
c Check if particles ipl0,ipl0+1,...,ipl0+dpl mix with other particles
           if (i < e1) then
            do j=1,dpl
             if (pl(ipk(j))%mx == 0) then
              call mixpl(ipk(j),ipl)
              if (ipk(j) <= ipl0) iplin=ipl
             end if
             ipli(i)%ip(ipli(i)%id+1:ipli(i)%id+pl(ipk(j))%dm)=
     &                                               pl(ipk(j))%im
             ipli(i)%id=ipli(i)%id+pl(ipk(j))%dm
            end do
           end if
          end if
         end do
        end do
        if (ipli(i)%id == 0) then
         sti=i
         call clean
         accept=.false.
         return
        end if
       else if (x1(i,1) > 0 .and. x1(i,2) < 0 .and. x1(i,3) == 0) then
        i1=x1(i,1)
        i3=x1(i,3)
        abi2=abs(x1(i,2))
        allocate (ipli(i)%ip(ipli(abi2)%id*mxvpl*mxmpl),stat=blad)
        ipli(i)%id=0
        do k2=1,ipli(abi2)%id
         i2=ipli(abi2)%ip(k2)
         ipl0=ipl
         call genpart(i1,i2,i3,ipl)
         dpl=ipl-ipl0
         if (dpl > 0) then
          call checkpl(ipl0,ipl,ipk)
          ipli(i)%ip(ipli(i)%id+1:ipli(i)%id+dpl)=ipk(:dpl)
          ipli(i)%id=ipli(i)%id+dpl
c Check if particles ipl0,ipl0+1,...,ipl0+dpl mix with other particles
          if (i < e1) then
           do j=1,dpl
            if (pl(ipk(j))%mx == 0) then
             call mixpl(ipk(j),ipl)
             if (ipk(j) <= ipl0) iplin=ipl
            end if
            ipli(i)%ip(ipli(i)%id+1:ipli(i)%id+pl(ipk(j))%dm)=
     &                                               pl(ipk(j))%im
            ipli(i)%id=ipli(i)%id+pl(ipk(j))%dm
           end do
          end if
         end if
        end do
        if (ipli(i)%id == 0) then
         sti=i
         call clean
         accept=.false.
         return
        end if
       else if (x1(i,1) < 0 .and. x1(i,2) > 0 .and. x1(i,3) > 0) then
        abi1=abs(x1(i,1))
        i2=x1(i,2)
        i3=x1(i,3)
        allocate (ipli(i)%ip(ipli(abi1)%id*mxvpl*mxmpl),stat=blad)
        ipli(i)%id=0
        do k1=1,ipli(abi1)%id
         i1=ipli(abi1)%ip(k1)
         ipl0=ipl
         call genpart(i1,i2,i3,ipl)
         dpl=ipl-ipl0
         if (dpl > 0) then
          call checkpl(ipl0,ipl,ipk)
          ipli(i)%ip(ipli(i)%id+1:ipli(i)%id+dpl)=ipk(:dpl)
          ipli(i)%id=ipli(i)%id+dpl
c Check if particles ipl0,ipl0+1,...,ipl0+dpl mix with other particles
          if (i < e1) then
           do j=1,dpl
            if (pl(ipk(j))%mx == 0) then
             call mixpl(ipk(j),ipl)
             if (ipk(j) <= ipl0) iplin=ipl
            end if
            ipli(i)%ip(ipli(i)%id+1:ipli(i)%id+pl(ipk(j))%dm)=
     &                                               pl(ipk(j))%im
            ipli(i)%id=ipli(i)%id+pl(ipk(j))%dm
           end do
          end if
         end if
        end do
        if (ipli(i)%id == 0) then
         sti=i
         call clean
         accept=.false.
         return
        end if
       else if (x1(i,1) < 0 .and. x1(i,2) > 0 .and. x1(i,3) < 0) then
        abi1=abs(x1(i,1))
        i2=x1(i,2)
        abi3=abs(x1(i,3))
        allocate (ipli(i)%ip(ipli(abi1)%id*ipli(abi3)%id*mxvpl*mxmpl),
     &                                                     stat=blad)
        ipli(i)%id=0
        do k1=1,ipli(abi1)%id
         i1=ipli(abi1)%ip(k1)
         do k3=1,ipli(abi3)%id
          i3=ipli(abi3)%ip(k3)
          ipl0=ipl
          call genpart(i1,i2,i3,ipl)
          dpl=ipl-ipl0
          if (dpl > 0) then
           call checkpl(ipl0,ipl,ipk)
           ipli(i)%ip(ipli(i)%id+1:ipli(i)%id+dpl)=ipk(:dpl)
           ipli(i)%id=ipli(i)%id+dpl
c Check if particles ipl0,ipl0+1,...,ipl0+dpl mix with other particles
           if (i < e1) then
            do j=1,dpl
             if (pl(ipk(j))%mx == 0) then
              call mixpl(ipk(j),ipl)
              if (ipk(j) <= ipl0) iplin=ipl
             end if
             ipli(i)%ip(ipli(i)%id+1:ipli(i)%id+pl(ipk(j))%dm)=
     &                                               pl(ipk(j))%im
             ipli(i)%id=ipli(i)%id+pl(ipk(j))%dm
            end do
           end if
          end if
         end do
        end do
        if (ipli(i)%id == 0) then
         sti=i
         call clean
         accept=.false.
         return
        end if
       else if (x1(i,1) < 0 .and. x1(i,2) > 0 .and. x1(i,3) == 0) then
        abi1=abs(x1(i,1))
        i2=x1(i,2)
        i3=x1(i,3)
        allocate (ipli(i)%ip(ipli(abi1)%id*mxvpl*mxmpl),stat=blad)
        ipli(i)%id=0
        do k1=1,ipli(abi1)%id
         i1=ipli(abi1)%ip(k1)
         ipl0=ipl
         call genpart(i1,i2,i3,ipl)
         dpl=ipl-ipl0
         if (dpl > 0) then
          call checkpl(ipl0,ipl,ipk)
          ipli(i)%ip(ipli(i)%id+1:ipli(i)%id+dpl)=ipk(:dpl)
          ipli(i)%id=ipli(i)%id+dpl
c Check if particles ipl0,ipl0+1,...,ipl0+dpl mix with other particles
          if (i < e1) then
           do j=1,dpl
            if (pl(ipk(j))%mx == 0) then
             call mixpl(ipk(j),ipl)
             if (ipk(j) <= ipl0) iplin=ipl
            end if
            ipli(i)%ip(ipli(i)%id+1:ipli(i)%id+pl(ipk(j))%dm)=
     &                                               pl(ipk(j))%im
            ipli(i)%id=ipli(i)%id+pl(ipk(j))%dm
           end do
          end if
         end if
        end do
        if (ipli(i)%id == 0) then
         sti=i
         call clean
         accept=.false.
         return
        end if
       else if (x1(i,1) < 0 .and. x1(i,2) < 0 .and. x1(i,3) > 0) then
        abi1=abs(x1(i,1))
        abi2=abs(x1(i,2))
        i3=x1(i,3)
        allocate (ipli(i)%ip(ipli(abi1)%id*ipli(abi2)%id*mxvpl*mxmpl),
     &                                                     stat=blad)
        ipli(i)%id=0
        do k1=1,ipli(abi1)%id
         i1=ipli(abi1)%ip(k1)
         do k2=1,ipli(abi2)%id
          i2=ipli(abi2)%ip(k2)
          ipl0=ipl
          call genpart(i1,i2,i3,ipl)
          dpl=ipl-ipl0
          if (dpl > 0) then
           call checkpl(ipl0,ipl,ipk)
           ipli(i)%ip(ipli(i)%id+1:ipli(i)%id+dpl)=ipk(:dpl)
           ipli(i)%id=ipli(i)%id+dpl
c Check if particles ipl0,ipl0+1,...,ipl0+dpl mix with other particles
           if (i < e1) then
            do j=1,dpl
             if (pl(ipk(j))%mx == 0) then
              call mixpl(ipk(j),ipl)
              if (ipk(j) <= ipl0) iplin=ipl
             end if
             ipli(i)%ip(ipli(i)%id+1:ipli(i)%id+pl(ipk(j))%dm)=
     &                                               pl(ipk(j))%im
             ipli(i)%id=ipli(i)%id+pl(ipk(j))%dm
            end do
           end if
          end if
         end do
        end do
        if (ipli(i)%id == 0) then
         sti=i
         call clean
         accept=.false.
         return
        end if
       else if (x1(i,1) < 0 .and. x1(i,2) < 0 .and. x1(i,3) < 0) then
        abi1=abs(x1(i,1))
        abi2=abs(x1(i,2))
        abi3=abs(x1(i,3))
        allocate (ipli(i)%ip(ipli(abi1)%id*ipli(abi2)%id*
     &                            ipli(abi3)%id*mxvpl*mxmpl),stat=blad)
        ipli(i)%id=0
        do k1=1,ipli(abi1)%id
         i1=ipli(abi1)%ip(k1)
         do k2=1,ipli(abi2)%id
          i2=ipli(abi2)%ip(k2)
          do k3=1,ipli(abi3)%id
           i3=ipli(abi3)%ip(k3)
           ipl0=ipl
           call genpart(i1,i2,i3,ipl)
           dpl=ipl-ipl0
           if (dpl > 0) then
            call checkpl(ipl0,ipl,ipk)
            ipli(i)%ip(ipli(i)%id+1:ipli(i)%id+dpl)=ipk(:dpl)
            ipli(i)%id=ipli(i)%id+dpl
c Check if particles ipl0,ipl0+1,...,ipl0+dpl mix with other particles
            if (i < e1) then
             do j=1,dpl
              if (pl(ipk(j))%mx == 0) then
               call mixpl(ipk(j),ipl)
               if (ipk(j) <= ipl0) iplin=ipl
              end if
              ipli(i)%ip(ipli(i)%id+1:ipli(i)%id+pl(ipk(j))%dm)=
     &                                               pl(ipk(j))%im
              ipli(i)%id=ipli(i)%id+pl(ipk(j))%dm
             end do
            end if
           end if
          end do
         end do
        end do
        if (ipli(i)%id == 0) then
         sti=i
         call clean
         accept=.false.
         return
        end if
       else if (x1(i,1) < 0 .and. x1(i,2) < 0 .and. x1(i,3) == 0) then
        abi1=abs(x1(i,1))
        abi2=abs(x1(i,2))
        i3=x1(i,3)
        allocate (ipli(i)%ip(ipli(abi1)%id*ipli(abi2)%id*mxvpl*mxmpl),
     &                                                     stat=blad)
        ipli(i)%id=0
        do k1=1,ipli(abi1)%id
         i1=ipli(abi1)%ip(k1)
         do k2=1,ipli(abi2)%id
          i2=ipli(abi2)%ip(k2)
          ipl0=ipl
          call genpart(i1,i2,i3,ipl)
          dpl=ipl-ipl0
          if (dpl > 0) then
           call checkpl(ipl0,ipl,ipk)
           ipli(i)%ip(ipli(i)%id+1:ipli(i)%id+dpl)=ipk(:dpl)
           ipli(i)%id=ipli(i)%id+dpl
c Check if particles ipl0,ipl0+1,...,ipl0+dpl mix with other particles
           if (i < e1) then
            do j=1,dpl
             if (pl(ipk(j))%mx == 0) then
              call mixpl(ipk(j),ipl)
              if (ipk(j) <= ipl0) iplin=ipl
             end if
             ipli(i)%ip(ipli(i)%id+1:ipli(i)%id+pl(ipk(j))%dm)=
     &                                               pl(ipk(j))%im
             ipli(i)%id=ipli(i)%id+pl(ipk(j))%dm
            end do
           end if
          end if
         end do
        end do
        if (ipli(i)%id == 0) then
         sti=i
         call clean
         accept=.false.
         return
        end if
       end if
      end do

      iplx=iplx+1

      do i=1,e2
c       ipl0=ipl
       if (x2(i,1) > 0 .and. x2(i,2) > 0 .and. x2(i,3) > 0) then
        i1=x2(i,1)
        i2=x2(i,2)
        i3=x2(i,3)
        ipl0=ipl
        call genpart(i1,i2,i3,ipl)
        dpl=ipl-ipl0
        allocate (iplj(i)%ip(dpl*mxmpl),stat=blad)
        if (dpl > 0) then
         call checkpl(ipl0,ipl,ipk)
         iplj(i)%id=dpl
         iplj(i)%ip(:iplj(i)%id)=ipk(:dpl)
c Check if particles ipl0,ipl0+1,...,ipl0+dpl mix with other particles
         if (i < e2) then
          do j=1,dpl
           if (pl(ipk(j))%mx == 0) then
            call mixpl(ipk(j),ipl)
            if (ipk(j) <= ipl0) iplin=ipl
           end if
           iplj(i)%ip(iplj(i)%id+1:iplj(i)%id+pl(ipk(j))%dm)=
     &                                               pl(ipk(j))%im
           iplj(i)%id=iplj(i)%id+pl(ipk(j))%dm
          end do
         end if
        else
         stj=i
         call clean
         accept=.false.
         return
        end if
       else if (x2(i,1) > 0 .and. x2(i,2) > 0 .and. x2(i,3) < 0) then
        i1=x2(i,1)
        i2=x2(i,2)
        abi3=abs(x2(i,3))
        allocate (iplj(i)%ip(iplj(abi3)%id*mxvpl*mxmpl),stat=blad)
        iplj(i)%id=0
        do k3=1,iplj(abi3)%id
         i3=iplj(abi3)%ip(k3)
         ipl0=ipl
         call genpart(i1,i2,i3,ipl)
         dpl=ipl-ipl0
         if (dpl > 0) then
          call checkpl(ipl0,ipl,ipk)
          iplj(i)%ip(iplj(i)%id+1:iplj(i)%id+dpl)=ipk(:dpl)
          iplj(i)%id=iplj(i)%id+dpl
c Check if particles ipl0,ipl0+1,...,ipl0+dpl mix with other particles
          if (i < e2) then
           do j=1,dpl
            if (pl(ipk(j))%mx == 0) then
             call mixpl(ipk(j),ipl)
             if (ipk(j) <= ipl0) iplin=ipl
            end if
            iplj(i)%ip(iplj(i)%id+1:iplj(i)%id+pl(ipk(j))%dm)=
     &                                               pl(ipk(j))%im
            iplj(i)%id=iplj(i)%id+pl(ipk(j))%dm
           end do
          end if
         end if
        end do
        if (iplj(i)%id == 0) then
         stj=i
         call clean
         accept=.false.
         return
        end if
       else if (x2(i,1) > 0 .and. x2(i,2) > 0 .and. x2(i,3) == 0) then
        i1=x2(i,1)
        i2=x2(i,2)
        i3=x2(i,3)
        ipl0=ipl
        call genpart(i1,i2,i3,ipl)
        dpl=ipl-ipl0
        allocate (iplj(i)%ip(dpl*mxmpl),stat=blad)
        if (dpl > 0) then
         call checkpl(ipl0,ipl,ipk)
         iplj(i)%id=dpl
         iplj(i)%ip(:iplj(i)%id)=ipk(:dpl)
c Check if particles ipl0,ipl0+1,...,ipl0+dpl mix with other particles
         if (i < e2) then
          do j=1,dpl
           if (pl(ipk(j))%mx == 0) then
            call mixpl(ipk(j),ipl)
            if (ipk(j) <= ipl0) iplin=ipl
           end if
           iplj(i)%ip(iplj(i)%id+1:iplj(i)%id+pl(ipk(j))%dm)=
     &                                               pl(ipk(j))%im
           iplj(i)%id=iplj(i)%id+pl(ipk(j))%dm
          end do
         end if
        else
         stj=i
         call clean
         accept=.false.
         return
        end if
       else if (x2(i,1) > 0 .and. x2(i,2) < 0 .and. x2(i,3) > 0) then
        i1=x2(i,1)
        abi2=abs(x2(i,2))
        i3=x2(i,3)
        allocate (iplj(i)%ip(iplj(abi2)%id*mxvpl*mxmpl),stat=blad)
        iplj(i)%id=0
        do k2=1,iplj(abi2)%id
         i2=iplj(abi2)%ip(k2)
         ipl0=ipl
         call genpart(i1,i2,i3,ipl)
         dpl=ipl-ipl0
         if (dpl > 0) then
          call checkpl(ipl0,ipl,ipk)
          iplj(i)%ip(iplj(i)%id+1:iplj(i)%id+dpl)=ipk(:dpl)
          iplj(i)%id=iplj(i)%id+dpl
c Check if particles ipl0,ipl0+1,...,ipl0+dpl mix with other particles
          if (i < e2) then
           do j=1,dpl
            if (pl(ipk(j))%mx == 0) then
             call mixpl(ipk(j),ipl)
             if (ipk(j) <= ipl0) iplin=ipl
            end if
            iplj(i)%ip(iplj(i)%id+1:iplj(i)%id+pl(ipk(j))%dm)=
     &                                               pl(ipk(j))%im
            iplj(i)%id=iplj(i)%id+pl(ipk(j))%dm
           end do
          end if
         end if
        end do
        if (iplj(i)%id == 0) then
         stj=i
         call clean
         accept=.false.
         return
        end if
       else if (x2(i,1) > 0 .and. x2(i,2) < 0 .and. x2(i,3) < 0) then
        i1=x2(i,1)
        abi2=abs(x2(i,2))
        abi3=abs(x2(i,3))
        allocate (iplj(i)%ip(iplj(abi2)%id*iplj(abi3)%id*mxvpl*mxmpl),
     &                                                     stat=blad)
        iplj(i)%id=0
        do k2=1,iplj(abi2)%id
         i2=iplj(abi2)%ip(k2)
         do k3=1,iplj(abi3)%id
          i3=iplj(abi3)%ip(k3)
          ipl0=ipl
          call genpart(i1,i2,i3,ipl)
          dpl=ipl-ipl0
          if (dpl > 0) then
           call checkpl(ipl0,ipl,ipk)
           iplj(i)%ip(iplj(i)%id+1:iplj(i)%id+dpl)=ipk(:dpl)
           iplj(i)%id=iplj(i)%id+dpl
c Check if particles ipl0,ipl0+1,...,ipl0+dpl mix with other particles
           if (i < e2) then
            do j=1,dpl
             if (pl(ipk(j))%mx == 0) then
              call mixpl(ipk(j),ipl)
              if (ipk(j) <= ipl0) iplin=ipl
             end if
             iplj(i)%ip(iplj(i)%id+1:iplj(i)%id+pl(ipk(j))%dm)=
     &                                               pl(ipk(j))%im
             iplj(i)%id=iplj(i)%id+pl(ipk(j))%dm
            end do
           end if
          end if
         end do
        end do
        if (iplj(i)%id == 0) then
         stj=i
         call clean
         accept=.false.
         return
        end if
       else if (x2(i,1) > 0 .and. x2(i,2) < 0 .and. x2(i,3) == 0) then
        i1=x2(i,1)
        i3=x2(i,3)
        abi2=abs(x2(i,2))
        allocate (iplj(i)%ip(iplj(abi2)%id*mxvpl*mxmpl),stat=blad)
        iplj(i)%id=0
        do k2=1,iplj(abi2)%id
         i2=iplj(abi2)%ip(k2)
         ipl0=ipl
         call genpart(i1,i2,i3,ipl)
         dpl=ipl-ipl0
         if (dpl > 0) then
          call checkpl(ipl0,ipl,ipk)
          iplj(i)%ip(iplj(i)%id+1:iplj(i)%id+dpl)=ipk(:dpl)
          iplj(i)%id=iplj(i)%id+dpl
c Check if particles ipl0,ipl0+1,...,ipl0+dpl mix with other particles
          if (i < e2) then
           do j=1,dpl
            if (pl(ipk(j))%mx == 0) then
             call mixpl(ipk(j),ipl)
             if (ipk(j) <= ipl0) iplin=ipl
            end if
            iplj(i)%ip(iplj(i)%id+1:iplj(i)%id+pl(ipk(j))%dm)=
     &                                               pl(ipk(j))%im
            iplj(i)%id=iplj(i)%id+pl(ipk(j))%dm
           end do
          end if
         end if
        end do
        if (iplj(i)%id == 0) then
         stj=i
         call clean
         accept=.false.
         return
        end if
       else if (x2(i,1) < 0 .and. x2(i,2) > 0 .and. x2(i,3) > 0) then
        abi1=abs(x2(i,1))
        i2=x2(i,2)
        i3=x2(i,3)
        allocate (iplj(i)%ip(iplj(abi1)%id*mxvpl*mxmpl),stat=blad)
        iplj(i)%id=0
        do k1=1,iplj(abi1)%id
         i1=iplj(abi1)%ip(k1)
         ipl0=ipl
         call genpart(i1,i2,i3,ipl)
         dpl=ipl-ipl0
         if (dpl > 0) then
          call checkpl(ipl0,ipl,ipk)
          iplj(i)%ip(iplj(i)%id+1:iplj(i)%id+dpl)=ipk(:dpl)
          iplj(i)%id=iplj(i)%id+dpl
c Check if particles ipl0,ipl0+1,...,ipl0+dpl mix with other particles
          if (i < e2) then
           do j=1,dpl
            if (pl(ipk(j))%mx == 0) then
             call mixpl(ipk(j),ipl)
             if (ipk(j) <= ipl0) iplin=ipl
            end if
            iplj(i)%ip(iplj(i)%id+1:iplj(i)%id+pl(ipk(j))%dm)=
     &                                               pl(ipk(j))%im
            iplj(i)%id=iplj(i)%id+pl(ipk(j))%dm
           end do
          end if
         end if
        end do
        if (iplj(i)%id == 0) then
         stj=i
         call clean
         accept=.false.
         return
        end if
       else if (x2(i,1) < 0 .and. x2(i,2) > 0 .and. x2(i,3) < 0) then
        abi1=abs(x2(i,1))
        i2=x2(i,2)
        abi3=abs(x2(i,3))
        allocate (iplj(i)%ip(iplj(abi1)%id*iplj(abi3)%id*mxvpl*mxmpl),
     &                                                     stat=blad)
        iplj(i)%id=0
        do k1=1,iplj(abi1)%id
         i1=iplj(abi1)%ip(k1)
         do k3=1,iplj(abi3)%id
          i3=iplj(abi3)%ip(k3)
          ipl0=ipl
          call genpart(i1,i2,i3,ipl)
          dpl=ipl-ipl0
          if (dpl > 0) then
           call checkpl(ipl0,ipl,ipk)
           iplj(i)%ip(iplj(i)%id+1:iplj(i)%id+dpl)=ipk(:dpl)
           iplj(i)%id=iplj(i)%id+dpl
c Check if particles ipl0,ipl0+1,...,ipl0+dpl mix with other particles
           if (i < e2) then
            do j=1,dpl
             if (pl(ipk(j))%mx == 0) then
              call mixpl(ipk(j),ipl)
              if (ipk(j) <= ipl0) iplin=ipl
             end if
             iplj(i)%ip(iplj(i)%id+1:iplj(i)%id+pl(ipk(j))%dm)=
     &                                               pl(ipk(j))%im
             iplj(i)%id=iplj(i)%id+pl(ipk(j))%dm
            end do
           end if
          end if
         end do
        end do
        if (iplj(i)%id == 0) then
         stj=i
         call clean
         accept=.false.
         return
        end if
       else if (x2(i,1) < 0 .and. x2(i,2) > 0 .and. x2(i,3) == 0) then
        abi1=abs(x2(i,1))
        i2=x2(i,2)
        i3=x2(i,3)
        allocate (iplj(i)%ip(iplj(abi1)%id*mxvpl*mxmpl),stat=blad)
        iplj(i)%id=0
        do k1=1,iplj(abi1)%id
         i1=iplj(abi1)%ip(k1)
         ipl0=ipl
         call genpart(i1,i2,i3,ipl)
         dpl=ipl-ipl0
         if (dpl > 0) then
          call checkpl(ipl0,ipl,ipk)
          iplj(i)%ip(iplj(i)%id+1:iplj(i)%id+dpl)=ipk(:dpl)
          iplj(i)%id=iplj(i)%id+dpl
c Check if particles ipl0,ipl0+1,...,ipl0+dpl mix with other particles
          if (i < e2) then
           do j=1,dpl
            if (pl(ipk(j))%mx == 0) then
             call mixpl(ipk(j),ipl)
             if (ipk(j) <= ipl0) iplin=ipl
            end if
            iplj(i)%ip(iplj(i)%id+1:iplj(i)%id+pl(ipk(j))%dm)=
     &                                               pl(ipk(j))%im
            iplj(i)%id=iplj(i)%id+pl(ipk(j))%dm
           end do
          end if
         end if
        end do
        if (iplj(i)%id == 0) then
         stj=i
         call clean
         accept=.false.
         return
        end if
       else if (x2(i,1) < 0 .and. x2(i,2) < 0 .and. x2(i,3) > 0) then
        abi1=abs(x2(i,1))
        abi2=abs(x2(i,2))
        allocate (iplj(i)%ip(iplj(abi1)%id*iplj(abi2)%id*mxvpl*mxmpl),
     &                                                     stat=blad)
        i3=x2(i,3)
        iplj(i)%id=0
        do k1=1,iplj(abi1)%id
         i1=iplj(abi1)%ip(k1)
         do k2=1,iplj(abi2)%id
          i2=iplj(abi2)%ip(k2)
          ipl0=ipl
          call genpart(i1,i2,i3,ipl)
          dpl=ipl-ipl0
          if (dpl > 0) then
           call checkpl(ipl0,ipl,ipk)
           iplj(i)%ip(iplj(i)%id+1:iplj(i)%id+dpl)=ipk(:dpl)
           iplj(i)%id=iplj(i)%id+dpl
c Check if particles ipl0,ipl0+1,...,ipl0+dpl mix with other particles
           if (i < e2) then
            do j=1,dpl
             if (pl(ipk(j))%mx == 0) then
              call mixpl(ipk(j),ipl)
              if (ipk(j) <= ipl0) iplin=ipl
             end if
             iplj(i)%ip(iplj(i)%id+1:iplj(i)%id+pl(ipk(j))%dm)=
     &                                               pl(ipk(j))%im
             iplj(i)%id=iplj(i)%id+pl(ipk(j))%dm
            end do
           end if
          end if
         end do
        end do
        if (iplj(i)%id == 0) then
         stj=i
         call clean
         accept=.false.
         return
        end if
       else if (x2(i,1) < 0 .and. x2(i,2) < 0 .and. x2(i,3) < 0) then
        abi1=abs(x2(i,1))
        abi2=abs(x2(i,2))
        abi3=abs(x2(i,3))
        allocate (iplj(i)%ip(iplj(abi1)%id*iplj(abi2)%id*
     &                    iplj(abi3)%id*mxvpl*mxmpl),stat=blad)
        iplj(i)%id=0
        do k1=1,iplj(abi1)%id
         i1=iplj(abi1)%ip(k1)
         do k2=1,iplj(abi2)%id
          i2=iplj(abi2)%ip(k2)
          do k3=1,iplj(abi3)%id
           i3=iplj(abi3)%ip(k3)
           ipl0=ipl
           call genpart(i1,i2,i3,ipl)
           dpl=ipl-ipl0
           if (dpl > 0) then
            call checkpl(ipl0,ipl,ipk)
            iplj(i)%ip(iplj(i)%id+1:iplj(i)%id+dpl)=ipk(:dpl)
            iplj(i)%id=iplj(i)%id+dpl
c Check if particles ipl0,ipl0+1,...,ipl0+dpl mix with other particles
            if (i < e2) then
             do j=1,dpl
              if (pl(ipk(j))%mx == 0) then
               call mixpl(ipk(j),ipl)
               if (ipk(j) <= ipl0) iplin=ipl
              end if
              iplj(i)%ip(iplj(i)%id+1:iplj(i)%id+pl(ipk(j))%dm)=
     &                                               pl(ipk(j))%im
              iplj(i)%id=iplj(i)%id+pl(ipk(j))%dm
             end do
            end if
           end if
          end do
         end do
        end do
        if (iplj(i)%id == 0) then
         stj=i
         call clean
         accept=.false.
         return
        end if
       else if (x2(i,1) < 0 .and. x2(i,2) < 0 .and. x2(i,3) == 0) then
        abi1=abs(x2(i,1))
        abi2=abs(x2(i,2))
        i3=x2(i,3)
        allocate (iplj(i)%ip(iplj(abi1)%id*iplj(abi2)%id*mxvpl*mxmpl),
     &                                                     stat=blad)
        iplj(i)%id=0
        do k1=1,iplj(abi1)%id
         i1=iplj(abi1)%ip(k1)
         do k2=1,iplj(abi2)%id
          i2=iplj(abi2)%ip(k2)
          ipl0=ipl
          call genpart(i1,i2,i3,ipl)
          dpl=ipl-ipl0
          if (dpl > 0) then
           call checkpl(ipl0,ipl,ipk)
           iplj(i)%ip(iplj(i)%id+1:iplj(i)%id+dpl)=ipk(:dpl)
           iplj(i)%id=iplj(i)%id+dpl
c Check if particles ipl0,ipl0+1,...,ipl0+dpl mix with other particles
           if (i < e2) then
            do j=1,dpl
             if (pl(ipk(j))%mx == 0) then
              call mixpl(ipk(j),ipl)
              if (ipk(j) <= ipl0) iplin=ipl
             end if
             iplj(i)%ip(iplj(i)%id+1:iplj(i)%id+pl(ipk(j))%dm)=
     &                                               pl(ipk(j))%im
             iplj(i)%id=iplj(i)%id+pl(ipk(j))%dm
            end do
           end if
          end if
         end do
        end do
        if (iplj(i)%id == 0) then
         stj=i
         call clean
         accept=.false.
         return
        end if
       else if (x2(i,1) > 0 .and. x2(i,2) == 0 .and. x2(i,3) == 0) then
        allocate (iplj(i)%ip(1),stat=blad)
        iplj(i)%id=1
        iplj(i)%ip(1)=x2(i,1)
       else
        stj=i
        call clean
        accept=.false.
        return
       end if
      end do

      lu=23

      open(unit=lu,status='scratch',action='readwrite')

      id0=id
      nkm0=nkm    ! 29 Sept. 2010

      do i1=1,ipli(e1)%id
       do i2=1,iplj(e2)%id
        i=ipli(e1)%ip(i1)
        j=iplj(e2)%ip(i2)
        id1=id
        call matchkk(i,e1,j,e2,iord,plord,ncls,id,sgn)
        if (id > id1) then
 1070    format(2(i10))
         call colinf(i,size(pl(i)%ip),j,size(pl(j)%ip),iord,clfc,lclfc,
     &               cst,ist,cso,iso)
         if (id == 1) then
          idr=idr+1
          write(70,1070) i,j
          write(111,'(3(i16))') sgn,id,idr
c          write(115,'(40(i2))',rec=idr) plord
          write(113,'(a)',rec=idr) clfc(1:lclfc)
          call wrcolrtn(clfc(1:lclfc),lclfc)
         else
          iorx=0
          do ii=1,idr
           read(113,'(a)',rec=ii) clfcx
           istr0=0
           lstr0=len_trim(clfcx)
           do
            istr=index(clfcx(istr0+1:),'*')
            if (istr > 0) then
             clfcy=clfcx(istr0+1:istr0+istr-1)
             istr0=istr0+istr
             if (index(clfc,trim(clfcy)) > 0) then
              iorx=ii
             else
              if (clfcy(1:2) == 'tt') then
c Try to use symmetry properties for tt
               icmm=index(clfcy,',')
               icmm=icmm+index(clfcy(icmm+1:),',')
               clfcy=clfcy(1:3)//clfcy(icmm+1:istr-2)//','//
     &               clfcy(4:icmm-1)//')'
               if (index(clfc,trim(clfcy)) > 0) then
                iorx=ii
               else
                iorx=0
                exit
               end if
              else
               iorx=0
               exit
              end if
             end if
            else
             clfcy=clfcx(istr0+1:lstr0)
             if (index(clfc,trim(clfcy)) > 0) then
              iorx=ii
             else
              if (clfcy(1:2) == 'tt') then
c Try to use symmetry properties for tt
               icmm=index(clfcy,',')
               icmm=icmm+index(clfcy(icmm+1:),',')
               clfcy=clfcy(1:3)//clfcy(icmm+1:lstr0-istr0-1)//','//
     &               clfcy(4:icmm-1)//')'
               if (index(clfc,trim(clfcy)) > 0) then
                iorx=ii
               else
                iorx=0
               end if
              else
               iorx=0
              end if
c              iorx=0
             end if
             exit
            end if
           end do
           if (iorx > 0) exit
          end do
          if (iorx > 0) then
           write(111,'(3(i16))') sgn,id,iorx
          else
           idr=idr+1
           write(70,1070) i,j
           write(111,'(3(i16))') sgn,id,idr
           write(113,'(a)',rec=idr) clfc(1:lclfc)
           call wrcolrtn(clfc(1:lclfc),lclfc)
          end if
         end if
                                            ! 11 April 2014
c         if (max(pl(i)%mx,pl(j)%mx) < 2) then ! Kinematics written only for
                                            ! the diagrams without mixing
          iss=0

          if (pl(i)%wr == 1 .and. pl(j)%wr == 0) then
           opli=0
           oplm=1
           oplc=''
           if (i > nexpl) then
            do iyy=1,size(pl(i)%ip)
             if (pl(pl(i)%ip(iyy))%mx > oplm) then
              oplm=pl(pl(i)%ip(iyy))%mx
              oplc='(with mixing)'
             else
              opli=opli+1
              call wrkinrtn(1,opli,pl(i)%ip(iyy))
             end if
            end do
            if (pl(i)%mx > oplm) then
             oplm=pl(i)%mx
             oplc='(with mixing)'
            else
             opli=opli+1
             call wrkinrtn(1,opli,i)
            end if
           end if
           iss1=iss
           oplj=0
           oplm=1
           opld=''
           if (j > nexpl) then
            do iyy=1,size(pl(j)%ip)
             if (pl(pl(j)%ip(iyy))%mx > oplm) then
              oplm=pl(pl(j)%ip(iyy))%mx
              opld='(with mixing)'
             else
              oplj=oplj+1
              call wrkinrtn(2,oplj,pl(j)%ip(iyy))
             end if
            end do
            if (pl(j)%mx > oplm) then
             oplm=pl(j)%mx
             opld='(with mixing)'
            else
             oplj=oplj+1
             call wrkinrtn(2,oplj,j)
            end if
           end if
          else if (pl(i)%wr == 0 .and. pl(j)%wr == 1) then
           oplj=0
           oplm=1
           opld=''
           if (j > nexpl) then
            do iyy=1,size(pl(j)%ip)
             if (pl(pl(j)%ip(iyy))%mx > oplm) then
              oplm=pl(pl(j)%ip(iyy))%mx
              opld='(with mixing)'
             else
              oplj=oplj+1
              call wrkinrtn(2,oplj,pl(j)%ip(iyy))
             end if
            end do
            if (pl(j)%mx > oplm) then
             oplm=pl(j)%mx
             opld='(with mixing)'
            else
             oplj=oplj+1
             call wrkinrtn(2,oplj,j)
            end if
           end if
           iss1=iss
           opli=0
           oplm=1
           oplc=''
           if (i > nexpl) then
            do iyy=1,size(pl(i)%ip)
             if (pl(pl(i)%ip(iyy))%mx > oplm) then
              oplm=pl(pl(i)%ip(iyy))%mx
              oplc='(with mixing)'
             else
              opli=opli+1
              call wrkinrtn(1,opli,pl(i)%ip(iyy))
             end if
            end do
            if (pl(i)%mx > oplm) then
             oplm=pl(i)%mx
             oplc='(with mixing)'
            else
             opli=opli+1
             call wrkinrtn(1,opli,i)
            end if
           end if
          else
           opli=0
           oplm=1
           oplc=''
           if (i > nexpl) then
            do iyy=1,size(pl(i)%ip)
             if (pl(pl(i)%ip(iyy))%mx > oplm) then
              oplm=pl(pl(i)%ip(iyy))%mx
              oplc='(with mixing)'
             else
              opli=opli+1
              call wrkinrtn(1,opli,pl(i)%ip(iyy))
             end if
            end do
            if (pl(i)%mx > oplm) then
             oplm=pl(i)%mx
             oplc='(with mixing)'
            else
             opli=opli+1
             call wrkinrtn(1,opli,i)
            end if
           end if
           iss1=iss
           oplj=0
           oplm=1
           opld=''
           if (j > nexpl) then
            do iyy=1,size(pl(j)%ip)
             if (pl(pl(j)%ip(iyy))%mx > oplm) then
              oplm=pl(pl(j)%ip(iyy))%mx
              opld='(with mixing)'
             else
              oplj=oplj+1
              call wrkinrtn(2,oplj,pl(j)%ip(iyy))
             end if
            end do
            if (pl(j)%mx > oplm) then
             oplm=pl(j)%mx
             opld='(with mixing)'
            else
             oplj=oplj+1
             call wrkinrtn(2,oplj,j)
            end if
           end if
          end if
          genkin=.true.
          call nokinem(e1,e2,x1,x2,oplc,opld,genkin)
          if (genkin) then
           call kinwrt
          end if ! genkin

c         end if
        end if
       end do
      end do
c Tu cos moge zrobic
      if (id > id0) then
       accept=.true.
      else
       accept=.false.       ! Introduced on 18 May, 2013
       do i1=1,ipli(e1)%id
        i=ipli(e1)%ip(i1)
        do ii=1,size(pl(i)%ip)
         j=pl(i)%ip(ii)
c         if (pl(j)%wr == 0) then 
         if (pl(j)%wr == 0 .and. j > iplin) then ! Corrected 13 Sept. 2007
          deallocate(pl(j)%ip,stat=blad)
         end if
        end do
       end do
c Corrected on 29 July, 2007:
c       do i2=1,ipli(e2)%id
c        i=ipli(e2)%ip(i2)
       do i2=1,iplj(e2)%id
        i=iplj(e2)%ip(i2)
        do ii=1,size(pl(i)%ip)
         j=pl(i)%ip(ii)
c         if (pl(j)%wr == 0) then 
         if (pl(j)%wr == 0 .and. j > iplin) then ! Corrected 13 Sept. 2007
          deallocate(pl(j)%ip,stat=blad)
         end if
        end do
       end do
       ipl=iplin
      end if

      close(lu,status='delete')

      if (accept) then
       ntop0=ntop
       ntop=ntop+1
      end if
      if (accept) then
       write(*,'(a,i12,a,i12,a,i12)') 'Topology',ntop,' of diagrams',
     &        id0+1,'    to',id
       do i=e1,1,-1
c        write(*,*) i,x1(i,:),':',ipli(i)%ip(1:ipli(i)%id)
        write(*,'(4i3)') i,x1(i,:)
       end do
       do i=e2,1,-1
c        write(*,*) i,x2(i,:),':',iplj(i)%ip(1:iplj(i)%id)
        write(*,'(4i3)') i,x2(i,:)
       end do

ckk>
c Added on 25 Feb. 2010:
c       write(ind1,'(i16)') ntop
c       write(ind2,'(i16)') id
c       cntn='nt'//trim(adjustl(ind1))//'='//trim(adjustl(ind2))
c       call wrntdcls(272,cntn)
c Changed on 5 Oct. 2010:
       if (nexpl > 4) then   ! 17 Jan. 2011
        write(ind1,'(i16)') ntop
        write(ind2,'(i16)') nkm
        cntn='nt'//trim(adjustl(ind1))//'='//trim(adjustl(ind2))
        call wrntdcls(272,cntn)
       end if                ! 17 Jan. 2011
ckk<
      end if

      do i=1,e1
       deallocate (ipli(i)%ip,stat=blad)
      end do
      do i=1,e2
       deallocate (iplj(i)%ip,stat=blad)
      end do
      deallocate (x1,x2,stat=blad)

      close(113)

      contains

       subroutine clean

        do ii=1,sti
         deallocate (ipli(ii)%ip,stat=blad)
        end do
        do ii=1,stj
         deallocate (iplj(ii)%ip,stat=blad)
        end do
        do ii=iplin+1,ipl
c         deallocate (pl(ii)%ip,stat=blad)
         deallocate (pl(ii)%ip,pl(ii)%im,stat=blad)   ! 1 Oct. 2014
         pl(ii)%dm=0                                  ! 1 Oct. 2014
         pl(ii)%mx=0                                  ! 1 Oct. 2014
        end do
        deallocate (x1,x2,stat=blad)

        ipl=iplin

       end subroutine clean

       subroutine colinf(ix,ixd,jx,jxd,iord,clfd,lcl1,cst,ist,cso,iso)
c Stores color information
        integer :: ix,ixd,jx,jxd,iord
        integer :: iy,iyd,jy,jyd,lcl0,lcl1,lclx,lcix,icix,lclj,ist,iso,
     &             lciy,lciz,icix1,iiy,indx(ixd+jxd)
        character(80) cix,ciy,ciz
        character(160) clx,clfd
        character(3), dimension(nexpl-3) :: cst,cso

c        open(13,file='colfs.txt',access='direct',form='formatted',
c     &                                                     recl=80)

        ist=0
        iso=0

        if (iord == 1) then
         iy=ix
         iyd=ixd
         jy=jx
         jyd=jxd
        else if (iord == -1) then
         iy=jx
         iyd=jxd
         jy=ix
         jyd=ixd
        else
         write(*,*)'checktop.f: Wrong value of iord, iord=',iord
        end if

        lcl0=1
        lclx=1
        clfd=''

        do iiy=1,iyd
         read(13,'(a)',rec=pl(iy)%ip(iiy)) clx
         if (clx(1:1) /= '1') then
          lcl0=lclx
          lclx=lclx+len_trim(clx)+1
c          lclx=lclx+len_trim(clx)
          clfd(lcl0:lclx)=trim(clx)//'*'
          indx(iiy)=pl(iy)%ip(iiy)
         else
          indx(iiy)=0
         end if
        end do

        read(13,'(a)',rec=iy) clx
        write(ind1,'(i16)') iy
        cix='i'//trim(adjustl(ind1))

        lcix=len_trim(cix)
c        icix=index(clx,trim(cix))
        icix=max(index(clx,trim(cix)//','),
     &           index(clx,trim(cix)//')'))
        if (pl(iy)%ct == 3 .and. jy > nexpl) then
         ist=ist+1
         write(ind1,'(i16)') ist
         ciy='j'//trim(adjustl(ind1))
         cst(ist)=trim(ciy)
        else if (pl(iy)%ct == 8 .and. jy > nexpl) then
         iso=iso+1
         write(ind1,'(i16)') iso
         ciy='k'//trim(adjustl(ind1))
         cso(iso)=trim(ciy)
        else
         write(ind1,'(i16)') jy
         ciy='i'//trim(adjustl(ind1))
        end if
        lciy=len_trim(ciy)
        if (icix > 0) then
         clx=clx(1:icix-1)//ciy(1:lciy)//clx(icix+lcix:)
        end if

        if (clx(1:1) /= '1') then
         lcl0=lclx
         lclx=lclx+len_trim(clx)
         clfd(lcl0:lclx)=trim(clx)//'*'
        end if

        if (index(clfd(1:lclx),'*',.true.) == lclx) then
         clx=clfd(1:lclx-1)
        else
         clx=clfd(1:lclx)
        end if

        do iiy=1,iyd
         write(ind1,'(i16)') pl(iy)%ip(iiy)
         cix='i'//trim(adjustl(ind1))
         lcix=len_trim(cix)
c         icix=max(index(clx,trim(cix)//','),
c     &            index(clx,trim(cix)//')'))
         icxa=index(clx,trim(cix)//',')
         icxb=index(clx,trim(cix)//')')
         if (icxa > 0 .and. icxb > 0) then
          icix=min(icxa,icxb)
         else
          icix=max(icxa,icxb)
         end if
         icix1=max(index(clx,trim(cix)//',',.true.),
     &             index(clx,trim(cix)//')',.true.))
         if (pl(pl(iy)%ip(iiy))%ct == 3) then
          ist=ist+1
          write(ind1,'(i16)') ist
          ciz='j'//trim(adjustl(ind1))
          cst(ist)=trim(ciz)
         else if (pl(pl(iy)%ip(iiy))%ct == 8) then
          iso=iso+1
          write(ind1,'(i16)') iso
          ciz='k'//trim(adjustl(ind1))
          cso(iso)=trim(ciz)
         else
          ciz=''
         end if
         lciz=len_trim(ciz)
         if (lciz > 0) then
          if (icix > 0 .and. icix1 > icix) then
           clx=clx(1:icix-1)//ciz(1:lciz)//clx(icix+lcix:icix1-1)//
     &                        ciz(1:lciz)//clx(icix1+lcix:)
          end if
          call sumdlts(clx,trim(ciz))
         end if
        end do

c        lclx=len_trim(clx)+1
c        clfd(1:lclx)=trim(clx)//'*'

        lclx=len_trim(clx)
        clfd(1:lclx)=trim(clx)

c        if (lclx > 1 .and. clfd(lclx:lclx) /= '*') then
        if (lclx > 0) then                  ! 23 July, 2014
         if (clfd(lclx:lclx) /= '*') then ! 23 July, 2014
          lclx=lclx+1
          clfd(1:lclx)=trim(clx)//'*'
         end if
        end if

        lclj=lclx
        lcl0=lclx

        do iiy=1,jyd
         read(13,'(a)',rec=pl(jy)%ip(iiy)) clx
         if (clx(1:1) /= '1') then
          lcl0=lclx
          lclx=lclx+len_trim(clx)+1
          clfd(lcl0+1:lclx)=trim(clx)//'*'
          indx(iyd+iiy)=pl(jy)%ip(iiy)
         else
          indx(iyd+iiy)=0
         end if
        end do

        read(13,'(a)',rec=jy) clx
        write(ind1,'(i16)') jy
        cix='i'//trim(adjustl(ind1))
        lcix=len_trim(cix)
        icix=max(index(clx,trim(cix)//','),
     &           index(clx,trim(cix)//')'))
        if (icix > 0) then
         clx=clx(1:icix-1)//ciy(1:lciy)//clx(icix+lcix:)
        end if
        if (clx(1:1) /= '1') then
         lcl0=lclx
         lclx=lclx+len_trim(clx)+1
         clfd(lcl0+1:lclx)=trim(clx)//'*'
        end if

        if (lclx > lclj) then
         clx=clfd(lclj+1:lclx-1)
         do iiy=1,jyd
          write(ind1,'(i16)') pl(jy)%ip(iiy)
          cix='i'//trim(adjustl(ind1))
          lcix=len_trim(cix)
c          icix=index(clx,trim(cix))
c          icix1=index(clx,trim(cix),.true.)
          icxa=index(clx,trim(cix)//',')
          icxb=index(clx,trim(cix)//')')
          if (icxa > 0 .and. icxb > 0) then
           icix=min(icxa,icxb)
          else
           icix=max(icxa,icxb)
          end if
          icix1=max(index(clx,trim(cix)//',',.true.),
     &              index(clx,trim(cix)//')',.true.))
          if (pl(pl(jy)%ip(iiy))%ct == 3) then
           ist=ist+1
           write(ind1,'(i16)') ist
           ciz='j'//trim(adjustl(ind1))
           cst(ist)=trim(ciz)
          else if (pl(pl(jy)%ip(iiy))%ct == 8) then
           iso=iso+1
           write(ind1,'(i16)') iso
           ciz='k'//trim(adjustl(ind1))
           cso(iso)=trim(ciz)
          else
           ciz=''
          end if
          lciz=len_trim(ciz)
          if (lciz > 0) then
           if (icix > 0 .and. icix1 > icix) then
            clx=clx(1:icix-1)//ciz(1:lciz)//clx(icix+lcix:icix1-1)//
     &                         ciz(1:lciz)//clx(icix1+lcix:)
           end if
           call sumdlts(clx,trim(ciz))
          end if
         end do

c         lclx=lclj+1+len_trim(clx)+1
         lclx=lclj+1+len_trim(clx)
         clfd(lclj+1:lclx)=trim(clx)//'*'
        end if

        clx=clfd(1:lclx)

        call sumdlts(clx,trim(ciy))

        lcl1=len_trim(clx)


c        if (clx(lcl1:lcl1) == '*') then
        if (lcl1 > 0) then                  ! 23 July, 2014
         if (clx(lcl1:lcl1) == '*') then    ! 23 July, 2014
          lcl1=lcl1-1
          clx=clx(1:lcl1)
         end if
        end if



        do iiy=1,iyd+jyd
         if (indx(iiy) > 0) then
          write(ind1,'(i16)') indx(iiy)
          cix='i'//trim(adjustl(ind1))
          icix=max(index(clx,trim(cix)//','),
     &             index(clx,trim(cix)//')'))
          if (icix > 0) then
           if (pl(indx(iiy))%ct == 3) then
            call sumdlts(clx,trim(cix))
            if (len_trim(clx) < lcl1-2*len_trim(cix)) then
             lcl1=len_trim(clx)
            else
             ist=ist+1
             write(ind1,'(i16)') ist
             ciz='j'//trim(adjustl(ind1))
             cst(ist)=trim(ciz)
            end if
           else if (pl(indx(iiy))%ct == 8) then
            call sumtmxs(clx,trim(cix))
            if (len_trim(clx) < lcl1-2*len_trim(cix)) then
             lcl1=len_trim(clx)
            else
             iso=iso+1
             write(ind1,'(i16)') iso
             ciz='k'//trim(adjustl(ind1))
             cso(iso)=trim(ciz)
            end if
           end if
          end if
         end if
        end do



        lcl1=len_trim(clx)

        do iiy=1,iso
         call sumtmxs(clx,trim(cso(iiy)))
         if (len_trim(clx) < lcl1-2*len_trim(cso(iiy))) then
          lcl1=len_trim(clx)
         end if
        end do



        if (clx(1:1) == '*') then
         if (lcl1 > 1) then
          clfd=clx(2:lcl1)
          lcl1=lcl1-1
         else
          lcl1=4
          clfd(1:lcl1)='onec'
         end if
        else if (clx(1:1) == ' ') then
         lcl1=4
         clfd(1:lcl1)='onec'
        else
         clfd=clx(1:lcl1)
        end if

c        close(13)

       end subroutine colinf

       subroutine sumdlts(clfx,cx)
c Checks if there are Kronecker deltas, d(cx,cy), present in colour
c factor clfx and removes them by replacing cx with cy

        character(*) clfx,cx
        character(80) cy
        character(1) blfx
        integer :: icx1,icx2,ilp1,ilp2,irp1,irp2,icm1,icm2,lcx,lcy


c        write(26,*) 'Summing ',cx,' in ',trim(clfx)

        blfx=clfx(1:1)

c        icx1=index(clfx,cx)
c        icx2=index(clfx,cx,.true.)
        icxa=index(clfx,trim(cx)//',')
        icxb=index(clfx,trim(cx)//')')
        if (icxa > 0 .and. icxb > 0) then
         icx1=min(icxa,icxb)
        else
         icx1=max(icxa,icxb)
        end if
        icx2=max(index(clfx,trim(cx)//',',.true.),
     &           index(clfx,trim(cx)//')',.true.))
        lcx=len(cx)

        if (icx1 > 0 .and. icx2 > icx1) then
         ilp1=index(clfx(1:icx1),'(',.true.)
         ilp2=index(clfx(1:icx2),'(',.true.)
         if (clfx(ilp1-1:ilp1-1) == 'd') then
          icm1=index(clfx(ilp1+1:icx1),',')
          irp1=index(clfx(ilp1+1:),')')
          if (icm1 > 0) then
           cy=clfx(ilp1+1:ilp1+icm1-1)
           lcy=len_trim(cy)
          else
c           lcy=index(clfx(icm1+1:),')')-1
c           cy=clfx(icm1+1:icm1+lcy)
c           cy=clfx(index(clfx,',')+1:index(clfx,')')-1)
           cy=clfx(ilp1+index(clfx(ilp1:),','):
     &             ilp1+index(clfx(ilp1+1:),')')-1)
           lcy=len_trim(cy)
          end if
          clfx=clfx(1:ilp1-3)//clfx(ilp1+irp1+1:icx2-1)//
     &         trim(cy)//clfx(icx2+lcx:)
         else if (clfx(ilp2-1:ilp2-1) == 'd') then
          icm2=index(clfx(ilp2+1:icx2),',')
          irp2=index(clfx(ilp2+1:),')')
          if (icm2 > 0) then
           cy=clfx(ilp2+1:ilp2+icm2-1)
           lcy=len_trim(cy)
          else
c           lcy=index(clfx(icx2+len(cx)+1:),')')-1
c           cy=clfx(icx2+len(cx)+1:icx2+len(cx)+lcy)
c           cy=clfx(index(clfx,',')+1:index(clfx,')')-1)
           cy=clfx(ilp2+index(clfx(ilp2:),','):
     &             ilp2+index(clfx(ilp2+1:),')')-1)
           lcy=len_trim(cy)
          end if
          clfx=clfx(1:icx1-1)//trim(cy)//clfx(icx1+lcx:ilp2-3)//
     &         clfx(ilp2+irp2+1:)
c         else
         end if

        end if

        if (clfx(1:1) == '*' .and. clfx(1:1) /= blfx) then
         clfx=clfx(2:)
        end if

c        write(26,*) 'Resulted in ',trim(clfx)

       end subroutine sumdlts


       subroutine sumtmxs(clfx,cx)
c Checks if there are two t matrices with the same colour octet index 
c cx in clfx and sums over it

        character(*) clfx,cx
        character(80) cy1,cy2
        integer :: icx1,icx2,ilp1,ilp2,irp1,irp2,icm1,icm2,lcx,lcy1,
     &             lcy2

c        icx1=index(clfx,cx)
c        icx2=index(clfx,cx,.true.)
        icxa=index(clfx,trim(cx)//',')
        icxb=index(clfx,trim(cx)//')')
        if (icxa > 0 .and. icxb > 0) then
         icx1=min(icxa,icxb)
        else
         icx1=max(icxa,icxb)
        end if
        icx2=max(index(clfx,trim(cx)//',',.true.),
     &           index(clfx,trim(cx)//')',.true.))
        lcx=len(cx)

        if (icx1 > 0 .and. icx2 > icx1) then
         ilp1=index(clfx(1:icx1),'(',.true.)
         ilp2=index(clfx(1:icx2),'(',.true.)
         if (clfx(ilp1-1:ilp1-1) == 't' .and. 
     &                            clfx(ilp2-1:ilp2-1) == 't') then
          icm1=index(clfx(ilp1+1:),',')
          irp1=index(clfx(ilp1+1:),')')
          icm2=index(clfx(ilp2+1:),',')
          irp2=index(clfx(ilp2+1:),')')

          lcy1=irp1-icm1-1
          cy1=clfx(ilp1+icm1+1:ilp1+icm1+lcy1)
          lcy2=irp2-icm2-1
          cy2=clfx(ilp2+icm2+1:ilp2+icm2+lcy2)
          clfx=clfx(1:ilp1-1)//'t('//trim(cy1)//','//trim(cy2)//')'//
     &         clfx(ilp1+irp1+1:ilp2-3)//clfx(ilp2+irp2+1:)
         end if

        end if

       end subroutine sumtmxs

       subroutine wrcolrtn(clfx,lclfx)
        integer :: lclfx,ido,ncncl=0
        character(lclfx) clfx
        character(lclfx+6) clfy
        character(80) indcl

        save ncncl

        if (icnclx == icncl0) then
         if (icnclx > 0) then
          write(126,'(a)') ' '
          write(126,'(a)') '      end'
          close(126)
          icncl=1
         end if
         icncl0=icncl0+1
         write(indcl,'(i16)') icncl0
         open(unit=126,file='cfdkk'//trim(adjustl(indcl))//'.f',
     &                            status='replace',action='write')
         write(126,'(a)') '      subroutine cfdkk'//trim(adjustl(indcl))
     &   //trim(cargm)
         write(126,'(a)') ' '
         write(126,'(a)') '      use rcolms'
         write(126,'(a)') ' '
         write(26,'(a)') '      call cfdkk'//trim(adjustl(indcl))
     &   //trim(cargm)
        end if


        ido=0

        do ii=1,ist
         if (max(index(clfx,trim(cst(ii))//','),
     &           index(clfx,trim(cst(ii))//')')) > 0) then
          write(126,900) trim(cst(ii))
          ido=ido+1
         end if
        end do
        do ii=1,iso
         if (max(index(clfx,trim(cso(ii))//','),
     &           index(clfx,trim(cso(ii))//')')) > 0) then
          write(126,905) trim(cso(ii))
          ido=ido+1
         end if
        end do
        write(ind1,'(i16)') idr
        write(126,910) trim(adjustl(ind1)),trim(adjustl(ind1))

        clfy='     &'//trim(clfx)

        call wrt72(126,clfy)

        do ii=1,ido
         write(126,990) 
        end do

        if (ido > 0) then
         ncncl=ncncl+1
         icnclx=ncncl/mxcmdc
         icncl=0
        end if

 900    format('      do ',a,'=1,3')
 905    format('      do ',a,'=1,8')
 910    format('      cfd(',a,')=cfd(',a,')+')
 920    format('     &',a)
 990    format('      end do')

       end subroutine wrcolrtn

       subroutine wrkinrtn(ij,oij,irc)
        integer :: ij,oij,irc,plorx(nexpl),izz,idl
        character(10) clmx,clwx

        read(20,'(26(i3))',rec=irc) plorx
        read(31,'(a10,a10)',rec=irc) clmx,clwx

        idl=0
        do izz=1,nexpl
         if (plorx(izz) > 0) then
          idl=idl+1
         else
          exit
         end if
        end do

        opl(ij,oij)%n=irc
        opl(ij,oij)%dx=idl
        opl(ij,oij)%m=adjustl(clmx)
        opl(ij,oij)%w=adjustl(clwx)

        allocate (opl(ij,oij)%ix(idl),stat=blad)
        allocate (opl(ij,oij)%iy(idl),stat=blad)
        allocate (opl(ij,oij)%ip(size(pl(irc)%ip)),stat=blad)
        opl(ij,oij)%ix=plorx(:idl)
        opl(ij,oij)%iy=plorx(:idl)
        opl(ij,oij)%ip=pl(irc)%ip

        opl(ij,oij)%ig=0
        opl(ij,oij)%ii=0

        do ixx=1,idl
         if (plorx(ixx) > nisp) then
          opl(ij,oij)%ig=opl(ij,oij)%ig+1
         else
          opl(ij,oij)%ii=opl(ij,oij)%ii+1
         end if
        end do

c        if (opl(ij,oij)%ig > mxmap) then
        if (opl(ij,oij)%ig+opl(ij,oij)%ii > mxmap) then
         opl(ij,oij)%im=0
        else
         opl(ij,oij)%im=1
        end if

       end subroutine wrkinrtn

       subroutine kinwrt
        use maxvals
        use psnname
        integer :: lkrn,ipn,icn0=0,icnx=0,icon1

        character(80) ckrn,indxx
        character(10) clmi,clmj,clwi,clwj

        save ckrn,indxx,icn0,icnx,lkrn

        sqsd=''
        sqse=''

        if (iconx == icon0) then
         icon0=icon0+1
         write(ind1,'(i16)') nexpl-2
         write(ind2,'(i16)') icon0
         ckrn='kinkk'//trim(adjustl(ind2))//'.f'
         lkrn=len_trim(ckrn)

         if (icnx == icn0) then
          icn0=icn0+1
          write(indxx,'(i16)') icn0
          psnnm='psub'//trim(adjustl(indxx))
          open(unit=4,file=trim(psnnm)//'.f',
     &                            status='replace',action='write')
          psubend=.false. ! 14 May 2010
          write(4,'(a)') '      subroutine '//trim(psnnm)//
     &                         '(ikin0,ecm,p,x,ndim,dlinv,dlips0)'
          write(4,'(a)') ' '
          write(4,'(a)') '      use partcls'
          if (nexpl > 4) then   ! 17 Jan. 2011
           write(4,'(a)') '      use ntnkin'
          end if                ! 17 Jan. 2011
          write(4,'(a)') ' '
          write(4,'(a)') '      implicit none'
          write(4,'(a)') ' '
          write(4,'(a)') '      integer, parameter :: dp=kind(1.d0),'//
     &                       'imom=0'
          write(4,'(a)') '      integer :: ikin0,ndim,ikin'
          write(4,'(a)') '      real(dp) :: ecm,p(0:3,nepl),x(ndim),'//
     &                     'dlinv,dlips,dlips0,rlips'
          write(4,'(a)') ' '
          close(4)
          write(29,'(a)') '      call '//trim(psnnm)//
     &                         '(ikin0,ecm,p,x,ndim,dlinv,dlips0)'
         end if

         open(unit=27,file=ckrn(1:lkrn),status='replace',action='write')

         write(27,'(a)') '      subroutine '//ckrn(1:lkrn-2)//
     &                   '(ikin,imom,ecm,m,p,x,ndim,dlips)'
         write(27,'(a)') ' '
         write(27,'(a)') '      use inprms'
         write(27,'(a)') '      use kincuts'
         if (nexpl > 4) then   ! 17 Jan. 2011
          write(27,'(a)') '      use ntnkin'
         end if                ! 17 Jan. 2011
         write(27,'(a)') ' '
         write(27,'(a)') '      implicit none'
         write(27,'(a)') ' '
         write(ind1,'(i16)') nexpl
         write(27,'(a)') '      integer, parameter :: nepl='
     &                                           //trim(adjustl(ind1))
         write(27,'(a)') '      integer :: ikin,imom,ndim'
         write(27,'(a)') '      real(dp) :: ecm,m(nepl),p(0:3,nepl),'//
     &                   'x(ndim),dlips,v(3),g'
c         write(27,'(a)') '      real(dp) :: wdm,ats,nfct,rs'
         if (nexpl > 4) then
          write(27,'(a$)') '      real(dp), dimension(0:3) :: '
          do ipn=1,nexpl-5
           write(ind1,'(i16)') ipn
           write(27,'(a$)') 'q'//trim(adjustl(ind1))//','
          end do
          write(ind1,'(i16)') nexpl-4
          write(27,'(a)') 'q'//trim(adjustl(ind1))
         end if
         if (nexpl > 4) then
          write(27,'(a$)') '      real(dp) :: '
          do ipn=1,nexpl-5
           write(ind1,'(i16)') ipn
           write(27,'(a$)') 'sqsl'//trim(adjustl(ind1))//','
          end do
          write(ind1,'(i16)') nexpl-4
          write(27,'(a)') 'sqsl'//trim(adjustl(ind1))
          write(27,'(a$)') '      real(dp) :: '
          do ipn=1,nexpl-5
           write(ind1,'(i16)') ipn
           write(27,'(a$)') 'sqsu'//trim(adjustl(ind1))//','
          end do
          write(ind1,'(i16)') nexpl-4
          write(27,'(a)') 'sqsu'//trim(adjustl(ind1))
          write(27,'(a$)') '      real(dp) :: '
          do ipn=1,nexpl-5
           write(ind1,'(i16)') ipn
           write(27,'(a$)') 'sqs'//trim(adjustl(ind1))//','
          end do
          write(ind1,'(i16)') nexpl-4
          write(27,'(a)') 'sqs'//trim(adjustl(ind1))
          write(27,'(a$)') '      real(dp) :: '
          do ipn=1,nexpl-5
           write(ind1,'(i16)') ipn
           write(27,'(a$)') 'ds'//trim(adjustl(ind1))//','
          end do
          write(ind1,'(i16)') nexpl-4
          write(27,'(a)') 'ds'//trim(adjustl(ind1))
          write(27,'(a$)') '      real(dp) :: '
          do ipn=1,nexpl-5
           write(ind1,'(i16)') ipn
           write(27,'(a$)') 's'//trim(adjustl(ind1))//','
          end do
          write(ind1,'(i16)') nexpl-4
          write(27,'(a)') 's'//trim(adjustl(ind1))
          write(27,'(a$)') '      real(dp) :: '
          do ipn=1,nexpl-5
           write(ind1,'(i16)') ipn
           write(27,'(a$)') 'dl'//trim(adjustl(ind1))//','
          end do
          write(ind1,'(i16)') nexpl-4
          write(27,'(a)') 'dl'//trim(adjustl(ind1))
         end if

         write(27,'(a)') ' '

         if (icon0 > 1) then
          open(unit=274,file='kinwrt1.tmp',status='old',action='read')
          read(274,'(i16)',end=104) icon1
          if (icon1+1 == icon0) then
           do
            read(274,'(a)',end=104) inds
            write(27,'(a)') inds
           end do
          end if
 104      close(274)
         end if

        end if

        if (ntop > ntop0) then             ! 25 Feb. 2010
c Added on 25 Feb. 2010
         if (did > dnk) then             ! 30 Sept. 2010
          open(unit=271,file='kinwrt.tmp',status='old',action='read')
          nkmt=nkmt+1        ! 18 Oct. 2010
          iconx=nkmt/mxcmdb          ! 18 Oct. 2010
          do
           read(271,'(a)',end=100) inds
           write(27,'(a)') trim(inds)
          end do
 100      close(271)
          if (iconx == icon0) then   ! 18 Oct. 2010
           write(27,'(a)') '      end if'
           write(27,'(a)') ' '
           write(27,'(a)') '      end'
           close(27)
           newfle=.true.
          end if                     ! 18 Oct. 2010
         end if                          ! 30 Sept. 2010
c Commented on 5 Oct. 2010:
c          write(ind1,'(i16)') ntop+1
c          write(27,'(a)') '      else if (ikin <= nt'
c     &                            //trim(adjustl(ind1))//') then'
        end if

c        iconx=id/mxcmdb
c        iconx=(ntop+1)/mxcmdb     ! 5 Oct. 2010
ckk> commented on 26 March, 2015     
c        write(*,'(a,i12,a,2i10)')'Diagram',id,'  made of particles:',i,j
ckk<
        read(31,'(a10,a10)',rec=i) clmi,clwi
c        write(*,*)'partcl_i:',trim(clmi),trim(clwi)
        read(31,'(a10,a10)',rec=j) clmj,clwj
c        write(*,*)'partcl_j:',trim(clmj),trim(clwj)
        if (clmi /= clmj) then
c         write(*,*)'checktop: Propagators of different particles joined'
         write(indi,'(i16)') i
         write(indj,'(i16)') j
         opld='(with mixing of particles '//trim(adjustl(indi))//
     &                             ' and '//trim(adjustl(indj))//')'
        end if

        if (nexpl == 4 .and. nisp == 2) then
ckk> 19 March 2010: 27 -> 271
c         open(unit=271,file='kinwrt.tmp',status='replace',
c     &                                          action='write')
         write(ind1,'(i16)') id
         write(27,'(a)') '      if (ikin == '
     &                            //trim(adjustl(ind1))//') then'

         if (e1 == 1 .and. e2 == 1) then
          if (x1(1,3) > 0 .or. x2(1,3) > 0) then
           write(27,'(a)')'       call kinff(imom,ecm,m(3),m(4),'
     &                  //'x(1:2),p(:,3),p(:,4),dlips)'
          else
           if (x1(1,1) > nisp .and. x1(1,2) > nisp .or.
     &         x2(1,1) > nisp .and. x2(1,2) > nisp) then
            write(27,'(a)')'       call kinff(imom,ecm,m(3),m(4),'
     &                   //'x(1:2),p(:,3),p(:,4),dlips)'
           else if (x1(1,1) == 1 .and. x1(1,2) == 3) then
            write(27,'(a)')'       call kintmap(imom,ctc,ecm,'
     &      //trim(adjustl(clmi))//',m(1),m(3),m(4),x(1:2),p(:,1),'
            write(27,'(a)')'     &              p(:,3),p(:,4),dlips)'
           else if (x1(1,1) == 1 .and. x1(1,2) == 4) then
            write(27,'(a)')'       call kintmap(imom,ctc,ecm,'
     &      //trim(adjustl(clmi))//',m(1),m(4),m(3),x(1:2),p(:,1),'
            write(27,'(a)')'     &              p(:,4),p(:,3),dlips)'
           else if (x1(1,1) == 2 .and. x1(1,2) == 3) then
            write(27,'(a)')'       call kintmap(imom,ctc,ecm,'
     &      //trim(adjustl(clmi))//',m(2),m(3),m(4),x(1:2),p(:,2),'
            write(27,'(a)')'     &              p(:,3),p(:,4),dlips)'
           else if (x1(1,1) == 2 .and. x1(1,2) == 4) then
            write(27,'(a)')'       call kintmap(imom,ctc,ecm,'
     &      //trim(adjustl(clmi))//',m(2),m(4),m(3),x(1:2),p(:,2),'
            write(27,'(a)')'     &              p(:,4),p(:,3),dlips)'
           end if
          end if
         else
          write(*,*)'checktop.f: Something must be wrong as it should'//
     &    ' be e1=e2 for nexpl=4',e1,e2
         end if
c         if (ntop > ntop0 .or. ntop0 == 0) then           ! 13 May 2010
          write(27,'(a)') '       if (dlips <= 0.d0) then'
          write(27,'(a)') '        dlips=0.d0'
          write(27,'(a)') '        return'
          write(27,'(a)') '       end if'
          write(27,'(a)') '      end if'
c         end if ! 13 May 2010

c         close(271)

        else if (nexpl > 4) then
         call wrkonix(opli,1)
         call wrkonix(oplj,2)
         isn=0
         isn0=0
         call wrkonopl(opli,1)
         isn0=isn
         call wrkonopl(oplj,2)
         if (isn0 > nexpl-4) then
          write(*,*)'isn0=',isn0,id
         end if

         if (isn0 > 0 .and. isn0 <= nexpl-4) then
          isn1=0  ! Introduced on 27 Jan. 2008
          call wrtss(1,opli)
         end if
         if (nexpl-4 > isn0) then
          isn1=isn0
          call wrtss(2,oplj)
         end if

         if (isn0 > 0 .and. isn0 <= nexpl-4) then
          sqsa(isn0)='ecm'
ckk> Corrected on 10 June 2010:
c          write(ind1,'(i16)') nexpl-4
c          sqsb(isn0)='sqsl'//trim(adjustl(ind1))
          write(ind1,'(i16)') nexpl-4
          if (opl(2,oplj)%s0 > 0) then
           write(ind2,'(i16)') opl(2,oplj)%s0
           sqsb(isn0)='m('//trim(adjustl(ind2))//')'
          else
           sqsb(isn0)='sqsl'//trim(adjustl(ind1))
          end if
ckk<
          sqse(nexpl-3)='sqs'//trim(adjustl(ind1))
          mome(nexpl-3)='q'//trim(adjustl(ind1))
          write(ind1,'(i16)') isn0
          sqsd(nexpl-3)='sqs'//trim(adjustl(ind1))
          momd(nexpl-3)='q'//trim(adjustl(ind1))
          if (isn0 == nexpl-4) then ! added on 20 March, 2008
           if (opl(1,opli)%s0 == 0) then
            write(ind1,'(i16)') opl(2,oplj)%s0
            sqse(nexpl-3)='m('//trim(adjustl(ind1))//')'
            mome(nexpl-3)='p(:,'//trim(adjustl(ind1))//')'
           end if
           if (opl(2,oplj)%s0 == 0) then
            write(ind1,'(i16)') opl(1,opli)%s0
            sqsd(nexpl-3)='m('//trim(adjustl(ind1))//')'
            momd(nexpl-3)='p(:,'//trim(adjustl(ind1))//')'
           end if
          end if
          isn1=0
c         call wrtss(1,opli)
         end if

         if (nexpl-4 > isn0) then
          sqsa(nexpl-4)='ecm'
          if (isn0 > 0) then
           write(ind1,'(i16)') isn0
           sqsb(nexpl-4)='sqs'//trim(adjustl(ind1))
          else if (isn0 == 0 .and. opl(1,opli)%s0 > 0) then
           write(ind1,'(i16)') opl(1,opli)%s0
           sqsb(nexpl-4)='m('//trim(adjustl(ind1))//')'
           sqsd(nexpl-3)='m('//trim(adjustl(ind1))//')'
           momd(nexpl-3)='p(:,'//trim(adjustl(ind1))//')'
           write(ind2,'(i16)') nexpl-4
           sqse(nexpl-3)='sqs'//trim(adjustl(ind2))
           mome(nexpl-3)='q'//trim(adjustl(ind2))
          end if
         end if

         if (isn0 == 0 .and. isn == 0) then      ! Special case: t-channel
          if (nexpl-4 > 0) then                  !(corrected on 15 April 2008)
           sqsa(nexpl-4)='ecm'                   ! topology for a 2 ->  2
          end if                                 ! process
          write(ind1,'(i16)') opl(1,opli)%s0
          sqsd(nexpl-3)='m('//trim(adjustl(ind1))//')'
          momd(nexpl-3)='p(:,'//trim(adjustl(ind1))//')'
          write(ind1,'(i16)') opl(2,oplj)%s0
          sqse(nexpl-3)='m('//trim(adjustl(ind1))//')'
          mome(nexpl-3)='p(:,'//trim(adjustl(ind1))//')'
         end if
         if (isn0 == 0 .and. isn > nexpl-4) then
          if (nexpl-4 > 0) then  ! Corrected on 24 Oct. 2007
           sqsa(nexpl-4)='ecm'
          end if
          if (opl(2,oplj)%i3 == 1) then
           write(ind2,'(i16)') nexpl-4
           sqse(nexpl-3)='sqs'//trim(adjustl(ind2))
           mome(nexpl-3)='q'//trim(adjustl(ind2))
           if (opl(2,oplj)%o3(1) > 0) then
            write(ind1,'(i16)') opl(2,oplj)%o3(1)
            if (nexpl-4 > 0) then  ! Corrected on 24 Oct. 2007
             sqsb(nexpl-4)='m('//trim(adjustl(ind1))//')'
            end if
            sqsd(nexpl-3)='m('//trim(adjustl(ind1))//')'
            momd(nexpl-3)='p(:,'//trim(adjustl(ind1))//')'
           else
            write(ind1,'(i16)') abs(opl(2,oplj)%o3(1))
            if (nexpl-4 > 0) then  ! Corrected on 24 Oct. 2007
             sqsb(nexpl-4)='sqsl'//trim(adjustl(ind1))
            end if
            sqsd(nexpl-3)='sqs'//trim(adjustl(ind1))
            momd(nexpl-3)='q'//trim(adjustl(ind1))
            if (nexpl-4 > 0) then  ! Corrected on 24 Oct. 2007
             write(ind2,'(i16)') nexpl-4
             sqsa(opl(2,abs(opl(2,oplj)%o3(1)))%s)='ecm'
             sqsb(opl(2,abs(opl(2,oplj)%o3(1)))%s)='sqs'//
     &                               trim(adjustl(ind2))
            end if
           end if

           if (opl(2,oplj)%o3(2) < 0) then
            if (nexpl-4 > 0) then  ! Corrected on 24 Oct. 2007
             write(ind1,'(i16)') nexpl-4
             sqsa(opl(2,abs(opl(2,oplj)%o3(2)))%s)='sqs'//
     &                               trim(adjustl(ind1))
             if (opl(2,oplj)%o3(3) < 0) then
c Introduced on 27 Oct. 2007:
              sqsa(opl(2,abs(opl(2,oplj)%o3(3)))%s)='sqs'//
     &                               trim(adjustl(ind1))
              write(ind1,'(i16)') opl(2,abs(opl(2,oplj)%o3(2)))%s
              write(ind2,'(i16)') opl(2,abs(opl(2,oplj)%o3(3)))%s
c Introduced on 30 Oct. 2007:
              sqsd(nexpl-4)='sqs'//trim(adjustl(ind1))
              momd(nexpl-4)='q'//trim(adjustl(ind1))
              sqse(nexpl-4)='sqs'//trim(adjustl(ind2))
              mome(nexpl-4)='q'//trim(adjustl(ind2))
              if (opl(2,abs(opl(2,oplj)%o3(2)))%s <
     &           opl(2,abs(opl(2,oplj)%o3(3)))%s) then
               sqsb(opl(2,abs(opl(2,oplj)%o3(2)))%s)='sqs'//
     &                                trim(adjustl(ind2))
               sqsb(opl(2,abs(opl(2,oplj)%o3(3)))%s)='sqsl'//
     &                                trim(adjustl(ind1))
              else
               sqsb(opl(2,abs(opl(2,oplj)%o3(2)))%s)='sqsl'//
     &                                trim(adjustl(ind2))
               sqsb(opl(2,abs(opl(2,oplj)%o3(3)))%s)='sqs'//
     &                                trim(adjustl(ind1))
              end if
              sqsl(nexpl-4)='sqsl'//trim(adjustl(ind1))//'+'//
     &                     'sqsl'//trim(adjustl(ind2))
              moml(nexpl-4)='q'//trim(adjustl(ind1))//'+'//
     &                     'q'//trim(adjustl(ind2))
             else
              write(ind2,'(i16)') opl(2,oplj)%o3(3)
              sqsb(opl(2,abs(opl(2,oplj)%o3(2)))%s)='m('//
     &                        trim(adjustl(ind2))//')'
c Introduced on 27 Oct. 2007:
              write(ind1,'(i16)') opl(2,abs(opl(2,oplj)%o3(2)))%s
              sqsl(nexpl-4)='sqsl'//trim(adjustl(ind1))//'+'//
     &                     'm('//trim(adjustl(ind2))//')'
              moml(nexpl-4)='q'//trim(adjustl(ind1))//'+p(:,'
     &                     //trim(adjustl(ind2))//')'
c Introduced on 30 Oct. 2007:
              sqsd(nexpl-4)='sqs'//trim(adjustl(ind1))
              momd(nexpl-4)='q'//trim(adjustl(ind1))
              sqse(nexpl-4)='m('//trim(adjustl(ind2))//')'
              mome(nexpl-4)='p(:,'//trim(adjustl(ind2))//')'
             end if
            end if
           else
c Introduced on 27 Oct. 2007:
            write(ind1,'(i16)') opl(2,oplj)%o3(2)
            if (nexpl-4 > 0) then 
             if (opl(2,oplj)%o3(3) < 0) then
              write(ind2,'(i16)') opl(2,abs(opl(2,oplj)%o3(3)))%s
              sqsl(nexpl-4)='m('//trim(adjustl(ind1))//')+sqsl'//
     &                      trim(adjustl(ind2))
              moml(nexpl-4)='p(:,'//trim(adjustl(ind1))//')+q'//
     &                      trim(adjustl(ind2))
c Introduced on 30 Oct. 2007:
              sqsd(nexpl-4)='m('//trim(adjustl(ind1))//')'
              momd(nexpl-4)='p(:,'//trim(adjustl(ind1))//')'
              sqse(nexpl-4)='sqs'//trim(adjustl(ind2))
              mome(nexpl-4)='q'//trim(adjustl(ind2))
             else
              write(ind2,'(i16)') opl(2,oplj)%o3(3)
              sqsl(nexpl-4)='m('//trim(adjustl(ind1))//')+m('//
     &                      trim(adjustl(ind2))//')'
              moml(nexpl-4)='p(:,'//trim(adjustl(ind1))//')+p(:,'//
     &                      trim(adjustl(ind2))//')'
              if (expl(opl(2,oplj)%o3(2)) == 'n' .and.
     &            expl(opl(2,oplj)%o3(3)) == 'n') nuga(nexpl-4)=1 ! 16 Nov, 2010
c Introduced on 30 Oct. 2007:
              sqsd(nexpl-4)='m('//trim(adjustl(ind1))//')'
              momd(nexpl-4)='p(:,'//trim(adjustl(ind1))//')'
              sqse(nexpl-4)='m('//trim(adjustl(ind2))//')'
              mome(nexpl-4)='p(:,'//trim(adjustl(ind2))//')'
             end if
            end if
           end if
c Commented on 30 Oct. 2007:
c          if (opl(2,oplj)%o3(3) < 0) then
c           if (opl(2,abs(opl(2,oplj)%o3(3)))%s0 == 0) then
c Commented on 28 Oct. 2007:
c            write(ind2,'(i16)') opl(2,abs(opl(2,oplj)%o3(3)))%s
c            sqsa(opl(2,abs(opl(2,oplj)%o3(3)))%s)='ecm'
c           else
c            write(ind2,'(i16)') opl(2,abs(opl(2,oplj)%o3(3)))%s0
c            sqsd(nexpl-3)='m('//trim(adjustl(ind2))//')'
c            momd(nexpl-3)='p(:,'//trim(adjustl(ind2))//')'
c           end if
c          end if
          else
           if (opl(2,oplj)%o2(1) < 0) then
            sqsa(opl(2,abs(opl(2,oplj)%o2(1)))%s)='ecm'
            if (opl(2,abs(opl(2,oplj)%o2(1)))%s0 == 0) then
             write(ind1,'(i16)') opl(2,abs(opl(2,oplj)%o2(1)))%s
             sqsd(nexpl-3)='sqs'//trim(adjustl(ind1))
             momd(nexpl-3)='q'//trim(adjustl(ind1))
             if (opl(2,oplj)%o2(2) < 0) then
              if (opl(2,abs(opl(2,oplj)%o2(2)))%s0 == 0) then
               write(ind2,'(i16)') opl(2,abs(opl(2,oplj)%o2(2)))%s
               if (opl(2,abs(opl(2,oplj)%o2(1)))%s < 
     &            opl(2,abs(opl(2,oplj)%o2(2)))%s) then
                sqsb(opl(2,abs(opl(2,oplj)%o2(1)))%s)='sqs'//
     &             trim(adjustl(ind2))
                sqsb(opl(2,abs(opl(2,oplj)%o2(2)))%s)='sqsl'//
     &             trim(adjustl(ind1))
c Introduced on 31 Oct. 2007
c               sqsb(nexpl-4)='sqsl'//trim(adjustl(ind1))
               else
                sqsb(opl(2,abs(opl(2,oplj)%o2(1)))%s)='sqsl'//
     &             trim(adjustl(ind2))
                sqsb(opl(2,abs(opl(2,oplj)%o2(2)))%s)='sqs'//
     &             trim(adjustl(ind1))
c Introduced on 31 Oct. 2007
c               sqsb(nexpl-4)='sqsl'//trim(adjustl(ind2))
               end if
c              sqsb(nexpl-4)='sqsl'//trim(adjustl(ind2))
              else
               write(ind2,'(i16)') opl(2,abs(opl(2,oplj)%o2(2)))%s0
               if (nexpl-4 > 0) then  ! Corrected on 24 Oct. 2007
                sqsb(nexpl-4)='m('//trim(adjustl(ind2))//')'
               end if
              end if
             else
              write(ind2,'(i16)') opl(2,oplj)%o2(2)
              if (nexpl-4 > 0) then  ! Corrected on 24 Oct. 2007
               sqsb(nexpl-4)='m('//trim(adjustl(ind2))//')'
              end if
             end if
            else
             write(ind1,'(i16)') opl(2,abs(opl(2,oplj)%o2(1)))%s0
             sqsd(nexpl-3)='m('//trim(adjustl(ind1))//')'
             momd(nexpl-3)='p(:,'//trim(adjustl(ind1))//')'
             if (opl(2,oplj)%o2(2) < 0) then
              if (nexpl-4 > 0) then  ! Corrected on 24 Oct. 2007
               sqsb(nexpl-4)='m('//trim(adjustl(ind1))//')'
              end if
             end if
            end if
           else
            write(ind1,'(i16)') opl(2,oplj)%o2(1)
            sqsd(nexpl-3)='m('//trim(adjustl(ind1))//')'
            momd(nexpl-3)='p(:,'//trim(adjustl(ind1))//')'
            if (opl(2,oplj)%o2(2) < 0) then
             if (nexpl-4 > 0) then  ! Corrected on 24 Oct. 2007
              sqsb(nexpl-4)='m('//trim(adjustl(ind1))//')'
             end if
            end if
           end if
           if (opl(2,oplj)%o2(2) < 0) then
            sqsa(opl(2,abs(opl(2,oplj)%o2(2)))%s)='ecm'
            if (opl(2,abs(opl(2,oplj)%o2(2)))%s0 == 0) then
             write(ind2,'(i16)') opl(2,abs(opl(2,oplj)%o2(2)))%s
             sqse(nexpl-3)='sqs'//trim(adjustl(ind2))
             mome(nexpl-3)='q'//trim(adjustl(ind2))
            else
             write(ind2,'(i16)') opl(2,abs(opl(2,oplj)%o2(2)))%s0
             sqse(nexpl-3)='m('//trim(adjustl(ind2))//')'
             mome(nexpl-3)='p(:,'//trim(adjustl(ind2))//')'
            end if
           else
            write(ind2,'(i16)') opl(2,oplj)%o2(2)
            sqse(nexpl-3)='m('//trim(adjustl(ind2))//')'
            mome(nexpl-3)='p(:,'//trim(adjustl(ind2))//')'
           end if
          end if
         end if
         if (ntop > ntop0 .or. ntop0 == 0) linwr=.true.! 14 May 2010
c Commented on 5 Oct. 2010
c         if (icony == icony0) then                     ! 14 May 2010
c          linwr=.true.
c          icony0=icony0+1
c         end if
c         icony=iconx

ckk>
         if (ntop > ntop0) then
          did=0
          dnk=0
         end if
         ind160=''
         nkm1=nkm
         nkm=nkm+1
         did=did+1
         do ixx=1,nexpl-4
          write(ind1,'(i16)') imap(ixx)
          write(ind160,'(a$)') trim(adjustl(ind160))//
     &    trim(adjustl(sqsl(ixx)))//trim(adjustl(ind1))//
     &    trim(adjustl(mapm(ixx)))
         end do
         open(273,file='kindff.txt',access='direct',form='formatted',
     &                                                     recl=160)
         if (nkm == 1) then
          write(273,'(a)',rec=nkm) trim(adjustl(ind160))
         else 
          do ixx=nkm-1,1,-1
           read(273,'(a)',rec=ixx) clkin
           if (trim(clkin) == trim(ind160)) then
            write(*,*)'Kinematics for diagram',id,' already generated'
            nkm=nkm-1
            dnk=dnk+1
            exit
           else if (ixx == 1) then
c            nkm=nkm+1
            write(273,'(a)',rec=nkm) trim(adjustl(ind160))
           end if
          end do
         end if
         close(273)

c         if (nkm > nkm1) then             ! 30 Sept. 2010

         if (iconx == icon0) then
          open(unit=274,file='kinwrt1.tmp',status='replace',
     &                                          action='write')
          if (nkm > nkm1) then
           write(274,'(i16)') icon0
           call swrtx(274)
          end if
          close(274)

          open(unit=4,file=trim(psnnm)//'.f',
     &           status='old',position='append',action='write')

          write(ind1,'(i16)') ntop

          if (icon0 == 1) then
           write(28,'(a)') '      if (ikin <= nt'//
     &                       trim(adjustl(ind1))//') then'
           write(4,'(a)') '      do ikin=1,nt'//trim(adjustl(ind1))
           indx0=trim(adjustl(ind1))
          else
           write(28,'(a)') '      else if (ikin <= nt'
     &                                  //trim(adjustl(ind1))//') then'
           write(4,'(a)') '      do ikin=nt'//trim(adjustl(indx0))//
     &                   '+1'//',nt'//trim(adjustl(ind1))
           indx0=trim(adjustl(ind1))
          end if
          write(28,'(a)') '       call '//ckrn(1:lkrn-2)//
     &            '(ikin,imom,ecm,m,p,x,ndim,dlips)'
          kinclse=.false.
          write(4,'(a)') '       if (aw(ikin) > 0.d0) then'
          write(4,'(a)') '        call '//ckrn(1:lkrn-2)//
     &                   '(ikin,imom,ecm,m,p,x,ndim,dlips)'
          write(4,'(a)') '        if (dlips > 0.d0) then'
          write(4,'(a)') '         if (ikin == ikin0) then'
          write(4,'(a)') '          rlips=abs(dlips-dlips0)/dlips'
          write(4,'(a)') '          if (rlips > 1.d-3) then'
          write(4,'(a)') '           iwkin(ikin)=iwkin(ikin)+1'
          write(4,'(a)') '           if (rlips > aw_w(ikin,1)) then'
          write(4,'(a)') '            aw_w(ikin,1)=rlips'
          write(4,'(a)') '            aw_w(ikin,2)=dlips0'
          write(4,'(a)') '            aw_w(ikin,3)=dlips'
          write(4,'(a)') '           end if'
          write(4,'(a)') '          end if'
          write(4,'(a)') '          dlips=dlips0'
          write(4,'(a)') '         end if'
          write(4,'(a)') '         dlinv=dlinv+aw(ikin)/dlips'
          write(4,'(a)') '        end if'
          write(4,'(a)') '       end if'
          write(4,'(a)') '      end do'

          close(4)

          idobl=idobl+1

          icnx=idobl/mxcmdb

          if (icnx == icn0) then
           open(unit=4,file=trim(psnnm)//'.f',
     &           status='old',position='append',action='write')
           write(4,'(a)') ' '
           write(4,'(a)') '      end subroutine '//trim(psnnm)
           psubend=.true. ! 14 May 2010
           close(4)
          end if

         else
          kinclse=.true.
          write(ind1,'(i16)') ntop+1
          if (icon0 == 1) then
           kcls1='      if (ikin <= nt'
     &                                  //trim(adjustl(ind1))//') then'
          else
           kcls1='      else if (ikin <= nt'
     &                                  //trim(adjustl(ind1))//') then'
          end if
          kcls2='       call '//ckrn(1:lkrn-2)//
     &            '(ikin,imom,ecm,m,p,x,ndim,dlips)'
          if (did > dnk) then
           call swrtx(27)
          end if
         end if

         write(ind1,'(i16)') ntop+1
         if (icon0 == 1) then
          psub1='      do ikin=1,nt'//trim(adjustl(ind1))
         else
          psub1='      do ikin=nt'//trim(adjustl(indx0))//'+1,nt'//
     &                                             trim(adjustl(ind1))
         end if
         psub2='        call '//ckrn(1:lkrn-2)//
     &                   '(ikin,imom,ecm,m,p,x,ndim,dlips)'

c         end if   ! 30 Sept. 2010

         if (ntop > ntop0 .or. ntop0 == 0) then           ! 25 Feb. 2010
          open(unit=271,file='kinwrt.tmp',status='replace',
     &                                                 action='write')
          iyy=nexpl-3
          do ixx=1,nexpl-4
           write(ind1,'(i16)') iyy
           write(ind2,'(i16)') iyy+1
           write(ind3,'(i16)') ixx
           write(271,'(a)') '       call kinff(imom,'//trim(sqsc(ixx))
     &     //','//trim(sqsd(ixx))//','//trim(sqse(ixx))//',x('//
     &     trim(adjustl(ind1))//':'//trim(adjustl(ind2))//'),'//
     &     trim(momd(ixx))//','//trim(mome(ixx))//
     &      ','//trim(dlip(ixx))//')'
           iyy=iyy+2
          end do
          write(ind1,'(i16)') iyy
          write(ind2,'(i16)') iyy+1
          write(271,'(a)') '       call kinff(imom,ecm'//','//
     &     trim(sqsd(nexpl-3))//','//trim(sqse(nexpl-3))//',x('//
     &     trim(adjustl(ind1))//':'//trim(adjustl(ind2))//'),'//
     &     trim(momd(nexpl-3))//','//trim(mome(nexpl-3))//
     &     ',dlips)'

          if (nexpl-4 > 0) then
           write(271,'(a$)') '       dlips=dlips'
 
           do ixx=1,nexpl-5
            write(ind1,'(i16)') ixx
            write(271,'(a$)') '*ds'//trim(adjustl(ind1))
           end do
           write(ind1,'(i16)') nexpl-4
           write(271,'(a)') '*ds'//trim(adjustl(ind1))
           write(271,'(a$)') '     &            '

           do ixx=1,nexpl-5
            write(ind1,'(i16)') ixx
            write(271,'(a$)') '*dl'//trim(adjustl(ind1))
           end do
           write(ind1,'(i16)') nexpl-4
           write(271,'(a)') '*dl'//trim(adjustl(ind1))
          end if
         end if  ! 25 Feb. 2010
        end if ! nexpl == 4

        if (nexpl-4 > 0) then ! 13 May 2010
         if (ntop > ntop0 .or. ntop0 == 0) then           ! 25 Feb. 2010
          write(271,'(a)') '       if (dlips <= 0.d0) then'
          write(271,'(a)') '        dlips=0.d0'
          write(271,'(a)') '        return'
          write(271,'(a)') '       end if'
c         if (nexpl-4 > 0) then ! 13 May 2010
          write(271,'(a)')
     &            '       if (imom == 1 .and. dlips > 0.d0) then'
c         end if ! 13 May 2010
         end if ! 25 Feb. 2010
        end if ! 13 May 2010
        do ixx=1,nexpl-4
         if (ntop > ntop0 .or. ntop0 == 0) then           ! 25 Feb. 2010
          write(ind1,'(i16)') ixx
          write(271,'(a)') '        v=q'//trim(adjustl(ind1))//'(1:)/q'
     &                                  //trim(adjustl(ind1))//'(0)'
c          write(27,'(a)') '        e=sqrt(s'//trim(adjustl(ind1))//'+'//
c     &    'v(1)*v(1)+v(2)*v(2)+v(3)*v(3))'
c          write(27,'(a)') '        v=v/e'
c          write(27,'(a)') '        g=e/'//trim(sqsc(ixx))
          write(271,'(a)') '        g=q'//trim(adjustl(ind1))//'(0)/'
     &                                //trim(sqsc(ixx))
         end if ! 25 Feb. 2010
         do iyy=1,momb(ixx)%d
          do izz=ixx-1,1,-1
c           if (trim(sqsa(izz)) == 'ecm') then
           if (trim(sqsa(izz)) == trim(sqsa(ixx))) then
            do iww=1,momb(izz)%d
             if (momb(izz)%p(iww) == momb(ixx)%p(iyy)) then
              momb(ixx)%p(iyy)=0
             end if
            end do
           end if
          end do
          if (momb(ixx)%p(iyy) > nisp) then
           write(ind1,'(i16)') momb(ixx)%p(iyy)
           ind2='p(:,'//trim(adjustl(ind1))//')'
           if (trim(momd(nexpl-3)) == trim(ind2)) then ! added on 20 March, 2008
           else if (trim(mome(nexpl-3)) == trim(ind2)) then ! added on 
                                                            ! 20 March, 2008
           else 
            if (ntop > ntop0 .or. ntop0 == 0) then           ! 25 Feb. 2010
             write(271,'(a)') '        call boost(g,v,'//
     &                                        trim(adjustl(ind2))//')'
            end if  ! 25 Feb. 2010
           end if
          end if
         end do
        end do
        do ixx=1,nexpl-4
         deallocate (momb(ixx)%p,stat=blad)
        end do
        if (nexpl-4 > 0) then
         if (ntop > ntop0 .or. ntop0 == 0) then           ! 25 Feb. 2010
          write(271,'(a)') '       end if'
          close(271)   ! 25 Feb. 2010
          ntop0=ntop0+1
         end if ! 25 Feb. 2010
        end if

c        if (iconx == icon0) then
c Added on 25 Feb. 2010
c         if (did > dnk) then     ! 30 Sept. 2010
c          open(unit=271,file='kinwrt.tmp',status='old',action='read')
c          write(27,*) 'Tu 2'
c          do
c           read(271,'(a)',end=101) inds
c           write(27,'(a)') inds
c          end do
c 101      close(271)
c         end if                  ! 30 Sept. 2010
c         write(27,'(a)') '      end if'
c         write(27,'(a)') ' '
c         write(27,'(a)') '      end'
c         close(27)
c        end if


        do ixx=1,opli
         deallocate (opl(1,ixx)%ix,stat=blad)
         deallocate (opl(1,ixx)%iy,stat=blad)
         deallocate (opl(1,ixx)%ip,stat=blad)
        end do
        do ixx=1,oplj
         deallocate (opl(2,ixx)%ix,stat=blad)
         deallocate (opl(2,ixx)%iy,stat=blad)
         deallocate (opl(2,ixx)%ip,stat=blad)
        end do

       end subroutine kinwrt



       subroutine swrtx(nut)
        integer :: nut
          if (linwr) then
           if (newfle .or. iconx == icon0) then           ! > 5 Oct. 2010
            write(ind1,'(i16)') ntop+1
            write(nut,'(a)') '      if (ikin <= nt'
     &                            //trim(adjustl(ind1))//') then'
            newfle=.false.     !  < 5 Oct. 2010
           else
            write(ind1,'(i16)') ntop+1
            write(nut,'(a)') '      else if (ikin <= nt'
     &                            //trim(adjustl(ind1))//') then'
           end if                          !  5 Oct. 2010
           do ixx=1,nexpl-4
            write(ind1,'(i16)') ixx
            write(nut,'(a)') '       sqsl'//trim(adjustl(ind1))//'='
     &            //trim(sqsl(ixx))
           end do
           if (nexpl-4 > 0) then
            write(nut,'(a)') '       if (imom /= 1) then'
           end if
           do ixx=1,nexpl-4
            write(ind1,'(i16)') ixx
            write(nut,'(a)') '        q'//trim(adjustl(ind1))//'='//
     &                                   trim(moml(ixx))
           end do
           if (nexpl-4 > 0) then
            write(nut,'(a)') '       end if'
           end if
           linwr=.false.
          end if                           ! 25 Feb. 2010

         if (nkm > nkm1) then             ! 30 Sept. 2010

c         write(ind1,'(i16)') id      ! added on 25 Feb. 2010
          write(ind1,'(i16)') nkm      ! 5 Oct. 2010
          write(nut,'(a)') '       if (ikin == '//trim(adjustl(ind1))//
     &                   ') then'
c          write(nut,*)'id=',id
          do ixx=isn0,1,-1
           write(ind1,'(i16)') ixx
           write(nut,'(a)')'       sqsu'//trim(adjustl(ind1))//'='//
     &         trim(sqsa(ixx))//'-'//trim(sqsb(ixx))
           call swrt(nut,adjustl(ind1),len_trim(adjustl(ind1)))
          end do
          do ixx=nexpl-4,isn0+1,-1
           write(ind1,'(i16)') ixx
           write(nut,'(a)')'       sqsu'//trim(adjustl(ind1))//'='//
     &         trim(sqsa(ixx))//'-'//trim(sqsb(ixx))
           call swrt(nut,adjustl(ind1),len_trim(adjustl(ind1)))
          end do
          write(nut,'(a)') '       end if' ! added on 25 Feb. 2010

         end if
       end subroutine swrtx

       subroutine swrt(lut,cinx,linx)
        integer :: lut,linx!,isnx
        character(linx) :: cinx

        if (imap(ixx) == 0) then
         write(lut,'(a)') '       ds'//cinx//'=(sqsu'//cinx//'-sqsl'
     &     //cinx//')*(sqsu'//cinx//'+sqsl'//cinx//')'
         write(lut,'(a)') '       if (imom == 1) then'
         write(lut,'(a)') '        s'//cinx//'=ds'//cinx//'*x('//cinx//
     &                   ')'//'+sqsl'//cinx//'*sqsl'//cinx
         write(lut,'(a)') '       else'
         write(lut,'(a)') '        s'//cinx//'=(q'//cinx//'(0)-q'//cinx
     &    //'(3))*(q'//cinx//'(0)+q'//cinx//'(3))-q'//cinx//'(1)*q'
     &    //cinx//'(1)-q'//cinx//'(2)*q'//cinx//'(2)'
         write(lut,'(a)') '       end if'
         write(lut,'(a)') '       sqs'//cinx//'=sqrt(s'//cinx//')'
        else if (imap(ixx) == 1) then
ckk>
c Changed on 1 April, 2008: 
c         write(lut,'(a)') '       call mapskk(imom,x('//cinx//'),q'
c     &   //cinx//','//trim(mapm(ixx))//','//trim(mapw(ixx))//
c     &   ',sqsl'//cinx//',sqsu'//cinx//',s'//cinx//',sqs'//cinx//
c     &   ',ds'//cinx//')'
c mappings of massless propagators done only for gluons and photons:
         if (trim(mapm(ixx)) == 'ma' .and. nuga(ixx) /= 1) then
          write(lut,'(a)') '       call mapakk(imom,x('//cinx//'),q'
     &    //cinx//',sqscut'//
     &    ',sqsl'//cinx//',sqsu'//cinx//',s'//cinx//',sqs'//cinx//
     &    ',ds'//cinx//')'
         else if (trim(mapm(ixx)) == 'mg') then
          write(lut,'(a)') '       call mapakk(imom,x('//cinx//'),q'
     &    //cinx//',sqscut'//
     &    ',sqsl'//cinx//',sqsu'//cinx//',s'//cinx//',sqs'//cinx//
     &    ',ds'//cinx//')'
         else
          write(lut,'(a)') '       call mapbkk(imom,x('//cinx//'),q'
     &    //cinx//','//trim(mapm(ixx))//','//trim(mapw(ixx))//
     &    ',sqsl'//cinx//',sqsu'//cinx//',s'//cinx//',sqs'//cinx//
     &    ',ds'//cinx//')'
         end if
ckk<
        else
         write(*,*) 'checktop.f: Wrong value of imap(',ixx,')=',
     &               imap(ixx)
        end if

       end subroutine swrt


       subroutine wrkonix(oplx,ox)

        integer :: oplx,ox,iqq,ivv,izz,iix

        do ixx=oplx,1,-1
         write(*,'(i3$)') ixx
         write(*,'(a$)') ':'
         do iix=1,opl(ox,ixx)%dx
          write(*,'(i3$)') opl(ox,ixx)%ix(iix)
         end do
         write(*,'(a$)') ' '//trim(opl(ox,ixx)%m)//' '
     &                      //trim(opl(ox,ixx)%w)
         do iix=1,size(pl(opl(ox,ixx)%n)%ip)
          write(*,'(i10$)') opl(ox,ixx)%ip(iix)
         end do
         write(*,'(i10)') opl(ox,ixx)%n
         do izz=size(opl(ox,ixx)%ip),1,-1
          do ivv=ixx-1,1,-1
           if (opl(ox,ixx)%ip(izz) == opl(ox,ivv)%n) then
            do iyy=1,opl(ox,ixx)%dx
             do iqq=1,opl(ox,ivv)%dx
              if (opl(ox,ixx)%ix(iyy) == opl(ox,ivv)%ix(iqq)) then
               if (iqq == opl(ox,ivv)%dx) then
                opl(ox,ixx)%ix(iyy)=-ivv
               else
                opl(ox,ixx)%ix(iyy)=0
               end if
              end if
             end do
            end do
           end if
          end do
         end do
c         write(*,*) ixx,':',opl(ox,ixx)%ix
        end do

        if (ox == 2) then
         if (len_trim(oplc) > 0) then
          write(*,'(a)') oplc
         else if (len_trim(opld) > 0) then
          write(*,'(a)') opld
         end if
        end if

       end subroutine wrkonix

       subroutine wrkonopl(oplx,ox)

        integer :: oplx,ox,opl2(2),opl3(3)

        do ixx=1,oplx
         isn2=0
         isn3=0
         opl(ox,ixx)%is=0
         opl(ox,ixx)%s0=0
         opl(ox,ixx)%o2=0
         opl(ox,ixx)%o3=0
         opl(ox,ixx)%i3=0
         do iyy=1,opl(ox,ixx)%dx
          if (opl(ox,ixx)%ix(iyy) > nisp) then
           opl(ox,ixx)%is=opl(ox,ixx)%is+1
           isn2=isn2+1
           opl(ox,ixx)%s0=opl(ox,ixx)%ix(iyy)
           opl2(isn2)=opl(ox,ixx)%ix(iyy)
           isn3=isn3+1
           opl3(isn3)=opl(ox,ixx)%ix(iyy)
           if (isn2 == 2) then
            isn=isn+1
            isn2=0
            opl(ox,ixx)%o2=opl2
            opl2=0
c            opl(ox,ixx)%is=0
            opl(ox,ixx)%s0=0
            if (isn <= nexpl-4) then
             allocate (momb(isn)%p(opl(ox,ixx)%ig),stat=blad)
             momb(isn)%d=opl(ox,ixx)%ig
             izz=0
             do iww=1,opl(ox,ixx)%dx
              if (opl(ox,ixx)%iy(iww) > nisp) then
               izz=izz+1
               momb(isn)%p(izz)=opl(ox,ixx)%iy(iww)
              end if
             end do
            end if
           end if
          else if (opl(ox,ixx)%ix(iyy) < 0) then
           if (opl(ox,abs(opl(ox,ixx)%ix(iyy)))%is > 0) then
            opl(ox,ixx)%is=opl(ox,ixx)%is
     &                    +opl(ox,abs(opl(ox,ixx)%ix(iyy)))%is
            opl(ox,ixx)%s0=opl(ox,abs(opl(ox,ixx)%ix(iyy)))%s0
            isn2=isn2+1
            opl2(isn2)=opl(ox,ixx)%ix(iyy)
            isn3=isn3+1
            opl3(isn3)=opl(ox,ixx)%ix(iyy)
            if (isn2 == 2) then
             isn=isn+1
             isn2=0
             opl(ox,ixx)%o2=opl2
             opl2=0
c             opl(ox,ixx)%is=0
             opl(ox,ixx)%s0=0
             if (isn <= nexpl-4) then
              allocate (momb(isn)%p(opl(ox,ixx)%ig),stat=blad)
              momb(isn)%d=opl(ox,ixx)%ig
              izz=0
              do iww=1,opl(ox,ixx)%dx
               if (opl(ox,ixx)%iy(iww) > nisp) then
                izz=izz+1
                momb(isn)%p(izz)=opl(ox,ixx)%iy(iww)
               end if
              end do
             end if
            end if
           end if
          end if
         end do
         if (isn3 == 3) then
          isn=isn+1
          opl(ox,ixx)%o3=opl3
          opl(ox,ixx)%i3=1
c          opl(ox,ixx)%is=0
          opl(ox,ixx)%s0=0
          if (isn <= nexpl-4) then
           allocate (momb(isn)%p(opl(ox,ixx)%ig),stat=blad)
           momb(isn)%d=opl(ox,ixx)%ig
           izz=0
           do iww=1,opl(ox,ixx)%dx
            if (opl(ox,ixx)%iy(iww) > nisp) then
             izz=izz+1
             momb(isn)%p(izz)=opl(ox,ixx)%iy(iww)
             if (opl3(3) > 0) then
              if (opl(ox,ixx)%iy(iww) == opl3(3)) then
               if (isn > 1) then ! added on 20 March, 2008
                momb(isn-1)%p(izz)=0
               end if
              end if
             else if (opl3(3) < 0) then
              do ivv=1,opl(ox,abs(opl3(3)))%dx
               if (opl(ox,ixx)%iy(iww) == opl(ox,abs(opl3(3)))%iy(ivv)) 
     &                                                             then
                if (isn > 1) then ! added on 20 March, 2008
                 momb(isn-1)%p(izz)=0
                end if
               end if
              end do
             end if
            end if
           end do
          end if
         end if
         if (isn > nexpl-4) then
          opl(ox,ixx)%s=nexpl-4
         else
          opl(ox,ixx)%s=isn
         end if
         if (opl(ox,ixx)%i3 == 1) then
          do iyy=1,3
c           if (opl(ox,ixx)%o3(iyy) < 0 .and. 
c     &              opl(ox,abs(opl(ox,ixx)%o3(iyy)))%s == 0) then
           if (opl(ox,ixx)%o3(iyy) < 0) then  ! 26 May, 2010
            if(opl(ox,abs(opl(ox,ixx)%o3(iyy)))%s == 0) then
             opl(ox,ixx)%o3(iyy)=opl(ox,abs(opl(ox,ixx)%o3(iyy)))%s0
            end if
           end if
          end do
         else
          do iyy=1,2
c           if (opl(ox,ixx)%o2(iyy) < 0 .and. 
c     &              opl(ox,abs(opl(ox,ixx)%o2(iyy)))%s == 0) then
           if (opl(ox,ixx)%o2(iyy) < 0) then  ! 26 May, 2010
            if (opl(ox,abs(opl(ox,ixx)%o2(iyy)))%s == 0) then
             opl(ox,ixx)%o2(iyy)=opl(ox,abs(opl(ox,ixx)%o2(iyy)))%s0
            end if
           end if
          end do
         end if

c         if (opl(ox,ixx)%i3 == 1) then
c          write(*,*) ixx,isn,opl(ox,ixx)%s0,opl(ox,ixx)%is,':',
c     &       opl(ox,ixx)%o3,' ',
c     &       trim(opl(ox,ixx)%m),' ',trim(opl(ox,ixx)%w)
c         else
c          write(*,*) ixx,isn,opl(ox,ixx)%s0,opl(ox,ixx)%is,':',
c     &       opl(ox,ixx)%o2,' ',
c     &       trim(opl(ox,ixx)%m),' ',trim(opl(ox,ixx)%w)
c         end if

        end do

       end subroutine wrkonopl

       subroutine wrtss(ox,oplx)

        integer :: ox,oplx

        imap(isn1+1:)=0
        mapm(isn1+1:)=''
        mapw(isn1+1:)=''

        do ixx=oplx,1,-1

        if (opl(ox,ixx)%s <= nexpl-4 .and. opl(ox,ixx)%s > 0) then
         if (opl(ox,ixx)%i3 == 1) then
          write(ind0,'(i16)') opl(ox,ixx)%s
          if (opl(ox,ixx)%s > 1) then ! added on 20 March, 2008
           sqsa(opl(ox,ixx)%s-1)='sqs'//trim(adjustl(ind0))
          end if ! added on 20 March, 2008
          if (opl(ox,ixx)%o3(3) > 0) then
           write(ind3,'(i16)') opl(ox,ixx)%o3(3)
           if (opl(ox,ixx)%s > 1) then ! added on 20 March, 2008
            sqsb(opl(ox,ixx)%s-1)='m('//trim(adjustl(ind3))//')'
           end if ! added on 20 March, 2008
          else if (opl(ox,ixx)%o3(3) < 0) then
           write(ind3,'(i16)') opl(ox,abs(opl(ox,ixx)%o3(3)))%s
           if (opl(ox,ixx)%s > 1) then ! added on 20 March, 2008
            sqsb(opl(ox,ixx)%s-1)='sqsl'//trim(adjustl(ind3))
           end if ! added on 20 March, 2008
          end if
          write(ind4,'(i16)') opl(ox,ixx)%s-1
          if (opl(ox,ixx)%o3(1) > 0) then
           write(ind1,'(i16)') opl(ox,ixx)%o3(1)
           if (opl(ox,ixx)%o3(2) > 0) then
            write(ind2,'(i16)') opl(ox,ixx)%o3(2)
            if (opl(ox,ixx)%s > 1) then ! added on 20 March, 2008
             sqsl(opl(ox,ixx)%s-1)='m('//trim(adjustl(ind1))//')+m('
     &            //trim(adjustl(ind2))//')'
             moml(opl(ox,ixx)%s-1)='p(:,'//trim(adjustl(ind1))//')+p(:,'
     &            //trim(adjustl(ind2))//')'
             if (expl(opl(ox,ixx)%o3(1)) == 'n' .and.  ! 16 Nov, 2010
     &           expl(opl(ox,ixx)%o3(2)) == 'n') nuga(opl(ox,ixx)%s-1)=1
             sqsc(opl(ox,ixx)%s-1)='sqs'//trim(adjustl(ind4))
             dlip(opl(ox,ixx)%s-1)='dl'//trim(adjustl(ind4))
             sqsd(opl(ox,ixx)%s-1)='m('//trim(adjustl(ind1))//')'
             sqse(opl(ox,ixx)%s-1)='m('//trim(adjustl(ind2))//')'
             momd(opl(ox,ixx)%s-1)='p(:,'//trim(adjustl(ind1))//')'
             mome(opl(ox,ixx)%s-1)='p(:,'//trim(adjustl(ind2))//')'
            end if  ! added on 20 March, 2008
            if (opl(ox,ixx)%o3(3) > 0) then
             sqsl(opl(ox,ixx)%s)='sqsl'//trim(adjustl(ind4))//'+m('
     &            //trim(adjustl(ind3))//')'
             moml(opl(ox,ixx)%s)='q'//trim(adjustl(ind4))//'+p(:,'
     &            //trim(adjustl(ind3))//')'
             write(inds,'(i16)') opl(ox,ixx)%s
             sqsc(opl(ox,ixx)%s)='sqs'//trim(adjustl(inds))
             dlip(opl(ox,ixx)%s)='dl'//trim(adjustl(inds))
             sqsd(opl(ox,ixx)%s)='sqs'//trim(adjustl(ind4))
             sqse(opl(ox,ixx)%s)='m('//trim(adjustl(ind3))//')'
             momd(opl(ox,ixx)%s)='q'//trim(adjustl(ind4))
             mome(opl(ox,ixx)%s)='p(:,'//trim(adjustl(ind3))//')'
             if (3 <= mxmap) then
              imap(opl(ox,ixx)%s)=1
              mapm(opl(ox,ixx)%s)=opl(ox,ixx)%m
              mapw(opl(ox,ixx)%s)=opl(ox,ixx)%w
             end if
            else if (opl(ox,ixx)%o3(3) < 0) then
             sqsl(opl(ox,ixx)%s)='sqsl'//trim(adjustl(ind4))//'+sqsl'
     &            //trim(adjustl(ind3))
             moml(opl(ox,ixx)%s)='q'//trim(adjustl(ind4))//'+q'
     &            //trim(adjustl(ind3))
             write(inds,'(i16)') opl(ox,ixx)%s
             sqsc(opl(ox,ixx)%s)='sqs'//trim(adjustl(inds))
             dlip(opl(ox,ixx)%s)='dl'//trim(adjustl(inds))
             sqsd(opl(ox,ixx)%s)='sqs'//trim(adjustl(ind4))
             sqse(opl(ox,ixx)%s)='sqs'//trim(adjustl(ind3))
             momd(opl(ox,ixx)%s)='q'//trim(adjustl(ind4))
             mome(opl(ox,ixx)%s)='q'//trim(adjustl(ind3))
             sqsa(opl(ox,abs(opl(ox,ixx)%o3(3)))%s)='sqs'//
     &             trim(adjustl(ind0))
             sqsb(opl(ox,abs(opl(ox,ixx)%o3(3)))%s)='sqs'//
     &             trim(adjustl(ind4))
             if (opl(ox,abs(opl(ox,ixx)%o3(3)))%ig
     &          +opl(ox,abs(opl(ox,ixx)%o3(3)))%ii+2 <= mxmap) then
              imap(opl(ox,ixx)%s)=1
              mapm(opl(ox,ixx)%s)=opl(ox,ixx)%m
              mapw(opl(ox,ixx)%s)=opl(ox,ixx)%w
             end if
            end if
           else if (opl(ox,ixx)%o3(2) < 0) then
            write(ind2,'(i16)') opl(ox,abs(opl(ox,ixx)%o3(2)))%s
            if (opl(ox,ixx)%s > 1) then ! added on 20 March, 2008
             sqsl(opl(ox,ixx)%s-1)='m('//trim(adjustl(ind1))//')+sqsl'
     &            //trim(adjustl(ind2))
             moml(opl(ox,ixx)%s-1)='p(:,'//trim(adjustl(ind1))//')+q'
     &            //trim(adjustl(ind2))
             sqsc(opl(ox,ixx)%s-1)='sqs'//trim(adjustl(ind4))
             dlip(opl(ox,ixx)%s-1)='dl'//trim(adjustl(ind4))
             sqsd(opl(ox,ixx)%s-1)='m('//trim(adjustl(ind1))//')'
             sqse(opl(ox,ixx)%s-1)='sqs'//trim(adjustl(ind2))
             momd(opl(ox,ixx)%s-1)='p(:,'//trim(adjustl(ind1))//')'
             mome(opl(ox,ixx)%s-1)='q'//trim(adjustl(ind2))
            end if
            sqsa(opl(ox,abs(opl(ox,ixx)%o3(2)))%s)='sqs'//
     &             trim(adjustl(ind4))
            sqsb(opl(ox,abs(opl(ox,ixx)%o3(2)))%s)='m('//
     &             trim(adjustl(ind1))//')'
            if (opl(ox,ixx)%o3(3) > 0) then
             sqsl(opl(ox,ixx)%s)='sqsl'//trim(adjustl(ind4))//'+m('
     &            //trim(adjustl(ind3))//')'
             moml(opl(ox,ixx)%s)='q'//trim(adjustl(ind4))//'+p(:,'
     &            //trim(adjustl(ind3))//')'
             write(inds,'(i16)') opl(ox,ixx)%s
             sqsc(opl(ox,ixx)%s)='sqs'//trim(adjustl(inds))
             dlip(opl(ox,ixx)%s)='dl'//trim(adjustl(inds))
             sqsd(opl(ox,ixx)%s)='sqs'//trim(adjustl(ind4))
             sqse(opl(ox,ixx)%s)='m('//trim(adjustl(ind3))//')'
             momd(opl(ox,ixx)%s)='q'//trim(adjustl(ind4))
             mome(opl(ox,ixx)%s)='p(:,'//trim(adjustl(ind3))//')'
c            if (opl(ox,abs(opl(ox,ixx)%o3(2)))%ig+2 <= mxmap) then
             if (opl(ox,abs(opl(ox,ixx)%o3(2)))%ig
     &          +opl(ox,abs(opl(ox,ixx)%o3(2)))%ii+2 <= mxmap) then
              imap(opl(ox,ixx)%s)=1
              mapm(opl(ox,ixx)%s)=opl(ox,ixx)%m
              mapw(opl(ox,ixx)%s)=opl(ox,ixx)%w
             end if
            else if (opl(ox,ixx)%o3(3) < 0) then
             sqsl(opl(ox,ixx)%s)='sqsl'//trim(adjustl(ind4))//'+sqsl'
     &            //trim(adjustl(ind3))
             moml(opl(ox,ixx)%s)='q'//trim(adjustl(ind4))//'+q'
     &            //trim(adjustl(ind3))
             write(inds,'(i16)') opl(ox,ixx)%s
             sqsc(opl(ox,ixx)%s)='sqs'//trim(adjustl(inds))
             dlip(opl(ox,ixx)%s)='dl'//trim(adjustl(inds))
             sqsd(opl(ox,ixx)%s)='sqs'//trim(adjustl(ind4))
             sqse(opl(ox,ixx)%s)='sqs'//trim(adjustl(ind3))
             momd(opl(ox,ixx)%s)='q'//trim(adjustl(ind4))
             mome(opl(ox,ixx)%s)='q'//trim(adjustl(ind3))
             sqsa(opl(ox,abs(opl(ox,ixx)%o3(3)))%s)='sqs'//
     &             trim(adjustl(ind0))
             sqsb(opl(ox,abs(opl(ox,ixx)%o3(3)))%s)='sqs'//
     &             trim(adjustl(ind4))
             if (opl(ox,abs(opl(ox,ixx)%o3(2)))%ig
     &          +opl(ox,abs(opl(ox,ixx)%o3(3)))%ig
     &          +opl(ox,abs(opl(ox,ixx)%o3(2)))%ii
     &          +opl(ox,abs(opl(ox,ixx)%o3(3)))%ii+1 <= mxmap) then
              imap(opl(ox,ixx)%s)=1
              mapm(opl(ox,ixx)%s)=opl(ox,ixx)%m
              mapw(opl(ox,ixx)%s)=opl(ox,ixx)%w
             end if
            end if
           end if
          else if (opl(ox,ixx)%o3(1) < 0) then
           if (opl(ox,abs(opl(ox,ixx)%o3(1)))%s0 == 0) then
            write(ind1,'(i16)') opl(ox,abs(opl(ox,ixx)%o3(1)))%s
            sqsa(opl(ox,abs(opl(ox,ixx)%o3(1)))%s)='sqs'//
     &             trim(adjustl(ind4))
            if (opl(ox,ixx)%o3(2) > 0) then
             write(ind2,'(i16)') opl(ox,ixx)%o3(2)
             if (opl(ox,ixx)%s > 1) then ! added on 20 March, 2008
              sqsl(opl(ox,ixx)%s-1)='sqsl'//trim(adjustl(ind1))//'+m('
     &             //trim(adjustl(ind2))//')'
              moml(opl(ox,ixx)%s-1)='q'//trim(adjustl(ind1))//'+p(:,'
     &            //trim(adjustl(ind2))//')'
              sqsc(opl(ox,ixx)%s-1)='sqs'//trim(adjustl(ind4))
              dlip(opl(ox,ixx)%s-1)='dl'//trim(adjustl(ind4))
              sqsd(opl(ox,ixx)%s-1)='sqs'//trim(adjustl(ind1))
              sqse(opl(ox,ixx)%s-1)='m('//trim(adjustl(ind2))//')'
              momd(opl(ox,ixx)%s-1)='q'//trim(adjustl(ind1))
              mome(opl(ox,ixx)%s-1)='p(:,'//trim(adjustl(ind2))//')'
             end if
             sqsb(opl(ox,abs(opl(ox,ixx)%o3(1)))%s)='m('//
     &             trim(adjustl(ind2))//')'
             if (opl(ox,ixx)%o3(3) > 0) then
              sqsl(opl(ox,ixx)%s)='sqsl'//trim(adjustl(ind4))//'+m('
     &            //trim(adjustl(ind3))//')'
              moml(opl(ox,ixx)%s)='q'//trim(adjustl(ind4))//'+p(:,'
     &            //trim(adjustl(ind3))//')'
              write(inds,'(i16)') opl(ox,ixx)%s
              sqsc(opl(ox,ixx)%s)='sqs'//trim(adjustl(inds))
              dlip(opl(ox,ixx)%s)='dl'//trim(adjustl(inds))
              sqsd(opl(ox,ixx)%s)='sqs'//trim(adjustl(ind4))
              sqse(opl(ox,ixx)%s)='m('//trim(adjustl(ind3))//')'
              momd(opl(ox,ixx)%s)='q'//trim(adjustl(ind4))
              mome(opl(ox,ixx)%s)='p(:,'//trim(adjustl(ind3))//')'
              if (opl(ox,abs(opl(ox,ixx)%o3(1)))%ig
     &           +opl(ox,abs(opl(ox,ixx)%o3(1)))%ii+2 <= mxmap) then
               imap(opl(ox,ixx)%s)=1
               mapm(opl(ox,ixx)%s)=opl(ox,ixx)%m
               mapw(opl(ox,ixx)%s)=opl(ox,ixx)%w
              end if
             else if (opl(ox,ixx)%o3(3) < 0) then
              sqsl(opl(ox,ixx)%s)='sqsl'//trim(adjustl(ind4))//'+sqsl'
     &            //trim(adjustl(ind3))
              moml(opl(ox,ixx)%s)='q'//trim(adjustl(ind4))//'+q'
     &            //trim(adjustl(ind3))
              write(inds,'(i16)') opl(ox,ixx)%s
              sqsc(opl(ox,ixx)%s)='sqs'//trim(adjustl(inds))
              dlip(opl(ox,ixx)%s)='dl'//trim(adjustl(inds))
              sqsd(opl(ox,ixx)%s)='sqs'//trim(adjustl(ind4))
              sqse(opl(ox,ixx)%s)='sqs'//trim(adjustl(ind3))
              momd(opl(ox,ixx)%s)='q'//trim(adjustl(ind4))
              mome(opl(ox,ixx)%s)='q'//trim(adjustl(ind3))
              sqsa(opl(ox,abs(opl(ox,ixx)%o3(3)))%s)='sqs'//
     &             trim(adjustl(ind0))
              if (opl(ox,ixx)%o3(1) > nisp) then     ! Added on 15 May, 2014
               write(ind5,'(i16)') opl(ox,ixx)%o3(1)
               sqsb(opl(ox,abs(opl(ox,ixx)%o3(3)))%s)='m('//
     &             trim(adjustl(ind5))//')'
              else if (opl(ox,ixx)%o3(2) > nisp) then
               write(ind5,'(i16)') opl(ox,ixx)%o3(2)
               sqsb(opl(ox,abs(opl(ox,ixx)%o3(3)))%s)='m('//
     &             trim(adjustl(ind5))//')'
              else
               sqsb(opl(ox,abs(opl(ox,ixx)%o3(3)))%s)='sqs'//
     &             trim(adjustl(ind4))
              end if
              if (opl(ox,abs(opl(ox,ixx)%o3(1)))%ig
     &           +opl(ox,abs(opl(ox,ixx)%o3(3)))%ig
     &           +opl(ox,abs(opl(ox,ixx)%o3(1)))%ii
     &           +opl(ox,abs(opl(ox,ixx)%o3(3)))%ii+1 <= mxmap) then
               imap(opl(ox,ixx)%s)=1
               mapm(opl(ox,ixx)%s)=opl(ox,ixx)%m
               mapw(opl(ox,ixx)%s)=opl(ox,ixx)%w
              end if
             end if
            else if (opl(ox,ixx)%o3(2) < 0) then
             if (opl(ox,abs(opl(ox,ixx)%o3(2)))%s0 == 0) then
              write(ind2,'(i16)') opl(ox,abs(opl(ox,ixx)%o3(2)))%s
              if (opl(ox,ixx)%s > 1) then ! added on 20 March, 2008
               sqsl(opl(ox,ixx)%s-1)='sqsl'//trim(adjustl(ind1))//
     &              '+sqsl'//trim(adjustl(ind2))
               moml(opl(ox,ixx)%s-1)='q'//trim(adjustl(ind1))//'+q'
     &            //trim(adjustl(ind2))
               sqsc(opl(ox,ixx)%s-1)='sqs'//trim(adjustl(ind4))
               dlip(opl(ox,ixx)%s-1)='dl'//trim(adjustl(ind4))
               sqsd(opl(ox,ixx)%s-1)='sqs'//trim(adjustl(ind1))
               sqse(opl(ox,ixx)%s-1)='sqs'//trim(adjustl(ind2))
               momd(opl(ox,ixx)%s-1)='q'//trim(adjustl(ind1))
               mome(opl(ox,ixx)%s-1)='q'//trim(adjustl(ind2))
              end if
              sqsa(opl(ox,abs(opl(ox,ixx)%o3(2)))%s)='sqs'//
     &             trim(adjustl(ind4))
              if (opl(ox,ixx)%o3(1) < opl(ox,ixx)%o3(2)) then
               sqsb(opl(ox,abs(opl(ox,ixx)%o3(1)))%s)='sqsl'//
     &              trim(adjustl(ind2))
               sqsb(opl(ox,abs(opl(ox,ixx)%o3(2)))%s)='sqs'//
     &              trim(adjustl(ind1))
              else
               sqsb(opl(ox,abs(opl(ox,ixx)%o3(1)))%s)='sqs'//
     &              trim(adjustl(ind2))
               sqsb(opl(ox,abs(opl(ox,ixx)%o3(2)))%s)='sqsl'//
     &             trim(adjustl(ind1))
              end if
              if (opl(ox,ixx)%o3(3) > 0) then
               sqsl(opl(ox,ixx)%s)='sqsl'//trim(adjustl(ind4))//'+m('
     &            //trim(adjustl(ind3))//')'
               moml(opl(ox,ixx)%s)='q'//trim(adjustl(ind4))//'+p(:,'
     &            //trim(adjustl(ind3))//')'
               write(inds,'(i16)') opl(ox,ixx)%s
               sqsc(opl(ox,ixx)%s)='sqs'//trim(adjustl(inds))
               dlip(opl(ox,ixx)%s)='dl'//trim(adjustl(inds))
               sqsd(opl(ox,ixx)%s)='sqs'//trim(adjustl(ind4))
               sqse(opl(ox,ixx)%s)='m('//trim(adjustl(ind3))//')'
               momd(opl(ox,ixx)%s)='q'//trim(adjustl(ind4))
               mome(opl(ox,ixx)%s)='p(:,'//trim(adjustl(ind3))//')'
               if (opl(ox,abs(opl(ox,ixx)%o3(1)))%ig
     &           +opl(ox,abs(opl(ox,ixx)%o3(2)))%ig
     &           +opl(ox,abs(opl(ox,ixx)%o3(1)))%ii
     &           +opl(ox,abs(opl(ox,ixx)%o3(2)))%ii+1 <= mxmap) then
                imap(opl(ox,ixx)%s)=1
                mapm(opl(ox,ixx)%s)=opl(ox,ixx)%m
                mapw(opl(ox,ixx)%s)=opl(ox,ixx)%w
               end if
              else if (opl(ox,ixx)%o3(3) < 0) then
               sqsl(opl(ox,ixx)%s)='sqsl'//trim(adjustl(ind4))//'+sqsl'
     &            //trim(adjustl(ind3))
               moml(opl(ox,ixx)%s)='q'//trim(adjustl(ind4))//'+q'
     &            //trim(adjustl(ind3))
               write(inds,'(i16)') opl(ox,ixx)%s
               sqsc(opl(ox,ixx)%s)='sqs'//trim(adjustl(inds))
               dlip(opl(ox,ixx)%s)='dl'//trim(adjustl(inds))
               sqsd(opl(ox,ixx)%s)='sqs'//trim(adjustl(ind4))
               sqse(opl(ox,ixx)%s)='sqs'//trim(adjustl(ind3))
               momd(opl(ox,ixx)%s)='q'//trim(adjustl(ind4))
               mome(opl(ox,ixx)%s)='q'//trim(adjustl(ind3))
               sqsa(opl(ox,abs(opl(ox,ixx)%o3(3)))%s)='sqs'//
     &              trim(adjustl(ind0))
               sqsb(opl(ox,abs(opl(ox,ixx)%o3(3)))%s)='sqs'//
     &              trim(adjustl(ind4))
               if (opl(ox,abs(opl(ox,ixx)%o3(1)))%ig
     &            +opl(ox,abs(opl(ox,ixx)%o3(2)))%ig
     &            +opl(ox,abs(opl(ox,ixx)%o3(3)))%ig 
     &            +opl(ox,abs(opl(ox,ixx)%o3(1)))%ii
     &            +opl(ox,abs(opl(ox,ixx)%o3(2)))%ii
     &            +opl(ox,abs(opl(ox,ixx)%o3(3)))%ii <= mxmap) then
                imap(opl(ox,ixx)%s)=1
                mapm(opl(ox,ixx)%s)=opl(ox,ixx)%m
                mapw(opl(ox,ixx)%s)=opl(ox,ixx)%w
               end if
              end if
             else ! opl(ox,abs(opl(ox,ixx)%o3(2)))%s0 ! 10 July, 2008:
              write(ind2,'(i16)') opl(ox,abs(opl(ox,ixx)%o3(2)))%s0
              if (opl(ox,ixx)%s > 1) then ! added on 20 March, 2008
               sqsl(opl(ox,ixx)%s-1)='sqsl'//trim(adjustl(ind1))//'+m('
     &            //trim(adjustl(ind2))//')'
               moml(opl(ox,ixx)%s-1)='q'//trim(adjustl(ind1))//'+p(:,'
     &            //trim(adjustl(ind2))//')'
               sqsc(opl(ox,ixx)%s-1)='sqs'//trim(adjustl(ind4))
               dlip(opl(ox,ixx)%s-1)='dl'//trim(adjustl(ind4))
               sqsd(opl(ox,ixx)%s-1)='sqs'//trim(adjustl(ind1))
               sqse(opl(ox,ixx)%s-1)='m('//trim(adjustl(ind2))//')'
               momd(opl(ox,ixx)%s-1)='q'//trim(adjustl(ind1))
               mome(opl(ox,ixx)%s-1)='p(:,'//trim(adjustl(ind2))//')'
              end if
              sqsa(opl(ox,abs(opl(ox,ixx)%o3(2)))%s)='sqs'//
     &             trim(adjustl(ind4))
ckk>
              sqsb(opl(ox,abs(opl(ox,ixx)%o3(1)))%s)='m('//
     &             trim(adjustl(ind2))//')'
c Commented on 20 July, 2009:
c              sqsb(opl(ox,abs(opl(ox,ixx)%o3(2)))%s)='sqs'//
c     &             trim(adjustl(ind1))
ckk<
              if (opl(ox,ixx)%o3(3) > 0) then
               sqsl(opl(ox,ixx)%s)='sqsl'//trim(adjustl(ind4))//'+m('
     &            //trim(adjustl(ind3))//')'
               moml(opl(ox,ixx)%s)='q'//trim(adjustl(ind4))//'+p(:,'
     &            //trim(adjustl(ind3))//')'
               write(inds,'(i16)') opl(ox,ixx)%s
               sqsc(opl(ox,ixx)%s)='sqs'//trim(adjustl(inds))
               dlip(opl(ox,ixx)%s)='dl'//trim(adjustl(inds))
               sqsd(opl(ox,ixx)%s)='sqs'//trim(adjustl(ind4))
               sqse(opl(ox,ixx)%s)='m('//trim(adjustl(ind3))//')'
               momd(opl(ox,ixx)%s)='q'//trim(adjustl(ind4))
               mome(opl(ox,ixx)%s)='p(:,'//trim(adjustl(ind3))//')'
               if (opl(ox,abs(opl(ox,ixx)%o3(1)))%ig
     &            +opl(ox,abs(opl(ox,ixx)%o3(2)))%ig
     &            +opl(ox,abs(opl(ox,ixx)%o3(1)))%ii
     &            +opl(ox,abs(opl(ox,ixx)%o3(2)))%ii+1 <= mxmap) then
                 imap(opl(ox,ixx)%s)=1
                 mapm(opl(ox,ixx)%s)=opl(ox,ixx)%m
                 mapw(opl(ox,ixx)%s)=opl(ox,ixx)%w
                end if
              else if (opl(ox,ixx)%o3(3) < 0) then
               sqsl(opl(ox,ixx)%s)='sqsl'//trim(adjustl(ind4))//'+sqsl'
     &            //trim(adjustl(ind3))
               moml(opl(ox,ixx)%s)='q'//trim(adjustl(ind4))//'+q'
     &            //trim(adjustl(ind3))
               write(inds,'(i16)') opl(ox,ixx)%s
               sqsc(opl(ox,ixx)%s)='sqs'//trim(adjustl(inds))
               dlip(opl(ox,ixx)%s)='dl'//trim(adjustl(inds))
               sqsd(opl(ox,ixx)%s)='sqs'//trim(adjustl(ind4))
               sqse(opl(ox,ixx)%s)='sqs'//trim(adjustl(ind3))
               momd(opl(ox,ixx)%s)='q'//trim(adjustl(ind4))
               mome(opl(ox,ixx)%s)='q'//trim(adjustl(ind3))
               sqsa(opl(ox,abs(opl(ox,ixx)%o3(3)))%s)='sqs'//
     &             trim(adjustl(ind0))
               sqsb(opl(ox,abs(opl(ox,ixx)%o3(3)))%s)='sqs'//
     &              trim(adjustl(ind4))
               if (opl(ox,abs(opl(ox,ixx)%o3(1)))%ig
     &            +opl(ox,abs(opl(ox,ixx)%o3(2)))%ig
     &            +opl(ox,abs(opl(ox,ixx)%o3(3)))%ig 
     &            +opl(ox,abs(opl(ox,ixx)%o3(1)))%ii
     &            +opl(ox,abs(opl(ox,ixx)%o3(2)))%ii
     &            +opl(ox,abs(opl(ox,ixx)%o3(3)))%ii <= mxmap) then
                imap(opl(ox,ixx)%s)=1
                mapm(opl(ox,ixx)%s)=opl(ox,ixx)%m
                mapw(opl(ox,ixx)%s)=opl(ox,ixx)%w
               end if
              end if
             end if
            end if
           else ! opl(ox,abs(opl(ox,ixx)%o3(1)))%s0 > 0 ! 14 July, 2008
            write(ind1,'(i16)') opl(ox,abs(opl(ox,ixx)%o3(1)))%s0
            sqsa(opl(ox,abs(opl(ox,ixx)%o3(1)))%s)='sqs'//
     &             trim(adjustl(ind4))
            if (opl(ox,ixx)%o3(2) > 0) then
             write(ind2,'(i16)') opl(ox,ixx)%o3(2)
             if (opl(ox,ixx)%s > 1) then ! added on 20 March, 2008
              sqsl(opl(ox,ixx)%s-1)='m('//trim(adjustl(ind1))//')+m('
     &             //trim(adjustl(ind2))//')'
              moml(opl(ox,ixx)%s-1)='p(:,'//trim(adjustl(ind1))//
     &                              ')+p(:,'//trim(adjustl(ind2))//')'
              sqsc(opl(ox,ixx)%s-1)='sqs'//trim(adjustl(ind4))
              dlip(opl(ox,ixx)%s-1)='dl'//trim(adjustl(ind4))
              sqsd(opl(ox,ixx)%s-1)='m('//trim(adjustl(ind1))//')'
              sqse(opl(ox,ixx)%s-1)='m('//trim(adjustl(ind2))//')'
              momd(opl(ox,ixx)%s-1)='p(:,'//trim(adjustl(ind1))//')'
              mome(opl(ox,ixx)%s-1)='p(:,'//trim(adjustl(ind2))//')'
             end if
             sqsb(opl(ox,abs(opl(ox,ixx)%o3(1)))%s)='m('//
     &             trim(adjustl(ind2))//')'
             if (opl(ox,ixx)%o3(3) > 0) then
              sqsl(opl(ox,ixx)%s)='sqsl'//trim(adjustl(ind4))//'+m('
     &            //trim(adjustl(ind3))//')'
              moml(opl(ox,ixx)%s)='q'//trim(adjustl(ind4))//'+p(:,'
     &            //trim(adjustl(ind3))//')'
              write(inds,'(i16)') opl(ox,ixx)%s
              sqsc(opl(ox,ixx)%s)='sqs'//trim(adjustl(inds))
              dlip(opl(ox,ixx)%s)='dl'//trim(adjustl(inds))
              sqsd(opl(ox,ixx)%s)='sqs'//trim(adjustl(ind4))
              sqse(opl(ox,ixx)%s)='m('//trim(adjustl(ind3))//')'
              momd(opl(ox,ixx)%s)='q'//trim(adjustl(ind4))
              mome(opl(ox,ixx)%s)='p(:,'//trim(adjustl(ind3))//')'
              if (opl(ox,abs(opl(ox,ixx)%o3(1)))%ig
     &           +opl(ox,abs(opl(ox,ixx)%o3(1)))%ii+2 <= mxmap) then
               imap(opl(ox,ixx)%s)=1
               mapm(opl(ox,ixx)%s)=opl(ox,ixx)%m
               mapw(opl(ox,ixx)%s)=opl(ox,ixx)%w
              end if
             else if (opl(ox,ixx)%o3(3) < 0) then
              sqsl(opl(ox,ixx)%s)='sqsl'//trim(adjustl(ind4))//'+sqsl'
     &            //trim(adjustl(ind3))
              moml(opl(ox,ixx)%s)='q'//trim(adjustl(ind4))//'+q'
     &            //trim(adjustl(ind3))
              write(inds,'(i16)') opl(ox,ixx)%s
              sqsc(opl(ox,ixx)%s)='sqs'//trim(adjustl(inds))
              dlip(opl(ox,ixx)%s)='dl'//trim(adjustl(inds))
              sqsd(opl(ox,ixx)%s)='sqs'//trim(adjustl(ind4))
              sqse(opl(ox,ixx)%s)='sqs'//trim(adjustl(ind3))
              momd(opl(ox,ixx)%s)='q'//trim(adjustl(ind4))
              mome(opl(ox,ixx)%s)='q'//trim(adjustl(ind3))
              sqsa(opl(ox,abs(opl(ox,ixx)%o3(3)))%s)='sqs'//
     &             trim(adjustl(ind0))
              sqsb(opl(ox,abs(opl(ox,ixx)%o3(3)))%s)='sqs'//
     &             trim(adjustl(ind4))
              if (opl(ox,abs(opl(ox,ixx)%o3(1)))%ig
     &           +opl(ox,abs(opl(ox,ixx)%o3(3)))%ig
     &           +opl(ox,abs(opl(ox,ixx)%o3(1)))%ii
     &           +opl(ox,abs(opl(ox,ixx)%o3(3)))%ii+1 <= mxmap) then
               imap(opl(ox,ixx)%s)=1
               mapm(opl(ox,ixx)%s)=opl(ox,ixx)%m
               mapw(opl(ox,ixx)%s)=opl(ox,ixx)%w
              end if
             end if
            else if (opl(ox,ixx)%o3(2) < 0) then
             if (opl(ox,abs(opl(ox,ixx)%o3(2)))%s0 == 0) then
              write(ind2,'(i16)') opl(ox,abs(opl(ox,ixx)%o3(2)))%s
              if (opl(ox,ixx)%s > 1) then ! added on 20 March, 2008
               sqsl(opl(ox,ixx)%s-1)='m('//trim(adjustl(ind1))//
     &                               ')+sqsl'//trim(adjustl(ind2))
               moml(opl(ox,ixx)%s-1)='p(:,'//trim(adjustl(ind1))//')+q'
     &            //trim(adjustl(ind2))
               sqsc(opl(ox,ixx)%s-1)='sqs'//trim(adjustl(ind4))
               dlip(opl(ox,ixx)%s-1)='dl'//trim(adjustl(ind4))
               sqsd(opl(ox,ixx)%s-1)='m('//trim(adjustl(ind1))//')'
               sqse(opl(ox,ixx)%s-1)='sqs'//trim(adjustl(ind2))
               momd(opl(ox,ixx)%s-1)='p(:,'//trim(adjustl(ind1))//')'
               mome(opl(ox,ixx)%s-1)='q'//trim(adjustl(ind2))
              end if
              sqsa(opl(ox,abs(opl(ox,ixx)%o3(2)))%s)='sqs'//
     &             trim(adjustl(ind4))
              sqsb(opl(ox,abs(opl(ox,ixx)%o3(1)))%s)='sqsl'//
     &             trim(adjustl(ind2))
              sqsb(opl(ox,abs(opl(ox,ixx)%o3(2)))%s)='m('//
     &              trim(adjustl(ind1))//')'
              if (opl(ox,ixx)%o3(3) > 0) then
               sqsl(opl(ox,ixx)%s)='sqsl'//trim(adjustl(ind4))//'+m('
     &            //trim(adjustl(ind3))//')'
               moml(opl(ox,ixx)%s)='q'//trim(adjustl(ind4))//'+p(:,'
     &            //trim(adjustl(ind3))//')'
               write(inds,'(i16)') opl(ox,ixx)%s
               sqsc(opl(ox,ixx)%s)='sqs'//trim(adjustl(inds))
               dlip(opl(ox,ixx)%s)='dl'//trim(adjustl(inds))
               sqsd(opl(ox,ixx)%s)='sqs'//trim(adjustl(ind4))
               sqse(opl(ox,ixx)%s)='m('//trim(adjustl(ind3))//')'
               momd(opl(ox,ixx)%s)='q'//trim(adjustl(ind4))
               mome(opl(ox,ixx)%s)='p(:,'//trim(adjustl(ind3))//')'
               if (opl(ox,abs(opl(ox,ixx)%o3(1)))%ig
     &           +opl(ox,abs(opl(ox,ixx)%o3(2)))%ig
     &           +opl(ox,abs(opl(ox,ixx)%o3(1)))%ii
     &           +opl(ox,abs(opl(ox,ixx)%o3(2)))%ii+1 <= mxmap) then
                imap(opl(ox,ixx)%s)=1
                mapm(opl(ox,ixx)%s)=opl(ox,ixx)%m
                mapw(opl(ox,ixx)%s)=opl(ox,ixx)%w
               end if
              else if (opl(ox,ixx)%o3(3) < 0) then
               sqsl(opl(ox,ixx)%s)='sqsl'//trim(adjustl(ind4))//'+sqsl'
     &            //trim(adjustl(ind3))
               moml(opl(ox,ixx)%s)='q'//trim(adjustl(ind4))//'+q'
     &            //trim(adjustl(ind3))
               write(inds,'(i16)') opl(ox,ixx)%s
               sqsc(opl(ox,ixx)%s)='sqs'//trim(adjustl(inds))
               dlip(opl(ox,ixx)%s)='dl'//trim(adjustl(inds))
               sqsd(opl(ox,ixx)%s)='sqs'//trim(adjustl(ind4))
               sqse(opl(ox,ixx)%s)='sqs'//trim(adjustl(ind3))
               momd(opl(ox,ixx)%s)='q'//trim(adjustl(ind4))
               mome(opl(ox,ixx)%s)='q'//trim(adjustl(ind3))
               sqsa(opl(ox,abs(opl(ox,ixx)%o3(3)))%s)='sqs'//
     &             trim(adjustl(ind0))
               sqsb(opl(ox,abs(opl(ox,ixx)%o3(3)))%s)='sqs'//
     &             trim(adjustl(ind4))
               if (opl(ox,abs(opl(ox,ixx)%o3(1)))%ig
     &            +opl(ox,abs(opl(ox,ixx)%o3(2)))%ig
     &            +opl(ox,abs(opl(ox,ixx)%o3(3)))%ig 
     &            +opl(ox,abs(opl(ox,ixx)%o3(1)))%ii
     &            +opl(ox,abs(opl(ox,ixx)%o3(2)))%ii
     &            +opl(ox,abs(opl(ox,ixx)%o3(3)))%ii <= mxmap) then
                imap(opl(ox,ixx)%s)=1
                mapm(opl(ox,ixx)%s)=opl(ox,ixx)%m
                mapw(opl(ox,ixx)%s)=opl(ox,ixx)%w
               end if
              end if
             else ! opl(ox,abs(opl(ox,ixx)%o3(2)))%s0 ! 10 July, 2008:
              write(ind2,'(i16)') opl(ox,abs(opl(ox,ixx)%o3(2)))%s0
              if (opl(ox,ixx)%s > 1) then ! added on 20 March, 2008
               sqsl(opl(ox,ixx)%s-1)='m('//trim(adjustl(ind1))//')+m('
     &            //trim(adjustl(ind2))//')'
               moml(opl(ox,ixx)%s-1)='p(:,'//trim(adjustl(ind1))//
     &                             ')+p(:,'//trim(adjustl(ind2))//')'
               sqsc(opl(ox,ixx)%s-1)='sqs'//trim(adjustl(ind4))
               dlip(opl(ox,ixx)%s-1)='dl'//trim(adjustl(ind4))
               sqsd(opl(ox,ixx)%s-1)='m('//trim(adjustl(ind1))//')'
               sqse(opl(ox,ixx)%s-1)='m('//trim(adjustl(ind2))//')'
               momd(opl(ox,ixx)%s-1)='p(:,'//trim(adjustl(ind1))//')'
               mome(opl(ox,ixx)%s-1)='p(:,'//trim(adjustl(ind2))//')'
              end if
              sqsa(opl(ox,abs(opl(ox,ixx)%o3(2)))%s)='sqs'//
     &             trim(adjustl(ind4))
              sqsb(opl(ox,abs(opl(ox,ixx)%o3(1)))%s)='m('//
     &             trim(adjustl(ind2))//')'
              sqsb(opl(ox,abs(opl(ox,ixx)%o3(2)))%s)='m('//
     &             trim(adjustl(ind1))//')'
              if (opl(ox,ixx)%o3(3) > 0) then
               sqsl(opl(ox,ixx)%s)='sqsl'//trim(adjustl(ind4))//'+m('
     &            //trim(adjustl(ind3))//')'
               moml(opl(ox,ixx)%s)='q'//trim(adjustl(ind4))//'+p(:,'
     &            //trim(adjustl(ind3))//')'
               write(inds,'(i16)') opl(ox,ixx)%s
               sqsc(opl(ox,ixx)%s)='sqs'//trim(adjustl(inds))
               dlip(opl(ox,ixx)%s)='dl'//trim(adjustl(inds))
               sqsd(opl(ox,ixx)%s)='sqs'//trim(adjustl(ind4))
               sqse(opl(ox,ixx)%s)='m('//trim(adjustl(ind3))//')'
               momd(opl(ox,ixx)%s)='q'//trim(adjustl(ind4))
               mome(opl(ox,ixx)%s)='p(:,'//trim(adjustl(ind3))//')'
               if (opl(ox,abs(opl(ox,ixx)%o3(1)))%ig
     &            +opl(ox,abs(opl(ox,ixx)%o3(2)))%ig
     &            +opl(ox,abs(opl(ox,ixx)%o3(1)))%ii
     &            +opl(ox,abs(opl(ox,ixx)%o3(2)))%ii+1 <= mxmap) then
                imap(opl(ox,ixx)%s)=1
                mapm(opl(ox,ixx)%s)=opl(ox,ixx)%m
                mapw(opl(ox,ixx)%s)=opl(ox,ixx)%w
               end if
              else if (opl(ox,ixx)%o3(3) < 0) then
               sqsl(opl(ox,ixx)%s)='sqsl'//trim(adjustl(ind4))//'+sqsl'
     &            //trim(adjustl(ind3))
               moml(opl(ox,ixx)%s)='q'//trim(adjustl(ind4))//'+q'
     &            //trim(adjustl(ind3))
               write(inds,'(i16)') opl(ox,ixx)%s
               sqsc(opl(ox,ixx)%s)='sqs'//trim(adjustl(inds))
               dlip(opl(ox,ixx)%s)='dl'//trim(adjustl(inds))
               sqsd(opl(ox,ixx)%s)='sqs'//trim(adjustl(ind4))
               sqse(opl(ox,ixx)%s)='sqs'//trim(adjustl(ind3))
               momd(opl(ox,ixx)%s)='q'//trim(adjustl(ind4))
               mome(opl(ox,ixx)%s)='q'//trim(adjustl(ind3))
               sqsa(opl(ox,abs(opl(ox,ixx)%o3(3)))%s)='sqs'//
     &             trim(adjustl(ind0))
               sqsb(opl(ox,abs(opl(ox,ixx)%o3(3)))%s)='sqs'//
     &              trim(adjustl(ind4))
               if (opl(ox,abs(opl(ox,ixx)%o3(1)))%ig
     &            +opl(ox,abs(opl(ox,ixx)%o3(2)))%ig
     &            +opl(ox,abs(opl(ox,ixx)%o3(3)))%ig 
     &            +opl(ox,abs(opl(ox,ixx)%o3(1)))%ii
     &            +opl(ox,abs(opl(ox,ixx)%o3(2)))%ii
     &            +opl(ox,abs(opl(ox,ixx)%o3(3)))%ii <= mxmap) then
                imap(opl(ox,ixx)%s)=1
                mapm(opl(ox,ixx)%s)=opl(ox,ixx)%m
                mapw(opl(ox,ixx)%s)=opl(ox,ixx)%w
               end if
              end if
             end if
            end if
           end if
          end if
         else ! opl(ox,ixx)%i3
          write(inds,'(i16)') opl(ox,ixx)%s
          sqsc(opl(ox,ixx)%s)='sqs'//trim(adjustl(inds))
          dlip(opl(ox,ixx)%s)='dl'//trim(adjustl(inds))
          if (opl(ox,ixx)%o2(1) > 0) then
           write(ind1,'(i16)') opl(ox,ixx)%o2(1)
           sqsla(opl(ox,ixx)%s)='m('//trim(adjustl(ind1))//')'
           momla(opl(ox,ixx)%s)='p(:,'//trim(adjustl(ind1))//')'
           sqsd(opl(ox,ixx)%s)='m('//trim(adjustl(ind1))//')'
           momd(opl(ox,ixx)%s)='p(:,'//trim(adjustl(ind1))//')'
           if (opl(ox,ixx)%o2(2) > 0) then
            write(ind2,'(i16)') opl(ox,ixx)%o2(2)
            sqslb(opl(ox,ixx)%s)='m('//trim(adjustl(ind2))//')'
            momlb(opl(ox,ixx)%s)='p(:,'//trim(adjustl(ind2))//')'
            sqse(opl(ox,ixx)%s)='m('//trim(adjustl(ind2))//')'
            mome(opl(ox,ixx)%s)='p(:,'//trim(adjustl(ind2))//')'
            sqsl(opl(ox,ixx)%s)=trim(sqsla(opl(ox,ixx)%s))//'+'//
     &                       trim(sqslb(opl(ox,ixx)%s))
            moml(opl(ox,ixx)%s)=trim(momla(opl(ox,ixx)%s))//'+'//
     &                       trim(momlb(opl(ox,ixx)%s))
            if (expl(opl(ox,ixx)%o2(1)) == 'n' .and.  ! 16 Nov, 2010
     &          expl(opl(ox,ixx)%o2(2)) == 'n') nuga(opl(ox,ixx)%s)=1
            if (2 <= mxmap) then
             imap(opl(ox,ixx)%s)=1
             mapm(opl(ox,ixx)%s)=opl(ox,ixx)%m
             mapw(opl(ox,ixx)%s)=opl(ox,ixx)%w
            end if
           else if (opl(ox,ixx)%o2(2) < 0) then
            if (opl(ox,abs(opl(ox,ixx)%o2(2)))%s0 == 0) then
             write(ind2,'(i16)') opl(ox,abs(opl(ox,ixx)%o2(2)))%s
             sqslb(opl(ox,ixx)%s)='sqsl'//trim(adjustl(ind2))
             momlb(opl(ox,ixx)%s)='q'//trim(adjustl(ind2))
             sqse(opl(ox,ixx)%s)='sqs'//trim(adjustl(ind2))
             mome(opl(ox,ixx)%s)='q'//trim(adjustl(ind2))
             sqsa(opl(ox,abs(opl(ox,ixx)%o2(2)))%s)='sqs'//
     &             trim(adjustl(inds))
             sqsb(opl(ox,abs(opl(ox,ixx)%o2(2)))%s)='m('//
     &             trim(adjustl(ind1))//')'
             sqsl(opl(ox,ixx)%s)=trim(sqsla(opl(ox,ixx)%s))//'+'//
     &                       trim(sqslb(opl(ox,ixx)%s))
             moml(opl(ox,ixx)%s)=trim(momla(opl(ox,ixx)%s))//'+'//
     &                       trim(momlb(opl(ox,ixx)%s))
c            if (opl(ox,abs(opl(ox,ixx)%o2(2)))%ig+1 <= mxmap) then
             if (opl(ox,abs(opl(ox,ixx)%o2(2)))%ig
     &          +opl(ox,abs(opl(ox,ixx)%o2(2)))%ii+1 <= mxmap) then
              imap(opl(ox,ixx)%s)=1
              mapm(opl(ox,ixx)%s)=opl(ox,ixx)%m
              mapw(opl(ox,ixx)%s)=opl(ox,ixx)%w
             end if
            else
             write(ind2,'(i16)') opl(ox,abs(opl(ox,ixx)%o2(2)))%s0
             sqslb(opl(ox,ixx)%s)='m('//trim(adjustl(ind2))//')'
             momlb(opl(ox,ixx)%s)='p(:,'//trim(adjustl(ind2))//')'
             sqse(opl(ox,ixx)%s)='m('//trim(adjustl(ind2))//')'
             mome(opl(ox,ixx)%s)='p(:,'//trim(adjustl(ind2))//')'
             sqsa(opl(ox,abs(opl(ox,ixx)%o2(2)))%s)='sqs'//
     &             trim(adjustl(inds))
             sqsb(opl(ox,abs(opl(ox,ixx)%o2(2)))%s)='m('//
     &             trim(adjustl(ind1))//')'
             sqsl(opl(ox,ixx)%s)=trim(sqsla(opl(ox,ixx)%s))//'+'//
     &                       trim(sqslb(opl(ox,ixx)%s))
             moml(opl(ox,ixx)%s)=trim(momla(opl(ox,ixx)%s))//'+'//
     &                       trim(momlb(opl(ox,ixx)%s))
             if (expl(opl(ox,ixx)%o2(1)) == 'n' .and.  ! 16 Nov, 2010
     &           expl(opl(ox,abs(opl(ox,ixx)%o2(2)))%s0) == 'n') 
     &                                        nuga(opl(ox,ixx)%s-1)=1
             if (2 <= mxmap) then
              imap(opl(ox,ixx)%s)=1
              mapm(opl(ox,ixx)%s)=opl(ox,ixx)%m
              mapw(opl(ox,ixx)%s)=opl(ox,ixx)%w
             end if
            end if
           end if
          else if (opl(ox,ixx)%o2(1) < 0) then
           if (opl(ox,abs(opl(ox,ixx)%o2(1)))%s0 == 0) then
            write(ind1,'(i16)') opl(ox,abs(opl(ox,ixx)%o2(1)))%s
            sqsla(opl(ox,ixx)%s)='sqsl'//trim(adjustl(ind1))
            momla(opl(ox,ixx)%s)='q'//trim(adjustl(ind1))
            sqsd(opl(ox,ixx)%s)='sqs'//trim(adjustl(ind1))
            momd(opl(ox,ixx)%s)='q'//trim(adjustl(ind1))
            if (opl(ox,ixx)%o2(2) > 0) then
             write(ind2,'(i16)') opl(ox,ixx)%o2(2)
             sqslb(opl(ox,ixx)%s)='m('//trim(adjustl(ind2))//')'
             momlb(opl(ox,ixx)%s)='p(:,'//trim(adjustl(ind2))//')'
             sqse(opl(ox,ixx)%s)='m('//trim(adjustl(ind2))//')'
             mome(opl(ox,ixx)%s)='p(:,'//trim(adjustl(ind2))//')'
             sqsa(opl(ox,abs(opl(ox,ixx)%o2(1)))%s)='sqs'//
     &             trim(adjustl(inds))
             sqsb(opl(ox,abs(opl(ox,ixx)%o2(1)))%s)='m('//
     &             trim(adjustl(ind2))//')'
             sqsl(opl(ox,ixx)%s)=trim(sqsla(opl(ox,ixx)%s))//'+'//
     &                       trim(sqslb(opl(ox,ixx)%s))
             moml(opl(ox,ixx)%s)=trim(momla(opl(ox,ixx)%s))//'+'//
     &                       trim(momlb(opl(ox,ixx)%s))
             if (opl(ox,abs(opl(ox,ixx)%o2(1)))%ig
     &          +opl(ox,abs(opl(ox,ixx)%o2(1)))%ii+1 <= mxmap) then
              imap(opl(ox,ixx)%s)=1
              mapm(opl(ox,ixx)%s)=opl(ox,ixx)%m
              mapw(opl(ox,ixx)%s)=opl(ox,ixx)%w
             end if
            else if (opl(ox,ixx)%o2(2) < 0) then
             sqsa(opl(ox,abs(opl(ox,ixx)%o2(1)))%s)='sqs'//
     &             trim(adjustl(inds))
             sqsa(opl(ox,abs(opl(ox,ixx)%o2(2)))%s)='sqs'//
     &             trim(adjustl(inds))
             if (opl(ox,abs(opl(ox,ixx)%o2(2)))%s0 == 0) then
              write(ind2,'(i16)') opl(ox,abs(opl(ox,ixx)%o2(2)))%s
              sqslb(opl(ox,ixx)%s)='sqsl'//trim(adjustl(ind2))
              momlb(opl(ox,ixx)%s)='q'//trim(adjustl(ind2))
              sqse(opl(ox,ixx)%s)='sqs'//trim(adjustl(ind2))
              mome(opl(ox,ixx)%s)='q'//trim(adjustl(ind2))
              sqsl(opl(ox,ixx)%s)=trim(sqsla(opl(ox,ixx)%s))//'+'//
     &                       trim(sqslb(opl(ox,ixx)%s))
              moml(opl(ox,ixx)%s)=trim(momla(opl(ox,ixx)%s))//'+'//
     &                       trim(momlb(opl(ox,ixx)%s))
              if (opl(ox,ixx)%o2(1) < opl(ox,ixx)%o2(2)) then
               sqsb(opl(ox,abs(opl(ox,ixx)%o2(1)))%s)='sqsl'//
     &             trim(adjustl(ind2))
               sqsb(opl(ox,abs(opl(ox,ixx)%o2(2)))%s)='sqs'//
     &             trim(adjustl(ind1))
              else
               sqsb(opl(ox,abs(opl(ox,ixx)%o2(1)))%s)='sqs'//
     &             trim(adjustl(ind2))
               sqsb(opl(ox,abs(opl(ox,ixx)%o2(2)))%s)='sqsl'//
     &             trim(adjustl(ind1))
              end if
              if (opl(ox,abs(opl(ox,ixx)%o2(1)))%ig
     &           +opl(ox,abs(opl(ox,ixx)%o2(2)))%ig
     &           +opl(ox,abs(opl(ox,ixx)%o2(1)))%ii
     &           +opl(ox,abs(opl(ox,ixx)%o2(2)))%ii <= mxmap) then
               imap(opl(ox,ixx)%s)=1
               mapm(opl(ox,ixx)%s)=opl(ox,ixx)%m
               mapw(opl(ox,ixx)%s)=opl(ox,ixx)%w
              end if
             else
              write(ind2,'(i16)') opl(ox,abs(opl(ox,ixx)%o2(2)))%s0
              sqslb(opl(ox,ixx)%s)='m('//trim(adjustl(ind2))//')'
              momlb(opl(ox,ixx)%s)='p(:,'//trim(adjustl(ind2))//')'
              sqse(opl(ox,ixx)%s)='m('//trim(adjustl(ind2))//')'
              mome(opl(ox,ixx)%s)='p(:,'//trim(adjustl(ind2))//')'
              sqsl(opl(ox,ixx)%s)=trim(sqsla(opl(ox,ixx)%s))//'+'//
     &                       trim(sqslb(opl(ox,ixx)%s))
              moml(opl(ox,ixx)%s)=trim(momla(opl(ox,ixx)%s))//'+'//
     &                       trim(momlb(opl(ox,ixx)%s))
              sqsb(opl(ox,abs(opl(ox,ixx)%o2(1)))%s)='m('//
     &             trim(adjustl(ind2))//')'
              if (opl(ox,abs(opl(ox,ixx)%o2(1)))%ig
     &           +opl(ox,abs(opl(ox,ixx)%o2(1)))%ii+1 <= mxmap) then
               imap(opl(ox,ixx)%s)=1
               mapm(opl(ox,ixx)%s)=opl(ox,ixx)%m
               mapw(opl(ox,ixx)%s)=opl(ox,ixx)%w
              end if
             end if
            end if
           else
            write(ind1,'(i16)') opl(ox,abs(opl(ox,ixx)%o2(1)))%s0
            sqsla(opl(ox,ixx)%s)='m('//trim(adjustl(ind1))//')'
            momla(opl(ox,ixx)%s)='p(:,'//trim(adjustl(ind1))//')'
            sqsd(opl(ox,ixx)%s)='m('//trim(adjustl(ind1))//')'
            momd(opl(ox,ixx)%s)='p(:,'//trim(adjustl(ind1))//')'
            if (opl(ox,ixx)%o2(2) > 0) then
             write(ind2,'(i16)') opl(ox,ixx)%o2(2)
             sqslb(opl(ox,ixx)%s)='m('//trim(adjustl(ind2))//')'
             momlb(opl(ox,ixx)%s)='p(:,'//trim(adjustl(ind2))//')'
             sqse(opl(ox,ixx)%s)='m('//trim(adjustl(ind2))//')'
             mome(opl(ox,ixx)%s)='p(:,'//trim(adjustl(ind2))//')'
             sqsl(opl(ox,ixx)%s)=trim(sqsla(opl(ox,ixx)%s))//'+'//
     &                       trim(sqslb(opl(ox,ixx)%s))
             moml(opl(ox,ixx)%s)=trim(momla(opl(ox,ixx)%s))//'+'//
     &                       trim(momlb(opl(ox,ixx)%s))
             if (expl(opl(ox,abs(opl(ox,ixx)%o2(1)))%s0) == 'n' 
     &           .and. expl(opl(ox,ixx)%o2(2)) == 'n') 
     &                              nuga(opl(ox,ixx)%s)=1  ! 16 Nov. 2010
             if (2 <= mxmap) then
              imap(opl(ox,ixx)%s)=1
              mapm(opl(ox,ixx)%s)=opl(ox,ixx)%m
              mapw(opl(ox,ixx)%s)=opl(ox,ixx)%w
             end if
            else if (opl(ox,ixx)%o2(2) < 0) then
             sqsa(opl(ox,abs(opl(ox,ixx)%o2(2)))%s)='sqs'//
     &             trim(adjustl(inds))
             if (opl(ox,abs(opl(ox,ixx)%o2(2)))%s0 == 0) then
              write(ind2,'(i16)') opl(ox,abs(opl(ox,ixx)%o2(2)))%s
              sqslb(opl(ox,ixx)%s)='sqsl'//trim(adjustl(ind2))
              momlb(opl(ox,ixx)%s)='q'//trim(adjustl(ind2))
              sqse(opl(ox,ixx)%s)='sqs'//trim(adjustl(ind2))
              mome(opl(ox,ixx)%s)='q'//trim(adjustl(ind2))
              sqsb(opl(ox,abs(opl(ox,ixx)%o2(2)))%s)='m('//
     &             trim(adjustl(ind1))//')'
              sqsl(opl(ox,ixx)%s)=trim(sqsla(opl(ox,ixx)%s))//'+'//
     &                       trim(sqslb(opl(ox,ixx)%s))
              moml(opl(ox,ixx)%s)=trim(momla(opl(ox,ixx)%s))//'+'//
     &                       trim(momlb(opl(ox,ixx)%s))
              if (1+opl(ox,abs(opl(ox,ixx)%o2(2)))%ig 
     &             +opl(ox,abs(opl(ox,ixx)%o2(2)))%ii <= mxmap) then
               imap(opl(ox,ixx)%s)=1
               mapm(opl(ox,ixx)%s)=opl(ox,ixx)%m
               mapw(opl(ox,ixx)%s)=opl(ox,ixx)%w
              end if
             else
              write(ind2,'(i16)') opl(ox,abs(opl(ox,ixx)%o2(2)))%s0
              sqslb(opl(ox,ixx)%s)='m('//trim(adjustl(ind2))//')'
              momlb(opl(ox,ixx)%s)='p(:,'//trim(adjustl(ind2))//')'
              sqse(opl(ox,ixx)%s)='m('//trim(adjustl(ind2))//')'
              mome(opl(ox,ixx)%s)='p(:,'//trim(adjustl(ind2))//')'
              sqsl(opl(ox,ixx)%s)=trim(sqsla(opl(ox,ixx)%s))//'+'//
     &                       trim(sqslb(opl(ox,ixx)%s))
              moml(opl(ox,ixx)%s)=trim(momla(opl(ox,ixx)%s))//'+'//
     &                       trim(momlb(opl(ox,ixx)%s))
              if (expl(opl(ox,abs(opl(ox,ixx)%o2(1)))%s0) == 'n' 
     &           .and. expl(opl(ox,abs(opl(ox,ixx)%o2(2)))%s0) == 'n') 
     &                              nuga(opl(ox,ixx)%s)=1  ! 16 Nov. 2010
              if (2 <= mxmap) then
               imap(opl(ox,ixx)%s)=1
               mapm(opl(ox,ixx)%s)=opl(ox,ixx)%m
               mapw(opl(ox,ixx)%s)=opl(ox,ixx)%w
              end if
             end if
            end if
           end if
          end if
         end if ! opl(ox,ixx)%i3
        end if ! opl(ox,ixx)%s

        end do

       end subroutine wrtss

      end 
